@@ -46,16 +46,14 @@ incompatibilities from 1.8.x:
 #include <locale.h>
 #endif
 
-#include <EXTERN.h>
-#include <perl.h>
-#include <XSUB.h>
-
-#include <glib.h>
-
 #undef PACKAGE
 #include "../../config.h"	/* for #define OLD_PERL */
 #include "xchat-plugin.h"
 
+#ifdef WIN32
+#define strcasecmp stricmp
+#endif
+
 struct perlscript
 {
 	char *name;
@@ -65,18 +63,54 @@ struct perlscript
 	void *gui_entry;
 };
 
+static xchat_plugin *ph; /* plugin handle */
+
+
+static int perl_load_file (char *script_name);
+
+
+
+/* leave this before XSUB.h, to avoid readdir() being redefined */
+static void
+perl_auto_load (void)
+{
+	DIR *dir;
+	struct dirent *ent;
+	const char *xdir;
+
+	xdir = xchat_get_info (ph, "xchatdir");
+	dir = opendir (xdir);
+	if (dir)
+	{
+		while ((ent = readdir (dir)))
+		{
+			int len = strlen (ent->d_name);
+			if (len > 3 && strcasecmp (".pl", ent->d_name + len - 3) == 0)
+			{
+				char *file = malloc (len + strlen (xdir) + 2);
+				sprintf (file, "%s/%s", xdir, ent->d_name);
+				perl_load_file (file);
+				free (file);
+			}
+		}
+		closedir (dir);
+	}
+}
+
+#include <EXTERN.h>
+#include <perl.h>
+#include <XSUB.h>
+#include <glib.h>
+
 static PerlInterpreter *my_perl = NULL;
 static GSList *perl_list = 0;
 static GSList *hook_list = 0;
-static xchat_plugin *ph; /* plugin handle */
-
 #ifdef OLD_PERL
 extern void boot_DynaLoader _((CV * cv));
 #else
 extern void boot_DynaLoader (pTHX_ CV* cv);
 #endif
 
-
 /* 
    execute_perl is modified in order to avoid crashing of xchat when a
    perl error occours. The embedded interpreter will instead print the
@@ -311,7 +345,7 @@ static XS (XS_IRC_add_message_handler)
 	if (strcasecmp (name, "inbound") == 0)	/* xchat 1.x.x compat */
 		name = "RAW LINE";
 
-	hook = xchat_hook_server (ph, name, PRI_NORM, perl_server_cb, tmp);
+	hook = xchat_hook_server (ph, name, XCHAT_PRI_NORM, perl_server_cb, tmp);
 
 	hook_list = g_slist_prepend (hook_list, hook);
 
@@ -329,7 +363,7 @@ static XS (XS_IRC_add_command_handler)
 
 	tmp = strdup (SvPV (ST (1), junk));
 
-	hook = xchat_hook_command (ph, SvPV (ST (0), junk), PRI_NORM,
+	hook = xchat_hook_command (ph, SvPV (ST (0), junk), XCHAT_PRI_NORM,
 										perl_command_cb, NULL, tmp);
 
 	hook_list = g_slist_prepend (hook_list, hook);
@@ -348,7 +382,7 @@ static XS (XS_IRC_add_print_handler)
 
 	tmp = strdup (SvPV (ST (1), junk));
 
-	hook = xchat_hook_print (ph, SvPV (ST (0), junk), PRI_NORM, perl_print_cb,
+	hook = xchat_hook_print (ph, SvPV (ST (0), junk), XCHAT_PRI_NORM, perl_print_cb,
 									 tmp);
 
 	hook_list = g_slist_prepend (hook_list, hook);
@@ -676,7 +710,10 @@ static XS (XS_IRC_dcc_list)
 		while (xchat_list_next (ph, list))
 		{
 			/* Make sure there is room on the stack */
-			EXTEND (SP, i + 10);
+			EXTEND (SP, i + 11);
+
+			XST_mPV (i, xchat_list_str (ph, list, "nick"));
+			i++;
 
 			file = xchat_list_str (ph, list, "file");
 			if (!file)
@@ -919,6 +956,7 @@ perl_init (void)
 #endif
 
 	my_perl = perl_alloc ();
+	PL_perl_destruct_level = 1;
 	perl_construct (my_perl);
 
 	warn = 0;
@@ -953,32 +991,6 @@ perl_load_file (char *script_name)
 	return execute_perl ("load_n_eval", script_name);
 }
 
-static void
-perl_auto_load (void)
-{
-	DIR *dir;
-	struct dirent *ent;
-	const char *xdir;
-
-	xdir = xchat_get_info (ph, "xchatdir");
-	dir = opendir (xdir);
-	if (dir)
-	{
-		while ((ent = readdir (dir)))
-		{
-			int len = strlen (ent->d_name);
-			if (len > 3 && strcasecmp (".pl", ent->d_name + len - 3) == 0)
-			{
-				char *file = malloc (len + strlen (xdir) + 2);
-				sprintf (file, "%s/%s", xdir, ent->d_name);
-				perl_load_file (file);
-				free (file);
-			}
-		}
-		closedir (dir);
-	}
-}
-
 /* checks for "~" in a file and expands */
 
 static char *
@@ -1037,7 +1049,7 @@ perl_command_unloadall (char *word[], char *word_eol[], void *userdata)
 {
 	perl_end ();
 
-	return EAT_XCHAT;
+	return XCHAT_EAT_XCHAT;
 }
 
 static int
@@ -1052,11 +1064,11 @@ perl_command_unload (char *word[], char *word_eol[], void *userdata)
 
 		/* if only unloading was possible with this shitty interface :( */
 
-		return EAT_XCHAT;
+		return XCHAT_EAT_XCHAT;
 	}
 #endif
 
-	return EAT_NONE;
+	return XCHAT_EAT_NONE;
 }
 
 static int
@@ -1078,10 +1090,10 @@ perl_command_load (char *word[], char *word_eol[], void *userdata)
 			xchat_print (ph, "Error Loading file\n");
 		}
 		free (file);
-		return EAT_XCHAT;
+		return XCHAT_EAT_XCHAT;
 	}
 
-	return EAT_NONE;
+	return XCHAT_EAT_NONE;
 }
 
 int
@@ -1095,11 +1107,9 @@ xchat_plugin_init (xchat_plugin *plugin_handle,
 	*plugin_version = VERSION;
 	*plugin_desc = "Perl scripting interface";
 
-	PL_perl_destruct_level = 1;
-
-	xchat_hook_command (ph, "load", PRI_NORM, perl_command_load, 0, 0);
-	xchat_hook_command (ph, "unload", PRI_NORM, perl_command_unload, 0, 0);
-	xchat_hook_command (ph, "unloadall", PRI_NORM, perl_command_unloadall, 0, 0);
+	xchat_hook_command (ph, "load", XCHAT_PRI_NORM, perl_command_load, 0, 0);
+	xchat_hook_command (ph, "unload", XCHAT_PRI_NORM, perl_command_unload, 0, 0);
+	xchat_hook_command (ph, "unloadall", XCHAT_PRI_NORM, perl_command_unloadall, 0, 0);
 
 	perl_auto_load ();
 
