@@ -359,7 +359,7 @@ Callback_Command(char *word[], char *word_eol[], void *userdata)
 	retobj = PyObject_CallFunction(hook->callback, "(OOO)", word_list,
 				       word_eol_list, hook->userdata);
 	if (retobj == Py_None) {
-		ret = EAT_NONE;
+		ret = XCHAT_EAT_NONE;
 		Py_DECREF(retobj);
 	} else if (retobj) {
 		ret = PyInt_AsLong(retobj);
@@ -440,7 +440,7 @@ Callback_Print(char *word[], void *userdata)
 	g_free(word_eol_raw);
 	g_free(word_eol);
 	if (retobj == Py_None) {
-		ret = EAT_NONE;
+		ret = XCHAT_EAT_NONE;
 		Py_DECREF(retobj);
 	} else if (retobj) {
 		ret = PyInt_AsLong(retobj);
@@ -945,15 +945,15 @@ Plugin_New(char *filename, PyMethodDef *xchat_methods, PyObject *xcoobj)
 		goto error;
 	}
 
-	PyModule_AddIntConstant(m, "EAT_NONE", EAT_NONE);
-	PyModule_AddIntConstant(m, "EAT_XCHAT", EAT_XCHAT);
-	PyModule_AddIntConstant(m, "EAT_PLUGIN", EAT_PLUGIN);
-	PyModule_AddIntConstant(m, "EAT_ALL", EAT_ALL);
-	PyModule_AddIntConstant(m, "PRI_HIGHEST", PRI_HIGHEST);
-	PyModule_AddIntConstant(m, "PRI_HIGH", PRI_HIGH);
-	PyModule_AddIntConstant(m, "PRI_NORM", PRI_NORM);
-	PyModule_AddIntConstant(m, "PRI_LOW", PRI_LOW);
-	PyModule_AddIntConstant(m, "PRI_LOWEST", PRI_LOWEST);
+	PyModule_AddIntConstant(m, "EAT_NONE", XCHAT_EAT_NONE);
+	PyModule_AddIntConstant(m, "EAT_XCHAT", XCHAT_EAT_XCHAT);
+	PyModule_AddIntConstant(m, "EAT_PLUGIN", XCHAT_EAT_PLUGIN);
+	PyModule_AddIntConstant(m, "EAT_ALL", XCHAT_EAT_ALL);
+	PyModule_AddIntConstant(m, "PRI_HIGHEST", XCHAT_PRI_HIGHEST);
+	PyModule_AddIntConstant(m, "PRI_HIGH", XCHAT_PRI_HIGH);
+	PyModule_AddIntConstant(m, "PRI_NORM", XCHAT_PRI_NORM);
+	PyModule_AddIntConstant(m, "PRI_LOW", XCHAT_PRI_LOW);
+	PyModule_AddIntConstant(m, "PRI_LOWEST", XCHAT_PRI_LOWEST);
 
 	o = Py_BuildValue("(ii)", VERSION_MAJOR, VERSION_MINOR);
 	if (o == NULL) {
@@ -1268,7 +1268,7 @@ Module_xchat_hook_command(PyObject *self, PyObject *args, PyObject *kwargs)
 	char *name;
 	PyObject *callback;
 	PyObject *userdata = Py_None;
-	int priority = PRI_NORM;
+	int priority = XCHAT_PRI_NORM;
 	char *help = NULL;
 	PyObject *plugin;
 	Hook *hook;
@@ -1306,7 +1306,7 @@ Module_xchat_hook_server(PyObject *self, PyObject *args, PyObject *kwargs)
 	char *name;
 	PyObject *callback;
 	PyObject *userdata = Py_None;
-	int priority = PRI_NORM;
+	int priority = XCHAT_PRI_NORM;
 	PyObject *plugin;
 	Hook *hook;
 	char *kwlist[] = {"name", "callback", "userdata", "priority", 0};
@@ -1342,7 +1342,7 @@ Module_xchat_hook_print(PyObject *self, PyObject *args, PyObject *kwargs)
 	char *name;
 	PyObject *callback;
 	PyObject *userdata = Py_None;
-	int priority = PRI_NORM;
+	int priority = XCHAT_PRI_NORM;
 	PyObject *plugin;
 	Hook *hook;
 	char *kwlist[] = {"name", "callback", "userdata", "priority", 0};
@@ -1735,7 +1735,7 @@ Command_Py(char *word[], char *word_eol[], void *userdata)
 	}
 	if (!ok)
 		xchat_print(ph, usage);
-	return EAT_ALL;
+	return XCHAT_EAT_ALL;
 }
 
 static int
@@ -1744,9 +1744,9 @@ Command_Load(char *word[], char *word_eol[], void *userdata)
 	int len = strlen(word[2]);
 	if (len > 3 && strcasecmp(".py", word[2]+len-3) == 0) {
 		Command_PyLoad(word[2]);
-		return EAT_XCHAT;
+		return XCHAT_EAT_XCHAT;
 	}
-	return EAT_NONE;
+	return XCHAT_EAT_NONE;
 }
 
 static int
@@ -1755,9 +1755,9 @@ Command_Unload(char *word[], char *word_eol[], void *userdata)
 	int len = strlen(word[2]);
 	if (len > 3 && strcasecmp(".py", word[2]+len-3) == 0) {
 		Command_PyUnload(word[2]);
-		return EAT_XCHAT;
+		return XCHAT_EAT_XCHAT;
 	}
-	return EAT_NONE;
+	return XCHAT_EAT_NONE;
 }
 
 /* ===================================================================== */
@@ -1830,10 +1830,10 @@ xchat_plugin_init(xchat_plugin *plugin_handle,
 	}
 
 
-	xchat_hook_command(ph, "", PRI_NORM, IInterp_Cmd, 0, 0);
-	xchat_hook_command(ph, "PY", PRI_NORM, Command_Py, usage, 0);
-	xchat_hook_command(ph, "LOAD", PRI_NORM, Command_Load, 0, 0);
-	xchat_hook_command(ph, "UNLOAD", PRI_NORM, Command_Unload, 0, 0);
+	xchat_hook_command(ph, "", XCHAT_PRI_NORM, IInterp_Cmd, 0, 0);
+	xchat_hook_command(ph, "PY", XCHAT_PRI_NORM, Command_Py, usage, 0);
+	xchat_hook_command(ph, "LOAD", XCHAT_PRI_NORM, Command_Load, 0, 0);
+	xchat_hook_command(ph, "UNLOAD", XCHAT_PRI_NORM, Command_Unload, 0, 0);
 #ifdef WITH_THREAD
 	thread_timer = xchat_hook_timer(ph, 300, Callback_ThreadTimer, NULL);
 #endif
