@@ -33,14 +33,11 @@
 #include "fe.h"
 #include "text.h"
 #include "xchatc.h"
-#ifdef USE_JCODE
-#include <locale.h>
-#endif
 
-#if defined (USE_XFT) || defined (USE_PANGO)
-#define DEF_FONT "Luxi Mono 12"
+#ifdef WIN32
+#define DEF_FONT "monospace 10"
 #else
-#define DEF_FONT "-b&h-lucidatypewriter-medium-r-normal-*-*-120-*-*-m-*-*-*"
+#define DEF_FONT "monospace 11"
 #endif
 
 void
@@ -253,11 +250,11 @@ get_xdir (void)
 	}
 	return xdir;
 #else
-	return "./conf";
+	return "./config";
 #endif
 }
 
-void
+static void
 check_prefs_dir (void)
 {
 	char *xdir = get_xdir ();
@@ -268,11 +265,11 @@ check_prefs_dir (void)
 #else
 		if (mkdir (xdir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
 #endif
-			fe_message (_("Cannot create ~/.xchat"), FALSE);
+			fe_message (_("Cannot create ~/.xchat2"), FALSE);
 	}
 }
 
-char *
+static char *
 default_file (void)
 {
 	static char *dfile = 0;
@@ -305,9 +302,14 @@ const struct prefs vars[] = {
 	{"dcc_auto_send", P_OFFINT (autodccsend), TYPE_BOOL},
 	{"dcc_blocksize", P_OFFINT (dcc_blocksize), TYPE_INT},
 	{"dcc_dir", P_OFFSET (dccdir), TYPE_STR},
+	{"dcc_completed_dir", P_OFFSET (dcc_completed_dir), TYPE_STR},
 	{"dcc_fast_send", P_OFFINT (fastdccsend), TYPE_BOOL},
 	{"dcc_ip", P_OFFSET (dcc_ip_str), TYPE_STR},
 	{"dcc_ip_from_server", P_OFFINT (ip_from_server), TYPE_BOOL},
+	{"dcc_global_max_get_cps", P_OFFINT (dcc_global_max_get_cps), TYPE_INT},
+	{"dcc_global_max_send_cps", P_OFFINT (dcc_global_max_send_cps), TYPE_INT},
+	{"dcc_max_get_cps", P_OFFINT (dcc_max_get_cps), TYPE_INT},
+	{"dcc_max_send_cps", P_OFFINT (dcc_max_send_cps), TYPE_INT},
 	{"dcc_permissions", P_OFFINT (dccpermissions), TYPE_INT},
 	{"dcc_port_first", P_OFFINT (first_dcc_send_port), TYPE_INT},
 	{"dcc_port_last", P_OFFINT (last_dcc_send_port), TYPE_INT},
@@ -336,9 +338,9 @@ const struct prefs vars[] = {
 	{"gui_input_style", P_OFFINT (style_inputbox), TYPE_BOOL},
 	{"gui_lagometer", P_OFFINT (lagometer), TYPE_INT},
 	{"gui_slist_edit", P_OFFINT (slist_edit), TYPE_BOOL},
+	{"gui_slist_select", P_OFFINT (slist_select), TYPE_INT},
 	{"gui_slist_skip", P_OFFINT (slist_skip), TYPE_BOOL},
 	{"gui_throttlemeter", P_OFFINT (throttlemeter), TYPE_INT},
-	{"gui_treeview", P_OFFINT (treeview), TYPE_BOOL},
 	{"gui_topicbar", P_OFFINT (topicbar), TYPE_BOOL},
 	{"gui_ulist_buttons", P_OFFINT (userlistbuttons), TYPE_BOOL},
 	{"gui_ulist_doubleclick", P_OFFSET (doubleclickuser), TYPE_STR},
@@ -401,10 +403,6 @@ const struct prefs vars[] = {
 	{"notify_timeout", P_OFFINT (notify_timeout), TYPE_INT},
 	{"notify_whois_online", P_OFFINT (whois_on_notifyonline), TYPE_BOOL},
 
-#ifdef USE_PANEL
-	{"panel_vertical", P_OFFINT (panel_vbox), TYPE_BOOL},
-	{"panelize_hide", P_OFFINT (panelize_hide), TYPE_BOOL},
-#endif
 	{"perl_warnings", P_OFFINT (perlwarnings), TYPE_BOOL},
 
 	{"sound_command", P_OFFSET (soundcmd), TYPE_STR},
@@ -440,11 +438,6 @@ const struct prefs vars[] = {
 	{"text_stripcolor", P_OFFINT (stripcolor), TYPE_BOOL},
 	{"text_wordwrap", P_OFFINT (wordwrap), TYPE_BOOL},
 
-#ifdef USE_TRANS
-	{"trans", P_OFFINT (use_trans), TYPE_BOOL},
-	{"trans_file", P_OFFSET (trans_file), TYPE_STR},
-#endif
-
 	{0, 0, 0},
 };
 
@@ -455,9 +448,6 @@ load_config (void)
 	char *cfg;
 	const char *username;
 	int res, val, i, fh;
-#ifdef	USE_JCODE
-	gchar *locale;
-#endif
 
 	check_prefs_dir ();
 	username = g_get_user_name ();
@@ -467,12 +457,14 @@ load_config (void)
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 
 	/* put in default values, anything left out is automatically zero */
+	prefs.timestamp_logs = 1;
+	prefs.truncchans = 20;
 	prefs.autoresume = 1;
 	prefs.show_away_once = 1;
 	prefs.show_away_message = 1;
 	prefs.indent_nicks = 1;
 	prefs.thin_separator = 1;
-	prefs.tabs_position = 1;
+	/*prefs.tabs_position = 1;*/ /* 0 = bottom */
 	prefs.fastdccsend = 1;
 	prefs.wordwrap = 1;
 	prefs.autosave = 1;
@@ -486,6 +478,7 @@ load_config (void)
 	prefs.privmsgtab = 1;
 	prefs.nickcompletion = 1;
 	prefs.style_inputbox = 1;
+	prefs.slist_select = 10;
 	prefs.nu_color = 4;
 	prefs.dccpermissions = 0600;
 	prefs.max_lines = 300;
@@ -517,13 +510,11 @@ load_config (void)
 	prefs.userlistbuttons = 1;
 	prefs.persist_chans = 1;
 	prefs.perc_color = 1;
-	prefs.dialog_width = 300;
-	prefs.dialog_height = 100;
 	prefs.dcc_send_fillspaces = 1;
 	prefs.mainwindow_save = 1;
 	strcpy (prefs.stamp_format, "[%H:%M] ");
 	strcpy (prefs.timestamp_log_format, "%b %d %H:%M:%S ");
-	strcpy (prefs.logmask, "%s,%c.xchatlog");
+	strcpy (prefs.logmask, "%n-%c.log");
 	strcpy (prefs.nick_suffix, ":");
 	strcpy (prefs.cmdchar, "/");
 	strcpy (prefs.nick1, username);
@@ -535,26 +526,21 @@ load_config (void)
 	strcpy (prefs.username, username);
 #ifdef WIN32
 	strcpy (prefs.sounddir, "./sound");
-	strcpy (prefs.dccdir, "./dcc");
+	strcpy (prefs.dccdir, "./download");
+	/*strcpy (prefs.dcc_completed_dir, "./dcc");*/
 #else
 	sprintf (prefs.sounddir, "%s/sound", g_get_home_dir ());
 	sprintf (prefs.dccdir, "%s/dcc", g_get_home_dir ());
+	/*strcpy (prefs.dcc_completed_dir, prefs.dccdir);*/
 #endif
 	strcpy (prefs.doubleclickuser, "QUOTE WHOIS %s");
 	strcpy (prefs.awayreason, _("I'm busy"));
-	strcpy (prefs.quitreason, "I like core dumps");
+	strcpy (prefs.quitreason, _("Client exiting"));
 	strcpy (prefs.partreason, prefs.quitreason);
 	strcpy (prefs.font_normal, DEF_FONT);
-	strcpy (prefs.dialog_font_normal, prefs.font_normal);
 	strcpy (prefs.soundcmd, "esdplay");
 	strcpy (prefs.dnsprogram, "host");
 
-#ifdef	USE_JCODE
-	locale = setlocale (LC_CTYPE, "");
-	if (locale != NULL && !g_strncasecmp (locale, "ja", 2))
-		prefs.kanji_conv = 1;
-#endif
-
 	fh = open (default_file (), OFLAGS | O_RDONLY);
 	if (fh != -1)
 	{
@@ -599,8 +585,10 @@ load_config (void)
 
 #ifdef WIN32
 		mkdir (prefs.dccdir);
+		mkdir (prefs.dcc_completed_dir);
 #else
 		mkdir (prefs.dccdir, S_IRUSR | S_IWUSR | S_IXUSR);
+		mkdir (prefs.dcc_completed_dir, S_IRUSR | S_IWUSR | S_IXUSR);
 #endif
 
 	}
