@@ -59,22 +59,125 @@ struct dccstat_info dccstat[] = {
 	{N_("Aborted"), 4 /*red */ },
 };
 
+static int dcc_global_throttle;	/* 0x1 = sends, 0x2 = gets */
+static int dcc_sendcpssum, dcc_getcpssum;
+
 static struct DCC *new_dcc (void);
 static void dcc_close (struct DCC *dcc, int dccstat, int destroy);
+static gboolean dcc_send_data (GIOChannel *, GIOCondition, struct DCC *);
+static gboolean dcc_read (GIOChannel *, GIOCondition, struct DCC *);
 
+static double
+timeval_diff (GTimeVal *greater,
+				 GTimeVal *less)
+{
+	long usecdiff;
+	double result;
+	
+	result = greater->tv_sec - less->tv_sec;
+	usecdiff = (long) greater->tv_usec - less->tv_usec;
+	result += (double) usecdiff / 1000000;
+	
+	return result;
+}
+
+static void
+dcc_unthrottle (struct DCC *dcc)
+{
+	/* don't unthrottle here, but delegate to funcs */
+	if (dcc->type == TYPE_RECV)
+		dcc_read (NULL, 0, dcc);
+	else
+		dcc_send_data (NULL, 0, dcc);
+}
 
 static void
 dcc_calc_cps (struct DCC *dcc)
 {
-	time_t sec;
+	GTimeVal now;
+	int pos, posdiff, oldcps;
+	double timediff, startdiff;
+	int glob_throttle_bit, wasthrottled;
+	int *cpssum, glob_limit;
 
-	sec = time (0) - dcc->starttime;
-	if (sec < 1)
-		sec = 1;
+	g_get_current_time (&now);
+
+	/* the pos we use for sends is an average
+		between pos and ack */
 	if (dcc->type == TYPE_SEND)
-		dcc->cps = (dcc->ack - dcc->resumable) / sec;
+	{
+		pos = (dcc->pos + dcc->ack) / 2;
+		glob_throttle_bit = 0x1;
+		cpssum = &dcc_sendcpssum;
+		glob_limit = prefs.dcc_global_max_send_cps;
+	}
 	else
-		dcc->cps = (dcc->pos - dcc->resumable) / sec;
+	{
+		pos = dcc->pos;
+		glob_throttle_bit = 0x2;
+		cpssum = &dcc_getcpssum;
+		glob_limit = prefs.dcc_global_max_get_cps;
+	}
+
+	if (!dcc->firstcpstv.tv_sec)
+		dcc->firstcpstv = now;
+	else
+	{
+		startdiff = timeval_diff (&now, &dcc->firstcpstv);
+		if (startdiff < 1)
+			startdiff = 1;
+		else if (startdiff > CPS_AVG_WINDOW)
+			startdiff = CPS_AVG_WINDOW;
+
+		timediff = timeval_diff (&now, &dcc->lastcpstv);
+		if (timediff > startdiff)
+			timediff = startdiff = 1;
+
+		posdiff = pos - dcc->lastcpspos;
+		oldcps = dcc->cps;
+		dcc->cps = ((double) posdiff / timediff) * (timediff / startdiff) +
+			(double) dcc->cps * (1.0 - (timediff / startdiff));
+
+		*cpssum += dcc->cps - oldcps;
+	}
+
+	dcc->lastcpspos = pos;
+	dcc->lastcpstv = now;
+
+	/* now check cps against set limits... */
+	wasthrottled = dcc->throttled;
+
+	/* check global limits first */
+	dcc->throttled &= ~0x2;
+	if (glob_limit > 0 && *cpssum >= glob_limit)
+	{
+		dcc_global_throttle |= glob_throttle_bit;
+		if (dcc->maxcps >= 0)
+			dcc->throttled |= 0x2;
+	}
+	else
+		dcc_global_throttle &= ~glob_throttle_bit;
+
+	/* now check per-connection limit */
+	if (dcc->maxcps > 0 && dcc->cps > dcc->maxcps)
+		dcc->throttled |= 0x1;
+	else
+		dcc->throttled &= ~0x1;
+
+	/* take action */
+	if (wasthrottled && !dcc->throttled)
+		dcc_unthrottle (dcc);
+}
+
+static void
+dcc_remove_from_sum (struct DCC *dcc)
+{
+	if (dcc->dccstat != STAT_ACTIVE)
+		return;
+	if (dcc->type == TYPE_SEND)
+		dcc_sendcpssum -= dcc->cps;
+	else if (dcc->type == TYPE_RECV)
+		dcc_getcpssum -= dcc->cps;
 }
 
 /* this is called from xchat.c:xchat_misc_checks() every 2 seconds. */
@@ -109,7 +212,8 @@ dcc_check_timeouts (void)
 			{
 				if (prefs.dccstalltimeout > 0)
 				{
-					if (tim - dcc->lasttime > prefs.dccstalltimeout)
+					if (!dcc->throttled
+						&& tim - dcc->lasttime > prefs.dccstalltimeout)
 					{
 						EMIT_SIGNAL (XP_TE_DCCSTALL, dcc->serv->front_session,
 										 dcctypes[(int) dcc->type],
@@ -207,10 +311,24 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 		dcc->sok = -1;
 	}
 
+	dcc_remove_from_sum (dcc);
+
 	if (dcc->fp != -1)
 	{
 		close (dcc->fp);
 		dcc->fp = -1;
+
+		if(dccstat == STAT_DONE)
+		{
+			/* if we just completed a dcc recieve, move the */
+			/* completed file to the completed directory */
+			if(dcc->type == TYPE_RECV)
+			{			
+				download_move_to_completed_dir(prefs.dccdir, prefs.dcc_completed_dir, 
+					dcc->file, prefs.dccpermissions);
+			}
+
+		}
 	}
 
 	dcc->dccstat = dccstat;
@@ -226,7 +344,9 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 		if (dcc->file)
 			free (dcc->file);
 		if (dcc->destfile)
-			free (dcc->destfile);
+			g_free (dcc->destfile);
+		if (dcc->destfile_fs)
+			g_free (dcc->destfile_fs);
 		free (dcc->nick);
 		free (dcc);
 		update_dcc_window (type);
@@ -299,48 +419,41 @@ dcc_notify_kill (struct server *serv)
 struct DCC *
 dcc_write_chat (char *nick, char *text)
 {
-#ifdef USE_TRANS
-	unsigned char *tbuf;
-#define TRANS_STAT_BUF_LEN 1024
-	static unsigned char sbuf[TRANS_STAT_BUF_LEN];
-#endif
 	struct DCC *dcc;
-	int len;
 
 	dcc = find_dcc (nick, "", TYPE_CHATRECV);
 	if (!dcc)
 		dcc = find_dcc (nick, "", TYPE_CHATSEND);
 	if (dcc && dcc->dccstat == STAT_ACTIVE)
 	{
+		char *locale;
+		int loc_len;
+		int len;
+
 		len = strlen (text);
-		dcc->size += len;
-#ifdef USE_TRANS
-		if (prefs.use_trans)
+
+		if (dcc->serv->encoding == NULL)	/* system */
 		{
-			if (len >= TRANS_STAT_BUF_LEN)
-				tbuf = malloc (len + 1);
-			else
-				tbuf = sbuf;
-			if (!tbuf)
-			{
-				return 0;
-			}
-			memcpy (tbuf, text, len + 1);	/* copy text with the NULL */
-			user2serv (tbuf);
-			send (dcc->sok, tbuf, len, 0);
-			if (tbuf != sbuf)
-			{
-				free (tbuf);
-			}
+			locale = NULL;
+			if (!prefs.utf8_locale)
+				locale = g_locale_from_utf8 (text, len, NULL, &loc_len, NULL);
 		} else
 		{
-#endif
-			send (dcc->sok, text, len, 0);
-#ifdef USE_TRANS
+			locale = g_convert (text, len, dcc->serv->encoding, "UTF-8", 0, &loc_len, 0);
 		}
-#endif
+
+		if (locale)
+		{
+			text = locale;
+			len = loc_len;
+		}
+
+		dcc->size += len;
+		send (dcc->sok, text, len, 0);
 		send (dcc->sok, "\n", 1, 0);
 		fe_dcc_update_chat_win ();
+		if (locale)
+			g_free (locale);
 		return dcc;
 	}
 	return 0;
@@ -355,7 +468,26 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	session *sess;
 	char *word[PDIWORDS];
 	char *po;
+	char *utf;
+	char *conv;
 	int ret, i;
+	int len, utf_len;
+
+	len = strlen (line);
+
+	if (dcc->serv->encoding == NULL)     /* system */
+		utf = g_locale_to_utf8 (line, len, NULL, &utf_len, NULL);
+	else
+		utf = g_convert (line, len, "UTF-8", dcc->serv->encoding, 0, &utf_len, 0);
+
+	if (utf)
+	{
+		line = utf;
+		len = utf_len;
+	}
+
+	/* we really need valid UTF-8 now */
+	conv = text_validate (&line, &len);
 
 	sess = find_dialog (dcc->serv, dcc->nick);
 	if (!sess)
@@ -371,15 +503,27 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	for (i = 5; i < PDIWORDS; i++)
 		word[i] = "\000";
 
-	ret = plugin_emit_server (sess, word[0], word, word);
+	ret = plugin_emit_print (sess, word);
 
 	/* did the plugin close it? */
 	if (!g_slist_find (dcc_list, dcc))
+	{
+		if (utf)
+			g_free (utf);
+		if (conv)
+			g_free (conv);
 		return 1;
+	}
 
 	/* did the plugin eat the event? */
 	if (ret)
+	{
+		if (utf)
+			g_free (utf);
+		if (conv)
+			g_free (conv);
 		return 0;
+	}
 
 	url_check (line);
 
@@ -391,12 +535,12 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		inbound_action (sess, tbuf, dcc->serv->nick, dcc->nick, line + 8, FALSE);
 	} else
 	{
-#ifdef USE_TRANS
-		if (prefs.use_trans)
-			serv2user (line);
-#endif
 		inbound_privmsg (dcc->serv, tbuf, dcc->nick, "", line);
 	}
+	if (utf)
+		g_free (utf);
+	if (conv)
+		g_free (conv);
 	return 0;
 }
 
@@ -410,6 +554,16 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 
 	while (1)
 	{
+		if (dcc->throttled)
+		{
+			fe_input_remove (dcc->iotag);
+			dcc->iotag = 0;
+			return FALSE;
+		}
+
+		if (!dcc->iotag)
+			dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read, dcc);
+
 		len = recv (dcc->sok, lbuf, sizeof (lbuf) - 2, 0);
 		if (len < 1)
 		{
@@ -462,9 +616,24 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	}
 }
 
+static void
+dcc_calc_average_cps (struct DCC *dcc)
+{
+	time_t sec;
+
+	sec = time (0) - dcc->starttime;
+	if (sec < 1)
+		sec = 1;
+	if (dcc->type == TYPE_SEND)
+		dcc->cps = (dcc->ack - dcc->resumable) / sec;
+	else
+		dcc->cps = (dcc->pos - dcc->resumable) / sec;
+}
+
 static gboolean
 dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
+	char *old;
 	char buf[4096];
 	guint32 pos;
 	int n;
@@ -473,27 +642,33 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 		if (dcc->resumable)
 		{
-			dcc->fp = open (dcc->destfile, O_WRONLY | O_APPEND | OFLAGS);
+			dcc->fp = open (dcc->destfile_fs, O_WRONLY | O_APPEND | OFLAGS);
 			dcc->pos = dcc->resumable;
 			dcc->ack = dcc->resumable;
 		} else
 		{
-			if (access (dcc->destfile, F_OK) == 0)
+			if (access (dcc->destfile_fs, F_OK) == 0)
 			{
 				n = 0;
 				do
 				{
 					n++;
-					sprintf (buf, "%s.%d", dcc->destfile, n);
+					sprintf (buf, "%s.%d", dcc->destfile_fs, n);
 				}
 				while (access (buf, F_OK) == 0);
+
+				g_free (dcc->destfile_fs);
+				dcc->destfile_fs = g_strdup (buf);
+
+				old = dcc->destfile;
+				dcc->destfile = g_filename_to_utf8 (buf, -1, 0, 0, 0);
+
 				EMIT_SIGNAL (XP_TE_DCCRENAME, dcc->serv->front_session,
-								 dcc->destfile, buf, NULL, NULL, 0);
-				free (dcc->destfile);
-				dcc->destfile = strdup (buf);
+								 old, dcc->destfile, NULL, NULL, 0);
+				g_free (old);
 			}
 			dcc->fp =
-				open (dcc->destfile, OFLAGS | O_TRUNC | O_WRONLY | O_CREAT,
+				open (dcc->destfile_fs, OFLAGS | O_TRUNC | O_WRONLY | O_CREAT,
 						prefs.dccpermissions);
 		}
 	}
@@ -506,6 +681,16 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	}
 	while (1)
 	{
+		if (dcc->throttled)
+		{
+			fe_input_remove (dcc->iotag);
+			dcc->iotag = 0;
+			return FALSE;
+		}
+
+		if (!dcc->iotag)
+			dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read, dcc);
+
 		n = recv (dcc->sok, buf, sizeof (buf), 0);
 		if (n < 1)
 		{
@@ -529,23 +714,24 @@ failedabort:
 
 		dcc->lasttime = time (0);
 
-		dcc_calc_cps (dcc);
-
 		if (dcc->pos >= dcc->size)
 		{
+			dcc_calc_average_cps (dcc);
 			sprintf (buf, "%d", dcc->cps);
 			dcc_close (dcc, STAT_DONE, FALSE);
 			EMIT_SIGNAL (XP_TE_DCCRECVCOMP, dcc->serv->front_session,
 							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
 			return TRUE;
 		}
+
+		dcc_calc_cps (dcc);
 	}
 }
 
 static gboolean
 dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
-	int er, sok = dcc->sok;
+	int er;
 	char host[128];
 	struct sockaddr_in addr;
 
@@ -561,7 +747,7 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	addr.sin_addr.s_addr = htonl (dcc->addr);
 
 	/* check if it's already connected */
-	if (connect (sok, (struct sockaddr *) &addr, sizeof (addr)) != 0)
+	if (connect (dcc->sok, (struct sockaddr *) &addr, sizeof (addr)) != 0)
 	{
 		er = sock_error ();
 #ifndef WIN32
@@ -585,13 +771,13 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	switch (dcc->type)
 	{
 	case TYPE_RECV:
-		dcc->iotag = fe_input_add (dcc->sok, 1, 0, 1, dcc_read, dcc);
+		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read, dcc);
 		EMIT_SIGNAL (XP_TE_DCCCONRECV, dcc->serv->front_session,
 						 dcc->nick, host, dcc->file, NULL, 0);
 		break;
 
 	case TYPE_CHATRECV:
-		dcc->iotag = fe_input_add (dcc->sok, 1, 0, 1, dcc_read_chat, dcc);
+		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
 		EMIT_SIGNAL (XP_TE_DCCCONCHAT, dcc->serv->front_session,
@@ -647,7 +833,7 @@ dcc_connect (struct DCC *dcc)
 			update_dcc_window (dcc->type);
 			return;
 		}
-		dcc->iotag = fe_input_add (dcc->sok, 0, 1, 1, dcc_connect_finished, dcc);
+		dcc->iotag = fe_input_add (dcc->sok, FIA_WRITE|FIA_EX, dcc_connect_finished, dcc);
 	}
 	
 	if (dcc->type == TYPE_RECV)
@@ -668,6 +854,21 @@ dcc_send_data (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	if (prefs.dcc_blocksize > 102400)	/* this is too much! */
 		prefs.dcc_blocksize = 102400;
 
+	if (dcc->throttled)
+	{
+		fe_input_remove (dcc->wiotag);
+		dcc->wiotag = 0;
+		return FALSE;
+	}
+
+	if (!dcc->fastsend)
+	{
+		if (dcc->ack < dcc->pos)
+			return TRUE;
+	}
+	else if (!dcc->wiotag)
+		dcc->wiotag = fe_input_add (sok, FIA_WRITE, dcc_send_data, dcc);
+
 	buf = malloc (prefs.dcc_blocksize);
 	if (!buf)
 		return TRUE;
@@ -751,6 +952,7 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 
 	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
 	{
+		dcc_calc_average_cps (dcc);
 		dcc_close (dcc, STAT_DONE, FALSE);
 		sprintf (buf, "%d", dcc->cps);
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
@@ -796,15 +998,22 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 	case TYPE_SEND:
 		if (dcc->fastsend)
-			dcc->wiotag = fe_input_add (sok, 0, 1, 0, dcc_send_data, dcc);
-		dcc->iotag = fe_input_add (sok, 1, 0, 1, dcc_read_ack, dcc);
+			dcc->wiotag = fe_input_add (sok, FIA_WRITE, dcc_send_data, dcc);
+		dcc->iotag = fe_input_add (sok, FIA_READ|FIA_EX, dcc_read_ack, dcc);
 		dcc_send_data (NULL, 0, (gpointer)dcc);
 		EMIT_SIGNAL (XP_TE_DCCCONSEND, dcc->serv->front_session,
 						 dcc->nick, host, dcc->file, NULL, 0);
 		break;
 
 	case TYPE_CHATSEND:
-		dcc->iotag = fe_input_add (dcc->sok, 1, 0, 1, dcc_read_chat, dcc);
+		if (prefs.autodialog)
+		{
+			char *cmd = malloc (8 + strlen (dcc->nick));
+			sprintf (cmd, "query %s", dcc->nick);
+			handle_command (dcc->serv->server_session, cmd, FALSE);
+			free (cmd);
+		}
+		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
 		EMIT_SIGNAL (XP_TE_DCCCONCHAT, dcc->serv->front_session,
@@ -896,7 +1105,7 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	listen (dcc->sok, 1);
 	set_blocking (dcc->sok);
 
-	dcc->iotag = fe_input_add (dcc->sok, 1, 0, 1, dcc_accept, dcc);
+	dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_accept, dcc);
 
 	return TRUE;
 }
@@ -904,23 +1113,26 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 static struct session *dccsess;
 static char *dccto;				  /* lame!! */
 static char *dcctbuf;
+static int dccmaxcps;
 static int recursive = FALSE;
 
 static void
 dcc_send_wild (char *file)
 {
-	dcc_send (dccsess, dcctbuf, dccto, file);
+	dcc_send (dccsess, dcctbuf, dccto, file, dccmaxcps);
 }
 
 /* tbuf is at least 400 bytes */
 
 void
-dcc_send (session *sess, char *tbuf, char *to, char *file)
+dcc_send (struct session *sess, char *tbuf, char *to, char *file, int maxcps)
 {
 	char outbuf[512];
 	struct stat st;
 	struct DCC *dcc;
+	char *file_fs;
 
+	/* this is utf8 */
 	file = expand_homedir (file);
 
 	if (!recursive && (strchr (file, '*') || strchr (file, '?')))
@@ -936,6 +1148,7 @@ dcc_send (session *sess, char *tbuf, char *to, char *file)
 		dccsess = sess;
 		dccto = to;
 		dcctbuf = tbuf;
+		dccmaxcps = maxcps;
 
 		free (file);
 
@@ -950,10 +1163,14 @@ dcc_send (session *sess, char *tbuf, char *to, char *file)
 	if (!dcc)
 		return;
 	dcc->file = file;
+	dcc->maxcps = maxcps;
 
-	if (stat (file, &st) != -1)
+	/* get the local filesystem encoding */
+	file_fs = g_filename_from_utf8 (file, -1, 0, 0, 0);
+
+	if (stat (file_fs, &st) != -1)
 	{
-		if (*file_part (file) && !S_ISDIR (st.st_mode))
+		if (*file_part (file_fs) && !S_ISDIR (st.st_mode))
 		{
 			if (st.st_size > 0)
 			{
@@ -962,9 +1179,10 @@ dcc_send (session *sess, char *tbuf, char *to, char *file)
 				dcc->dccstat = STAT_QUEUED;
 				dcc->size = st.st_size;
 				dcc->type = TYPE_SEND;
-				dcc->fp = open (file, OFLAGS | O_RDONLY);
+				dcc->fp = open (file_fs, OFLAGS | O_RDONLY);
 				if (dcc->fp != -1)
 				{
+					g_free (file_fs);
 					if (dcc_listen_init (dcc, sess))
 					{
 						char havespaces = 0;
@@ -1004,6 +1222,7 @@ dcc_send (session *sess, char *tbuf, char *to, char *file)
 			}
 		}
 	}
+	g_free (file_fs);
 	PrintTextf (sess, _("Cannot access %s\n"), dcc->file);
 	dcc_close (dcc, 0, TRUE);
 }
@@ -1062,7 +1281,7 @@ dcc_change_nick (struct server *serv, char *oldnick, char *newnick)
 		dcc = (struct DCC *) list->data;
 		if (dcc->serv == serv)
 		{
-			if (!rfc_casecmp (dcc->nick, oldnick))
+			if (!serv->p_cmp (dcc->nick, oldnick))
 			{
 				if (dcc->nick)
 					free (dcc->nick);
@@ -1102,7 +1321,7 @@ dcc_get_nick (struct session *sess, char *nick)
 	while (list)
 	{
 		dcc = (struct DCC *) list->data;
-		if (!rfc_casecmp (nick, dcc->nick))
+		if (!sess->server->p_cmp (nick, dcc->nick))
 		{
 			if (dcc->dccstat == STAT_QUEUED && dcc->type == TYPE_RECV)
 			{
@@ -1336,9 +1555,8 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 
 			dcc->file = strdup (file);
 
-			dcc->destfile =
-				malloc (strlen (prefs.dccdir) + strlen (nick) + strlen (file) +
-						  4);
+			dcc->destfile = g_malloc (strlen (prefs.dccdir) + strlen (nick) +
+											  strlen (file) + 4);
 
 			strcpy (dcc->destfile, prefs.dccdir);
 			if (prefs.dccdir[strlen (prefs.dccdir) - 1] != '/')
@@ -1361,8 +1579,11 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			}
 			strcat (dcc->destfile, file);
 
+			/* get the local filesystem encoding */
+			dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
+
 			dcc->resumable = 0;
-			if (stat (dcc->destfile, &st) != -1)
+			if (stat (dcc->destfile_fs, &st) != -1)
 			{
 				if (st.st_size < size)
 					dcc->resumable = st.st_size;
@@ -1377,6 +1598,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->pasvid = pasvid;
 			dcc->size = size;
 			dcc->nick = strdup (nick);
+			dcc->maxcps = prefs.dcc_max_get_cps;
 			if (prefs.autodccsend)
 			{
 				if (prefs.autoresume && dcc->resumable)
