@@ -288,6 +288,11 @@ inbound_chanmsg (server *serv, char *tbuf, char *chan, char *from, char *text,
 		return;
 
 	sess->highlight_tab = TRUE;
+	if (sess != current_tab)
+	{
+		sess->msg_said = TRUE;
+		sess->new_data = FALSE;
+	}
 
 	user = find_name (sess, from);
 	if (user)
@@ -331,7 +336,7 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 	session *sess;
 	GSList *list = sess_list;
 
-	if (!rfc_casecmp (nick, serv->nick))
+	if (!serv->p_cmp (nick, serv->nick))
 	{
 		me = TRUE;
 		safe_strcpy (serv->nick, newnick, NICKLEN);
@@ -355,7 +360,7 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 				}
 				change_nick (sess, nick, newnick);
 			}
-			if (!rfc_casecmp (sess->channel, nick))
+			if (!serv->p_cmp (sess->channel, nick))
 			{
 				safe_strcpy (sess->channel, newnick, CHANLEN);
 				fe_set_channel (sess);
@@ -402,7 +407,7 @@ find_session_from_waitchannel (char *chan, struct server *serv)
 		sess = (session *) list->data;
 		if (sess->server == serv && sess->channel[0] == 0 && sess->type == SESS_CHANNEL)
 		{
-			if (!rfc_casecmp (chan, sess->waitchannel))
+			if (!serv->p_cmp (chan, sess->waitchannel))
 				return sess;
 		}
 		list = list->next;
@@ -427,7 +432,7 @@ inbound_ujoin (server *serv, char *chan, char *nick, char *ip)
 			sess = find_unused_session (serv);
 			if (!sess)
 				/* last resort, open a new tab/window */
-				sess = new_ircwindow (serv, NULL, SESS_CHANNEL);
+				sess = new_ircwindow (serv, chan, SESS_CHANNEL);
 		}
 	}
 
@@ -656,7 +661,7 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
 			{
 				if (!sess->hide_join_part)
 					EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
-			} else if (sess->type == SESS_DIALOG && !rfc_casecmp (sess->channel, nick))
+			} else if (sess->type == SESS_DIALOG && !serv->p_cmp (sess->channel, nick))
 			{
 				EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
 			}
@@ -772,7 +777,12 @@ inbound_notice (server *serv, char *outbuf, char *to, char *nick, char *msg,
 			sess = find_session_from_nick (nick, serv);
 		}
 		if (!sess)
-			sess = serv->front_session;
+		{
+			if (server_notice)	
+				sess = serv->server_session;
+			else
+				sess = serv->front_session;
+		}
 	}
 
 	if (msg[0] == 1)
