@@ -554,6 +554,23 @@ inbound_005 (server * serv, char *word[])
 		} else if (strncmp (word[w], "WATCH=", 6) == 0)
 		{
 			serv->supports_watch = TRUE;
+		} else if (strncmp (word[w], "NETWORK=", 8) == 0)
+		{
+			if (serv->networkname)
+				free (serv->networkname);
+			serv->networkname = strdup (word[w] + 8);
+		} else if (strncmp (word[w], "CASEMAPPING=", 12) == 0)
+		{
+			if (strcmp (word[w] + 12, "ascii") == 0)	/* bahamut */
+				serv->p_cmp = (void *)strcasecmp;
+		} else if (strncmp (word[w], "CHARSET=", 8) == 0)
+		{
+			if (strcasecmp (word[w] + 8, "UTF-8") == 0)
+			{
+				if (serv->encoding)
+					free (serv->encoding);
+				serv->encoding = strdup ("UTF-8");
+			}
 		}
 
 		w++;
