@@ -60,108 +60,6 @@ static void help (session *sess, char *tbuf, char *helpcmd, int quiet);
 static int cmd_server (session *sess, char *tbuf, char *word[], char *word_eol[]);
 static void handle_say (session *sess, char *text, int check_spch);
 
-#ifdef USE_TRANS
-
-static unsigned char trans_serv2user[256];
-static unsigned char trans_user2serv[256];
-
-void
-serv2user (unsigned char *s)
-{
-	for (; *s; ++s)
-		*s = trans_serv2user[*s];
-}
-
-void
-user2serv (unsigned char *s)
-{
-	for (; *s; ++s)
-		*s = trans_user2serv[*s];
-}
-
-int
-load_trans_table (char *full_path)
-{
-	int tf, i, st, val = 0, t;
-	char r;
-
-	if ((tf = open (full_path, OFLAGS | O_RDONLY)) != -1)
-	{
-		st = 0;
-		i = 0;
-		t = 0;
-		while (read (tf, &r, 1) == 1)
-		{
-			switch (st)
-			{
-			case 0:					  /*nothing yet... */
-				if (r == '0')
-					st = 1;
-				break;
-			case 1:
-				if (r == 'x')
-					st = 2;
-				else
-					st = 0;
-				break;
-			case 2:
-				if (r <= '9' && r >= '0')
-					val = 16 * (r - '0');
-				else if (r <= 'f' && r >= 'a')
-					val = 16 * (r - 'a' + 10);
-				else if (r <= 'F' && r >= 'A')
-					val = 16 * (r - 'A' + 10);
-				else
-				{
-					st = 0;
-					break;
-				}
-				st = 3;
-				break;
-			case 3:
-				if (r <= '9' && r >= '0')
-					val += r - '0';
-				else if (r <= 'f' && r >= 'a')
-					val += r - 'a' + 10;
-				else if (r <= 'F' && r >= 'A')
-					val += r - 'A' + 10;
-				else
-				{
-					st = 0;
-					break;
-				}
-				st = 0;
-				if (t == 0)
-					trans_serv2user[i++] = val;
-				else
-					trans_user2serv[i++] = val;
-				if (i == 256)
-				{
-					if (t == 1)
-					{
-						close (tf);
-						return 1;
-					}
-					t = 1;
-					i = 0;
-				}
-				break;
-			default:				  /* impossible */
-				close (tf);
-				return 0;
-			}
-		}
-		close (tf);
-	}
-	for (tf = 0; tf < 256; ++tf)
-	{
-		trans_user2serv[tf] = tf;
-		trans_serv2user[tf] = tf;
-	}
-	return 0;
-}
-
-#endif /* !USE_TRANS */
 
 static void
 notj_msg (struct session *sess)
@@ -250,13 +148,14 @@ server_sendquit (session * sess)
 }
 
 void
-process_data_init (unsigned char *buf, char *cmd, char *word[],
+process_data_init (unsigned char *buf, unsigned char *cmd, char *word[],
 						 char *word_eol[], int handle_quotes)
 {
 	int wordcount = 2;
 	int space = FALSE;
 	int quote = FALSE;
 	int j = 0;
+	int len;
 
 	word[1] = buf;
 	word_eol[1] = cmd;
@@ -281,6 +180,7 @@ process_data_init (unsigned char *buf, char *cmd, char *word[],
 				quote = FALSE;
 			else
 				quote = TRUE;
+			cmd++;
 			break;
 		case ' ':
 			if (!quote)
@@ -299,15 +199,26 @@ process_data_init (unsigned char *buf, char *cmd, char *word[],
 
 					space = TRUE;
 				}
+				cmd++;
 				break;
 			}
 		default:
 def:
 			space = FALSE;
-			buf[j] = *cmd;
-			j++;
+			len = g_utf8_skip[*cmd];
+			if (len == 1)
+			{
+				buf[j] = *cmd;
+				j++;
+				cmd++;
+			} else
+			{
+				/* skip past a multi-byte utf8 char */
+				memcpy (buf + j, cmd, len);
+				j += len;
+				cmd += len;
+			}
 		}
-		cmd++;
 	}
 }
 
@@ -462,7 +373,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 
 		mask = user->hostname;
 
-		at = strchr (mask, '@');
+		at = strchr (mask, '@');	/* FIXME: utf8 */
 		if (!at)
 			return;					  /* can't happen? */
 		*at = 0;
@@ -646,6 +557,7 @@ cmd_ctcp (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 			while (1)
 			{
+				/* FIXME: utf8 */
 				if (*cmd == ' ' || *cmd == 0)
 					break;
 				*cmd = toupper (*cmd);
@@ -767,25 +679,40 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		}
 		if (!strcasecmp (type, "SEND"))
 		{
-			char *nick = word[3];
-			if (*nick)
+			int i = 3, maxcps;
+			char *nick, *file;
+
+			nick = word[i];
+			if (!*nick)
+				return FALSE;
+
+			maxcps = prefs.dcc_max_send_cps;
+			if (!strncasecmp(nick, "-maxcps=", 8))
 			{
-				int i = 4;
-				char *file;
-				while (1)
-				{
-					file = word[i];
-					if (!*file && i == 4)
-					{
-						fe_dcc_send_filereq (sess, nick);
-						return TRUE;
-					}
-					if (!*file)
-						break;
-					dcc_send (sess, tbuf, nick, file);
-					i++;
-				}
+				maxcps = atoi(nick + 8);
+				i++;
+				nick = word[i];
+				if (!*nick)
+					return FALSE;
 			}
+
+			i++;
+
+			file = word[i];
+			if (!*file)
+			{
+				fe_dcc_send_filereq (sess, nick, maxcps);
+				return TRUE;
+			}
+
+			do
+			{
+				dcc_send (sess, tbuf, nick, file, maxcps);
+				i++;
+				file = word[i];
+			}
+			while (*file);
+
 			return TRUE;
 		}
 	} else
@@ -1436,7 +1363,7 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			/* Parent path */
 			close(fds[1]);
 			s->childpid = pid;
-			s->iotag = fe_input_add (s->myfd, 1, 0, 1, exec_data, s);
+			s->iotag = fe_input_add (s->myfd, FIA_READ|FIA_EX, exec_data, s);
 			sess->running_exec = s;
 			return TRUE;
 		}
@@ -1486,6 +1413,78 @@ cmd_gate (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+typedef struct
+{
+	char *cmd;
+	session *sess;
+} getvalinfo;
+
+static void
+get_int_cb (int cancel, int val, getvalinfo *info)
+{
+	char buf[512];
+
+	if (!cancel)
+	{
+		snprintf (buf, sizeof (buf), "%s %d", info->cmd, val);
+		if (is_session (info->sess))
+			handle_command (info->sess, buf, FALSE);
+	}
+
+	free (info->cmd);
+	free (info);
+}
+
+static int
+cmd_getint (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	getvalinfo *info;
+
+	if (!word[4][0])
+		return FALSE;
+
+	info = malloc (sizeof (*info));
+	info->cmd = strdup (word[3]);
+	info->sess = sess;
+
+	fe_get_int (word[4], atoi (word[2]), get_int_cb, info);
+
+	return TRUE;
+}
+
+static void
+get_str_cb (int cancel, char *val, getvalinfo *info)
+{
+	char buf[512];
+
+	if (!cancel)
+	{
+		snprintf (buf, sizeof (buf), "%s %s", info->cmd, val);
+		if (is_session (info->sess))
+			handle_command (info->sess, buf, FALSE);
+	}
+
+	free (info->cmd);
+	free (info);
+}
+
+static int
+cmd_getstr (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	getvalinfo *info;
+
+	if (!word[4][0])
+		return FALSE;
+
+	info = malloc (sizeof (*info));
+	info->cmd = strdup (word[3]);
+	info->sess = sess;
+
+	fe_get_str (word[4], word[2], get_str_cb, info);
+
+	return TRUE;
+}
+
 static int
 cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1808,6 +1807,9 @@ cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 #endif
 
+	sprintf (tbuf, "Unknown file type %s\n", word[2]);
+	PrintText (sess, tbuf);
+
 	return FALSE;
 }
 
@@ -2107,7 +2109,7 @@ cmd_recv (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (*word_eol[2])
 	{
-		sess->server->p_inline (sess->server, word_eol[2]);
+		sess->server->p_inline (sess->server, word_eol[2], strlen (word_eol[2]));
 		return TRUE;
 	}
 
@@ -2159,6 +2161,13 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 #endif
 
+	/* connect by Network name */
+	if (word[3][0] == 0)
+	{
+		if (servlist_connect_by_netname (sess, word[2]))
+			return TRUE;
+	}
+
 	server_name = word[2 + offset];
 	port = word[3 + offset];
 	pass = word[4 + offset];
@@ -2298,6 +2307,8 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	case 0:
 			PrintText (sess, _("No such plugin found.\n"));
 			break;
+	case 1:
+			return TRUE;
 	case 2:
 			PrintText (sess, _("That plugin is refusing to unload.\n"));
 			break;
@@ -2358,7 +2369,7 @@ cmd_wallchop (struct session *sess, char *tbuf, char *word[],
 			{
 				i = 0;
 				sprintf (tbuf + strlen (tbuf),
-							" :[@%s] %s\r\n", sess->channel, word_eol[2]);
+							" :[@%s] %s", sess->channel, word_eol[2]);
 				sess->server->p_raw (sess->server, tbuf);
 				strcpy (tbuf, "NOTICE ");
 			}
@@ -2453,10 +2464,11 @@ const struct commands xc_cmds[] = {
 	{"CYCLE", cmd_cycle, 1, 1,
 	 N_("CYCLE, parts current channel and immediately rejoins")},
 	{"DCC", cmd_dcc, 0, 0,
-	 N_("\n" "DCC GET <nick>          - receive an offered file\n"
-	 "DCC SEND <nick> <file>  - send a file to someone\n"
-	 "DCC LIST                - show DCC list\n"
-	 "DCC CHAT <nick>         - offer DCC CHAT to someone\n"
+	 N_("\n"
+	 "DCC GET <nick>                     - accept an offered file\n"
+	 "DCC SEND [-maxcps=#] <nick> [file] - send a file to someone\n"
+	 "DCC LIST                           - show DCC list\n"
+	 "DCC CHAT <nick>                    - offer DCC CHAT to someone\n"
 	 "DCC CLOSE <type> <nick> <file>         example:\n"
 	 "         /dcc close send johnsmith file.tar.gz")},
 	{"DEBUG", cmd_debug, 0, 0, 0},
@@ -2487,6 +2499,8 @@ const struct commands xc_cmds[] = {
 	 N_("FLUSHQ, flushes the current server's send queue")},
 	{"GATE", cmd_gate, 0, 0,
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
+	{"GETINT", cmd_getint, 0, 0, "GETINT <default> <command> <prompt>"},
+	{"GETSTR", cmd_getstr, 0, 0, "GETSTR <default> <command> <prompt>"},
 	{"HELP", cmd_help, 0, 0, 0},
 	{"HOP", cmd_hop, 1, 1,
 	 N_("HOP <nick>, gives chanhalf-op status to the nick (needs chanop)")},
@@ -2623,7 +2637,7 @@ help (session *sess, char *tbuf, char *helpcmd, int quiet)
  * - this beast is used for UserCommands, UserlistButtons and CTCP replies   */
 
 int
-auto_insert (char *dest, char *src, char *word[], char *word_eol[],
+auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
 				 char *a, char *c, char *d, char *h, char *n, char *s)
 {
 	int num;
@@ -2721,13 +2735,23 @@ auto_insert (char *dest, char *src, char *word[], char *word_eol[],
 				}
 				dest += strlen (dest);
 			}
+			src++;
 		} else
 		{
+			utf_len = g_utf8_skip[src[0]];
+			if (utf_len == 1)
+			{
 		 lamecode:
-			dest[0] = src[0];
-			dest++;
+				dest[0] = src[0];
+				dest++;
+				src++;
+			} else
+			{
+				memcpy (dest, src, utf_len);
+				dest += utf_len;
+				src += utf_len;
+			}
 		}
-		src++;
 	}
 
 	dest[0] = 0;
@@ -2950,6 +2974,7 @@ handle_say (session *sess, char *text, int check_spch)
 
 		if (strlen (text) > max)
 		{
+			/* FIXME: utf8 */
 			t = text[max];
 			text[max] = 0;			  /* insert a NULL terminator to shorten it */
 		}
@@ -3055,8 +3080,7 @@ handle_command (session *sess, char *cmd, int check_spch)
 		PrintText (sess, _("Unknown Command. Try /help\n"));
 		goto xit;
 	}
-	snprintf (tbuf, sizeof (tbuf), "%s\r\n", cmd);
-	sess->server->p_raw (sess->server, tbuf);
+	sess->server->p_raw (sess->server, cmd);
 
 xit:
 	command_level--;
