@@ -28,7 +28,12 @@
 #include "cfgfiles.h"
 #include "ignore.h"
 #include "text.h"
+#define PLUGIN_C
+typedef struct session xchat_context;
+#include "xchat-plugin.h"
 #include "plugin.h"
+
+
 #include "xchatc.h"
 
 /* the USE_PLUGIN define only removes libdl stuff */
@@ -43,36 +48,33 @@
 
 #define DEBUG(t) PrintText(0,t)
 
-typedef int (xchat_cmd_cb) (char *word[], char *word_eol[], void *user_data);
-typedef int (xchat_serv_cb) (char *word[], char *word_eol[], void *user_data);
-typedef int (xchat_print_cb) (char *word[], void *user_data);
-typedef int (xchat_timer_cb) (void *user_data);
-typedef int (xchat_init_func) (xchat_plugin *, char **, char **, char **, char *);
-typedef int (xchat_deinit_func) (xchat_plugin *);
-
-typedef struct session xchat_context;
-
-typedef struct xchat_hook
+/* crafted to be an even 32 bytes */
+struct _xchat_hook
 {
 	xchat_plugin *pl;	/* the plugin to which it belongs */
 	char *name;			/* "xdcc" */
 	void *callback;	/* pointer to xdcc_callback */
 	char *help_text;	/* help_text for commands only */
 	void *userdata;	/* passed to the callback */
-	int tag;				/* for timers only */
+	int tag;				/* for timers & FDs only */
 	int type;			/* HOOK_* */
-	int pri;				/* priority */
-} xchat_hook;
+	int pri;	/* fd */	/* priority / fd for HOOK_FD only */
+};
 
-typedef struct xchat_list
+struct _xchat_list
 {
 	int type;			/* LIST_* */
 	GSList *pos;		/* current pos */
 	GSList *next;		/* next pos */
-} xchat_list;
+};
 
-#define PLUGIN_C
-#include "xchat-plugin.h"
+typedef int (xchat_cmd_cb) (char *word[], char *word_eol[], void *user_data);
+typedef int (xchat_serv_cb) (char *word[], char *word_eol[], void *user_data);
+typedef int (xchat_print_cb) (char *word[], void *user_data);
+typedef int (xchat_fd_cb) (int fd, int flags, void *user_data);
+typedef int (xchat_timer_cb) (void *user_data);
+typedef int (xchat_init_func) (xchat_plugin *, char **, char **, char **, char *);
+typedef int (xchat_deinit_func) (xchat_plugin *);
 
 enum
 {
@@ -87,7 +89,9 @@ enum
 	HOOK_COMMAND,	/* /command */
 	HOOK_SERVER,	/* PRIVMSG, NOTICE, numerics */
 	HOOK_PRINT,		/* All print events */
-	HOOK_TIMER
+	HOOK_TIMER,		/* timeouts */
+	HOOK_FD,			/* sockets & fds */
+	HOOK_DELETED	/* marked for deletion */
 };
 
 GSList *plugin_list = NULL;	/* export for plugingui.c */
@@ -205,6 +209,33 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 
 	if (!fake)
 	{
+#ifdef WIN32
+		pl->xchat_hook_command = xchat_hook_command;
+		pl->xchat_hook_server = xchat_hook_server;
+		pl->xchat_hook_print = xchat_hook_print;
+		pl->xchat_hook_timer = xchat_hook_timer;
+		pl->xchat_hook_fd = xchat_hook_fd;
+		pl->xchat_unhook = xchat_unhook;
+		pl->xchat_print = xchat_print;
+		pl->xchat_printf = xchat_printf;
+		pl->xchat_command = xchat_command;
+		pl->xchat_commandf = xchat_commandf;
+		pl->xchat_nickcmp = xchat_nickcmp;
+		pl->xchat_set_context = xchat_set_context;
+		pl->xchat_find_context = xchat_find_context;
+		pl->xchat_get_context = xchat_get_context;
+		pl->xchat_get_info = xchat_get_info;
+		pl->xchat_get_prefs = xchat_get_prefs;
+		pl->xchat_list_get = xchat_list_get;
+		pl->xchat_list_free = xchat_list_free;
+		pl->xchat_list_fields = xchat_list_fields;
+		pl->xchat_list_str = xchat_list_str;
+		pl->xchat_list_next = xchat_list_next;
+		pl->xchat_list_int = xchat_list_int;
+		pl->xchat_plugingui_add = xchat_plugingui_add;
+		pl->xchat_plugingui_remove = xchat_plugingui_remove;
+#endif
+
 		/* run xchat_plugin_init, if it returns 0, close the plugin */
 		if (((xchat_init_func *)init_func) (pl, &pl->name, &pl->desc, &pl->version, arg) == 0)
 		{
@@ -230,7 +261,9 @@ plugin_kill (char *name, int by_filename)
 	while (list)
 	{
 		pl = list->data;
-		if ((by_filename && strcasecmp (name, pl->filename) == 0) ||
+		/* static-plugins (plugin-timer.c) have a NULL filename */
+		if ((by_filename && pl->filename && strcasecmp (name, pl->filename) == 0) ||
+			 (by_filename && pl->filename && strcasecmp (name, file_part (pl->filename)) == 0) ||
 			(!by_filename && strcasecmp (name, pl->name) == 0))
 		{
 			/* statically linked plugins have a NULL filename */
@@ -334,9 +367,9 @@ plugin_auto_load (session *sess)
 {
 	ps = sess;
 #ifdef WIN32
-	for_files (get_xdir (), "*.dll", plugin_auto_load_cb);
+	for_files ("./plugins", "*.dll", plugin_auto_load_cb);
 #else
-	for_files (XCHATDIR"/plugins", "*.so", plugin_auto_load_cb);
+	for_files (XCHATLIBDIR"/plugins", "*.so", plugin_auto_load_cb);
 	for_files (get_xdir (), "*.so", plugin_auto_load_cb);
 #endif
 }
@@ -382,7 +415,7 @@ plugin_hook_run (session *sess, char *name, char *word[], char *word_eol[], int
 	{
 		list = plugin_hook_find (list, type, name);
 		if (!list)
-			return eat;
+			goto xit;
 
 		hook = list->data;
 		next = list->next;
@@ -402,15 +435,35 @@ plugin_hook_run (session *sess, char *name, char *word[], char *word_eol[], int
 			break;
 		}
 
-		if (ret & EAT_ALL)
-			return 1;
-		if (ret & EAT_PLUGIN)
-			return eat;	/* stop running plugins */
-		if (ret & EAT_XCHAT)
+		if ((ret & XCHAT_EAT_XCHAT) && (ret & XCHAT_EAT_PLUGIN))
+		{
+			eat = 1;
+			goto xit;
+		}
+		if (ret & XCHAT_EAT_PLUGIN)
+			goto xit;	/* stop running plugins */
+		if (ret & XCHAT_EAT_XCHAT)
 			eat = 1;	/* eventually we'll return 1, but continue running plugins */
 
 		list = next;
 	}
+
+xit:
+	/* really remove deleted hooks now */
+	list = hook_list;
+	while (list)
+	{
+		hook = list->data;
+		next = list->next;
+		if (hook->type == HOOK_DELETED)
+		{
+			hook_list = g_slist_remove (hook_list, hook);
+			free (hook);
+		}
+		list = next;
+	}
+
+	return eat;
 }
 
 /* execute a plugged in command. Called from outbound.c */
@@ -497,6 +550,21 @@ plugin_insert_hook (xchat_hook *new_hook)
 	hook_list = g_slist_append (hook_list, new_hook);
 }
 
+static gboolean
+plugin_fd_cb (GIOChannel *source, GIOCondition condition, xchat_hook *hook)
+{
+	int flags = 0;
+
+	if (condition & G_IO_IN)
+		flags |= XCHAT_FD_READ;
+	if (condition & G_IO_OUT)
+		flags |= XCHAT_FD_WRITE;
+	if (condition & G_IO_PRI)
+		flags |= XCHAT_FD_EXCEPTION;
+
+	return ((xchat_fd_cb *)hook->callback) (hook->pri, flags, hook->userdata);
+}
+
 /* allocate and add a hook to our list. Used for all 4 types */
 
 static xchat_hook *
@@ -568,25 +636,24 @@ plugin_show_help (session *sess, char *cmd)
 void *
 xchat_unhook (xchat_plugin *ph, xchat_hook *hook)
 {
-	void *userdata;
-
 	/* perl.c trips this */
-	if (!g_slist_find (hook_list, hook))
+	if (hook->type == HOOK_DELETED || !g_slist_find (hook_list, hook))
 		return NULL;
 
 	if (hook->type == HOOK_TIMER && hook->tag != 0)
 		fe_timeout_remove (hook->tag);
 
-	hook_list = g_slist_remove (hook_list, hook);
+	if (hook->type == HOOK_FD)
+		fe_input_remove (hook->tag);
+
+	hook->type = HOOK_DELETED;	/* expunge later */
 
-	userdata = hook->userdata;
 	if (hook->name)
-		free (hook->name);	/* NULL for timers */
+		free (hook->name);	/* NULL for timers & fds */
 	if (hook->help_text)
 		free (hook->help_text);	/* NULL for non-commands */
-	free (hook);
 
-	return userdata;
+	return hook->userdata;
 }
 
 xchat_hook *
@@ -601,22 +668,35 @@ xchat_hook *
 xchat_hook_server (xchat_plugin *ph, char *name, int pri, xchat_serv_cb *callb,
 					 	 void *userdata)
 {
-	return plugin_add_hook (ph, HOOK_SERVER, pri, name, NULL, callb, 0, userdata);
+	return plugin_add_hook (ph, HOOK_SERVER, pri, name, 0, callb, 0, userdata);
 }
 
 xchat_hook *
 xchat_hook_print (xchat_plugin *ph, char *name, int pri, xchat_print_cb *callb,
 					   void *userdata)
 {
-	return plugin_add_hook (ph, HOOK_PRINT, pri, name, NULL, callb, 0, userdata);
+	return plugin_add_hook (ph, HOOK_PRINT, pri, name, 0, callb, 0, userdata);
 }
 
 xchat_hook *
 xchat_hook_timer (xchat_plugin *ph, int timeout, xchat_timer_cb *callb,
 					   void *userdata)
 {
-	return plugin_add_hook (ph, HOOK_TIMER, 0, NULL, NULL, callb, timeout,
-									userdata);
+	return plugin_add_hook (ph, HOOK_TIMER, 0, 0, 0, callb, timeout, userdata);
+}
+
+xchat_hook *
+xchat_hook_fd (xchat_plugin *ph, int fd, int flags,
+					xchat_fd_cb *callb, void *userdata)
+{
+	xchat_hook *hook;
+
+	hook = plugin_add_hook (ph, HOOK_FD, 0, 0, 0, callb, 0, userdata);
+	hook->pri = fd;
+	/* plugin hook_fd flags correspond exactly to FIA_* flags (fe.h) */
+	hook->tag = fe_input_add (fd, flags, plugin_fd_cb, hook);
+
+	return hook;
 }
 
 void
@@ -674,7 +754,7 @@ xchat_commandf (xchat_plugin *ph, char *format, ...)
 int
 xchat_nickcmp (xchat_plugin *ph, char *s1, char *s2)
 {
-	return rfc_casecmp (s1, s2);
+	return ((session *)ph->context)->server->p_cmp (s1, s2);
 }
 
 xchat_context *
@@ -707,7 +787,8 @@ xchat_find_context (xchat_plugin *ph, char *servname, char *channel)
 		serv = slist->data;
 		if (servname == NULL ||
 			 rfc_casecmp (servname, serv->servername) == 0 ||
-			 strcasecmp (servname, serv->hostname) == 0)
+			 strcasecmp (servname, serv->hostname) == 0 ||
+			 (serv->networkname && strcasecmp (servname, serv->networkname) == 0))
 		{
 			if (channel == NULL)
 				return serv->front_session;
@@ -755,6 +836,9 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x30f5a8: /* host */
 		return sess->server->hostname;
 
+	case 0x6de15a2e:	/* network */
+		return sess->server->networkname;
+
 	case 0x339763: /* nick */
 		return sess->server->nick;
 
