@@ -27,6 +27,10 @@
 
 #include "xchat.h"
 
+#ifdef WIN32
+#include "inet.h"	/* for gethostname() */
+#endif
+
 #include "ctcp.h"
 #include "fe.h"
 #include "ignore.h"
@@ -381,7 +385,7 @@ process_numeric (session * sess, char *outbuf, int n,
 			fe_add_chan_list (sess->server, word[4], word[5], word_eol[6] + 1);
 		} else
 		{
-			sprintf (outbuf, "%-16.16s %-7d %s\017\n",
+			sprintf (outbuf, "%-16s %-7d %s\017\n",
 						word[4], atoi (word[5]), word_eol[6] + 1);
 			PrintText (sess, outbuf);
 		}
@@ -446,7 +450,7 @@ process_numeric (session * sess, char *outbuf, int n,
 				goto def;
 		} else
 		{
-			if (!rfc_casecmp (word[8], serv->nick))
+			if (!serv->p_cmp (word[8], serv->nick))
 				inbound_foundip (sess, word[6]);
 		}
 		break;
@@ -604,7 +608,7 @@ process_named_msg (session *sess, char *type, char *outbuf,
 
 		if (*chan == ':')
 			chan++;
-		if (!rfc_casecmp (nick, serv->nick))
+		if (!serv->p_cmp (nick, serv->nick))
 			inbound_ujoin (serv, chan, nick, ip);
 		else
 			inbound_join (serv, chan, nick, ip);
@@ -763,21 +767,26 @@ process_named_servermsg (session *sess, char *buf, char *word_eol[])
 /* irc_inline() - 1 single line received from serv */
 
 static void
-irc_inline (server *serv, char *buf)
+irc_inline (server *serv, char *buf, int len)
 {
 	session *sess, *tmp;
 	char *type, *text;
 	char *word[PDIWORDS];
 	char *word_eol[PDIWORDS];
 	char outbuf[4096];
-	char pdibuf[522];				  /* 1 line can't exceed 512 bytes!! */
+	char pdibuf_static[522]; /* 1 line can potentially be 512*6 in utf8 */
+	char *pdibuf = pdibuf_static;
+
+	/* need more than 522? fall back to malloc */
+	if (len >= sizeof (pdibuf_static))
+		pdibuf = malloc (len + 1);
 
 	sess = serv->front_session;
 
 	if (buf[0] == ':')
 	{
 		/* split line into words and words_to_end_of_line */
-		process_data_init (pdibuf, buf + 1, word, word_eol, FALSE);
+		process_data_init (pdibuf, buf, word, word_eol, FALSE);
 
 		/* find a context for this message */
 		if (is_channel (serv, word[3]))
@@ -789,20 +798,26 @@ irc_inline (server *serv, char *buf)
 
 		/* for server messages, the 2nd word is the "message type" */
 		type = word[2];
+
+		word[0] = type;
+		word_eol[1] = buf;	/* keep the ":" for plugins */
+		if (plugin_emit_server (sess, type, word, word_eol))
+			goto xit;
+		word[1]++;
+		word_eol[1] = buf + 1;	/* but not for xchat internally */
+
 	} else
 	{
 		process_data_init (pdibuf, buf, word, word_eol, FALSE);
-		type = word[1];
+		word[0] = type = word[1];
+		if (plugin_emit_server (sess, type, word, word_eol))
+			goto xit;
 	}
 
-	word[0] = type;
-	if (plugin_emit_server (sess, type, word, word_eol))
-		return;
-
 	if (buf[0] != ':')
 	{
 		process_named_servermsg (sess, buf, word_eol);
-		return;
+		goto xit;
 	}
 
 	/* see if the second word is a numeric */
@@ -810,13 +825,17 @@ irc_inline (server *serv, char *buf)
 	{
 		text = word_eol[4];
 		if (*text == ':')
-				text++;
+			text++;
 
 		process_numeric (sess, outbuf, atoi (word[2]), word, word_eol, text);
-		return;
+	} else
+	{
+		process_named_msg (sess, type, outbuf, word, word_eol);
 	}
 
-	process_named_msg (sess, type, outbuf, word, word_eol);	
+xit:
+	if (pdibuf != pdibuf_static)
+		free (pdibuf);
 }
 
 void
@@ -850,4 +869,5 @@ proto_fill_her_up (server *serv)
 	serv->p_names = irc_names;
 	serv->p_ping = irc_ping;
 	serv->p_raw = irc_raw;
+	serv->p_cmp = rfc_casecmp;	/* can be changed by 005 in modes.c */
 }
