@@ -55,10 +55,6 @@
 #include "ssl.h"
 #endif
 
-#ifdef USE_JCODE
-#include "jcode.h"
-#endif
-
 #ifdef WIN32
 #include "identd.c"
 #endif
@@ -74,23 +70,32 @@ static void server_disconnect (session * sess, int sendquit, int err);
 static int server_cleanup (server * serv);
 static void server_connect (server *serv, char *hostname, int port, int no_login);
 
+
+/* actually send to the socket. This might do a character translation or
+   send via SSL */
+
 static int
-tcp_send_real_real (server *serv, char *buf, int len)
+tcp_send_real (server *serv, char *buf, int len)
 {
-	int ret = 0;
-	char *locale = NULL;
+	int ret;
+	char *locale;
+	int loc_len;
 
-	if (!serv->use_utf8)
+	fe_add_rawlog (serv, buf, len, TRUE);
+
+	if (serv->encoding == NULL)	/* system */
 	{
-		int loc_len;
-
-		locale = g_locale_from_utf8 (buf, len, NULL, &loc_len, NULL);
-		if (locale)
-			len = loc_len;
+		locale = NULL;
+		if (!prefs.utf8_locale)
+			locale = g_locale_from_utf8 (buf, len, NULL, &loc_len, NULL);
+	} else
+	{
+		locale = g_convert (buf, len, serv->encoding, "UTF-8", 0, &loc_len, 0);
 	}
 
 	if (locale)
 	{
+		len = loc_len;
 #ifdef USE_OPENSSL
 		if (!serv->ssl)
 			ret = send (serv->sok, locale, len, 0);
@@ -115,45 +120,6 @@ tcp_send_real_real (server *serv, char *buf, int len)
 	return ret;
 }
 
-/* actually send to the socket. This might do a character translation or
-   send via SSL */
-
-static int
-tcp_send_real (server *serv, char *buf, int len)
-{
-	int ret;
-	unsigned char *tbuf = buf;
-#ifdef USE_TRANS
-#define TRANS_STAT_BUF_LEN 1024
-	static unsigned char sbuf[TRANS_STAT_BUF_LEN];
-
-	if (prefs.use_trans)
-	{
-		if (len >= TRANS_STAT_BUF_LEN)
-			tbuf = malloc (len + 1);
-		else
-			tbuf = sbuf;
-		if (!tbuf)
-			return -1;
-		memcpy (tbuf, buf, len + 1);
-		user2serv (tbuf);
-	}
-#endif
-
-	fe_add_rawlog (serv, tbuf, TRUE);
-	ret = tcp_send_real_real (serv, tbuf, len);
-
-#ifdef USE_TRANS
-	if (prefs.use_trans)
-	{
-		if (tbuf != sbuf)
-			free (tbuf);
-	}
-#endif
-
-	return ret;
-}
-
 /* new throttling system, uses the same method as the Undernet
    ircu2.10 server; under test, a 200-line paste didn't flood
    off the client */
@@ -216,39 +182,14 @@ tcp_send_len (server *serv, char *buf, int len)
 	char *dbuf;
 	int noqueue = !serv->outbound_queue;
 
-#ifdef USE_JCODE
-	if (prefs.kanji_conv)
-	{
-		buf = kanji_conv_auto (buf, "ISO-2022-JP");
-		if (!buf)
-			return -1;
-		len = strlen (buf);
-	}
-
-	if (!prefs.throttle)
-	{
-		int ret = tcp_send_real (serv, buf, len);
-		if (prefs.kanji_conv)
-			free (buf);
-		return ret;
-	}
-#else
-
 	if (!prefs.throttle)
 		return tcp_send_real (serv, buf, len);
 
-#endif
-
 	dbuf = malloc (len + 2);	/* first byte is the priority */
 	dbuf[0] = 1;	/* pri 1 for most things */
 	memcpy (dbuf + 1, buf, len);
 	dbuf[len + 1] = 0;
 
-#ifdef USE_JCODE
-	if (prefs.kanji_conv)
-		free (buf);
-#endif
-
 	/* privmsg and notice get a lower priority */
 	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
 		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
@@ -263,11 +204,11 @@ tcp_send_len (server *serv, char *buf, int len)
 	return 1;
 }
 
-int
+/*int
 tcp_send (server *serv, char *buf)
 {
 	return tcp_send_len (serv, buf, strlen (buf));
-}
+}*/
 
 void
 tcp_sendf (server *serv, char *fmt, ...)
@@ -305,41 +246,42 @@ close_socket (int sok)
 /* handle 1 line of text received from the server */
 
 static void
-server_inline (server *serv, char *line)
+server_inline (server *serv, char *line, int len)
 {
-	char *utf = NULL;
-#ifdef USE_JCODE
-	char *oline;
+	char *utf;
+	char *conv;
+	int utf_len;
 
-	oline = line;
-	if (prefs.kanji_conv)
-		line = kanji_conv_to_locale (line);
-#endif
-
-	if (!serv->use_utf8)
-		utf = g_locale_to_utf8 (line, -1, NULL, NULL, NULL);
-	if (utf)
+	if (serv->encoding == NULL)	/* system */
 	{
-		fe_add_rawlog (serv, utf, FALSE);
-		url_check (utf);
-
-		/* let proto-irc.c handle it */
-		serv->p_inline (serv, utf);
-
-		g_free (utf);
+		utf = NULL;
+		if (!prefs.utf8_locale)
+			utf = g_locale_to_utf8 (line, len, NULL, &utf_len, NULL);
 	} else
 	{
-		fe_add_rawlog (serv, line, FALSE);
-		url_check (line);
-
-		/* let proto-irc.c handle it */
-		serv->p_inline (serv, line);
+		utf = g_convert (line, len, "UTF-8", serv->encoding, 0, &utf_len, 0);
 	}
 
-#ifdef USE_JCODE
-	if (oline != line)
-		free (line);
-#endif
+	if (utf)
+	{
+		line = utf;
+		len = utf_len;
+	}
+
+	/* we really need valid UTF-8 now */
+	conv = text_validate (&line, &len);
+
+	fe_add_rawlog (serv, line, len, FALSE);
+	url_check (line);
+
+	/* let proto-irc.c handle it */
+	serv->p_inline (serv, line, len);
+
+	if (utf)
+		g_free (utf);
+
+	if (conv)
+		g_free (conv);
 }
 
 /* read data from socket */
@@ -371,10 +313,21 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 					return TRUE;
 				error = sock_error ();
 			}
-			if (prefs.autoreconnect)
-				auto_reconnect (serv, FALSE, error);
-			else
+			if (!serv->end_of_motd)
+			{
 				server_disconnect (serv->server_session, FALSE, error);
+				if (!servlist_cycle (serv))
+				{
+					if (prefs.autoreconnect)
+						auto_reconnect (serv, FALSE, error);
+				}
+			} else
+			{
+				if (prefs.autoreconnect)
+					auto_reconnect (serv, FALSE, error);
+				else
+					server_disconnect (serv->server_session, FALSE, error);
+			}
 			return TRUE;
 		}
 
@@ -391,25 +344,21 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 
 			case '\n':
 				serv->linebuf[serv->pos] = 0;
-#ifdef USE_TRANS
-				if (prefs.use_trans)
-					serv2user (serv->linebuf);
-#endif
 				if (prefs.stripcolor)
 				{
 					temp = strip_color (serv->linebuf);
-					server_inline (serv, temp);
+					server_inline (serv, temp, strlen (temp));
 					free (temp);
 				} else
 				{
-					server_inline (serv, serv->linebuf);
+					server_inline (serv, serv->linebuf, serv->pos);
 				}
 				serv->pos = 0;
 				break;
 
 			default:
 				serv->linebuf[serv->pos] = lbuf[i];
-				if (serv->pos > 519)
+				if (serv->pos >= (sizeof (serv->linebuf) - 1))
 					fprintf (stderr,
 								"*** XCHAT WARNING: Buffer overflow - shit server!\n");
 				else
@@ -442,12 +391,14 @@ server_connected (server * serv)
 	prefs.wait_on_exit = TRUE;
 	serv->connected = TRUE;
 	set_nonblocking (serv->sok);
-	serv->iotag = fe_input_add (serv->sok, 1, 0, 1, server_read, serv);
+	serv->iotag = fe_input_add (serv->sok, FIA_READ|FIA_EX, server_read, serv);
 	if (!serv->no_login)
 	{
 		EMIT_SIGNAL (XP_TE_CONNECTED, serv->server_session, NULL, NULL, NULL,
 						 NULL, 0);
-		serv->p_login (serv, prefs.username, prefs.realname);
+		serv->p_login (serv,
+							(serv->username) ? serv->username : prefs.username,
+							(serv->realname) ? serv->realname : prefs.realname);
 	} else
 	{
 		EMIT_SIGNAL (XP_TE_SERVERCONNECTED, serv->server_session, NULL, NULL,
@@ -493,7 +444,7 @@ server_stopconnecting (server * serv)
 
 #ifdef USE_OPENSSL
 #define	SSLTMOUT	10				  /* seconds */
-void
+static void
 ssl_cb_info (SSL * s, int where, int ret)
 {
 /*	char buf[128];*/
@@ -705,10 +656,7 @@ auto_reconnect (server *serv, int send_quit, int err)
 	int del;
 
 	if (serv->server_session == NULL)
-	{
-		puts("server_sess was 0!");
 		return;
-	}
 
 	list = sess_list;
 	while (list)				  /* make sure auto rejoin can work */
@@ -1329,8 +1277,11 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	fe_set_away (serv);
 	server_flush_queue (serv);
 
-	/* pipe is #defined on win32 in gwin32.h to use _pipe and O_BINARY */
+#ifdef WIN32
+	if (_pipe (read_des, 4096, _O_BINARY) < 0)
+#else
 	if (pipe (read_des) < 0)
+#endif
 		return;
 #ifdef __EMX__ /* os/2 */
 	setmode (read_des[0], O_BINARY);
@@ -1360,9 +1311,8 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	}
 #endif
 	serv->childpid = pid;
-	/* the 3 tells input_add that it's a fd, not a socket */
 	serv->iotag =
-		fe_input_add (serv->childread, 3, 0, 0, server_read_child, serv);
+		fe_input_add (serv->childread, FIA_READ|FIA_FD, server_read_child, serv);
 }
 
 void server_fill_her_up (server *serv)
