@@ -141,6 +141,105 @@ static char *strhebpatch(char *dest, const char *src)
 
 #endif /* !USE_HEBREW */
 
+/* check if a word is clickable */
+
+int
+text_word_check (char *word)
+{
+	session *sess = current_sess;
+	char *at, *dot;
+	int i, dots;
+	int len = strlen (word);
+
+	if ((word[0] == '@' || word[0] == '+') && word[1] == '#')
+		return WORD_CHANNEL;
+
+	if (word[0] == '#' && word[1] != '#' && word[1] != 0)
+		return WORD_CHANNEL;
+
+	if (!strncasecmp (word, "irc://", 6))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "irc.", 4))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "ftp.", 4))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "ftp:", 4))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "www.", 4))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "http:", 5))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "gopher:", 7))
+		return WORD_URL;
+
+	if (!strncasecmp (word, "https:", 6))
+		return WORD_URL;
+
+	if (find_name (sess, word))
+		return WORD_NICK;
+
+	at = strchr (word, '@');	  /* check for email addy */
+	dot = strrchr (word, '.');
+	if (at && dot)
+	{
+		if ((unsigned long) at < (unsigned long) dot)
+		{
+			if (strchr (word, '*'))
+				return WORD_HOST;
+			else
+				return WORD_EMAIL;
+		}
+	}
+
+	/* check if it's an IP number */
+	dots = 0;
+	for (i = 0; i < len; i++)
+	{
+		if (word[i] == '.')
+			dots++;
+		else if (!isdigit (word[i]))
+		{
+			dots = 0;
+			break;
+		}
+	}
+	if (dots == 3)
+		return WORD_HOST;
+
+	if (!strncasecmp (word + len - 5, ".html", 5))
+		return WORD_HOST;
+
+	if (!strncasecmp (word + len - 4, ".org", 4))
+		return WORD_HOST;
+
+	if (!strncasecmp (word + len - 4, ".net", 4))
+		return WORD_HOST;
+
+	if (!strncasecmp (word + len - 4, ".com", 4))
+		return WORD_HOST;
+
+	if (!strncasecmp (word + len - 4, ".edu", 4))
+		return WORD_HOST;
+
+	if (len > 5)
+	{
+		if (word[len - 3] == '.' &&
+			 isalpha (word[len - 2]) && isalpha (word[len - 1]))
+			return WORD_HOST;
+	}
+
+	if (sess->type == SESS_DIALOG)
+		return WORD_DIALOG;
+
+	return 0;
+}
+
 void
 log_close (session *sess)
 {
@@ -159,7 +258,7 @@ log_close (session *sess)
 }
 
 static void
-log_create_filename (char *buf, char *servname, char *channame)
+log_create_filename (char *buf, char *servname, char *channame, char *netname)
 {
 	char fn[256];
 	char *dir, *sep;
@@ -193,7 +292,7 @@ log_create_filename (char *buf, char *servname, char *channame)
 #else
 		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
 #endif
-	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", "", servname);
+	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
 	free (channame);
 
 	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir (), fn);
@@ -243,13 +342,16 @@ log_create_filename (char *buf, char *servname, char *channame)
 }
 
 static int
-log_open_file (char *servname, char *channame)
+log_open_file (char *servname, char *channame, char *netname)
 {
 	char buf[512];
 	int fd;
 	time_t currenttime;
 
-	log_create_filename (buf, servname, channame);
+	if (!netname)
+		netname = "NETWORK";
+
+	log_create_filename (buf, servname, channame, netname);
 #ifdef WIN32
 	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY | OFLAGS, S_IREAD|S_IWRITE);
 #else
@@ -269,7 +371,8 @@ void
 log_open (session *sess)
 {
 	log_close (sess);
-	sess->logfd = log_open_file (sess->server->servername, sess->channel);
+	sess->logfd = log_open_file (sess->server->servername, sess->channel,
+										  sess->server->networkname);
 }
 
 static void
@@ -294,9 +397,36 @@ log_write (session *sess, char *text)
 	}
 }
 
+char *
+text_validate (char **text, int *len)
+{
+	char *utf;
+	int utf_len;
+
+	/* valid utf8? */
+	if (g_utf8_validate (*text, *len, 0))
+		return NULL;
+
+	/* maybe it's iso-8859-1 */
+	utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, 0);
+	if (!utf)       /* should never happen; all text is iso-8859-1 valid */
+	{
+		*text = g_strdup ("%INVALID%");
+		*len = 8;
+	} else
+	{
+		*text = utf;
+		*len = utf_len;
+	}
+
+	return utf;
+}
+
 void
 PrintText (session *sess, unsigned char *text)
 {
+	char *conv;
+
 	if (!sess)
 	{
 		if (!sess_list)
@@ -304,6 +434,17 @@ PrintText (session *sess, unsigned char *text)
 		sess = (session *) sess_list->data;
 	}
 
+	/* make sure it's valid utf8 */
+	if (text[0] == 0)
+	{
+		text = "\n";
+		conv = NULL;
+	} else
+	{
+		int len = -1;
+		conv = text_validate ((char **)&text, &len);
+	}
+
 	log_write (sess, text);
 
 #ifdef USE_HEBREW
@@ -326,6 +467,9 @@ PrintText (session *sess, unsigned char *text)
 #ifdef USE_HEBREW
 	}
 #endif
+
+	if (conv)
+		g_free (conv);
 }
 
 void
@@ -956,13 +1100,13 @@ pevent_find (char *name, int *i_i)
 	{
 		if (j == NUM_XP)
 			j = 0;
-		if (j == i)
-			return -1;
 		if (strcmp (te[j].name, name) == 0)
 		{
 			*i_i = j;
 			return j;
 		}
+		if (j == i)
+			return -1;
 		j++;
 	}
 }
@@ -1014,7 +1158,7 @@ pevent_load (char *filename)
 
 		if (strcmp (buf, "event_name") == 0)
 		{
-			if (penum)
+			if (penum >= 0)
 				pevent_trigger_load (&penum, &text, &snd);
 			penum = pevent_find (ofs, &i);
 			continue;
@@ -1323,24 +1467,6 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
-static void
-play_wave (char *file)
-{
-	char buf[512];
-
-	snprintf (buf, sizeof (buf), "%s/%s", prefs.sounddir, file);
-	if (access (buf, R_OK) == 0)
-	{
-		snprintf (buf, sizeof (buf), "%s %s/%s", prefs.soundcmd, prefs.sounddir, file);
-		buf[sizeof (buf) - 1] = '\0';
-		xchat_exec (buf);
-	} else
-	{
-		snprintf (buf, sizeof (buf), "Cannot read %s/%s", prefs.sounddir, file);
-		fe_message (buf, FALSE);
-	}
-}
-
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1361,7 +1487,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 		return;
 
 	if (te[index].sound)
-		play_wave (te[index].sound);
+		fe_play_wave (te[index].sound);
 
 	display_event (pntevts[index], sess, te[index].num_args, word);
 }
