@@ -21,18 +21,21 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #ifdef WIN32
 #include <sys/timeb.h>
 #include <winsock.h>
 #include <process.h>
 #else
 #include <sys/time.h>
-#include <fcntl.h>
-#include <dirent.h>
 #include <sys/utsname.h>
 #endif
+#include <fcntl.h>
+#include <dirent.h>
 #include <errno.h>
 #include "xchat.h"
+#include "xchatc.h"
 #include <ctype.h>
 #include "util.h"
 #include "../../config.h"
@@ -1216,3 +1219,170 @@ const unsigned char rfc_tolowertab[] =
 	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
 	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };*/
+
+/* Code borrowed from gtk-gnutella */
+/* Takes care of moving a file from a temporary download location to a completed location */
+void
+download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir, 
+	char *output_name, int dccpermissions)
+{
+	/* Move a complete file to move_file_path */
+
+	char dl_src[4096];
+	char dl_dest[4096];
+	char dl_tmp[4096];
+	int return_tmp, return_tmp2;
+	struct stat buf;
+
+	/* if dcc_dir and dcc_completed_dir are the same then we are done */
+	if (0 == strcmp (dcc_dir, dcc_completed_dir) ||
+		 0 == dcc_completed_dir[0])
+		return;			/* Already in "completed dir" */
+
+	snprintf (dl_src, sizeof (dl_src), "%s/%s", dcc_dir, output_name);
+	snprintf (dl_dest, sizeof (dl_dest), "%s/%s", dcc_completed_dir,
+			   output_name);
+
+	dl_src[sizeof(dl_src)-1] = '\0';
+	dl_dest[sizeof(dl_dest)-1] = '\0';
+
+	/*
+	 * If, by extraordinary, there is already a file in the "completed dir"
+	 * with the same name, don't overwrite the existing file.
+	 *
+	 * NB: we assume either there is only one gnutella servent running, or if
+	 * several ones are running, that they are configured to use different
+	 * download and completed dirs.
+	 *
+	 *		--RAM, 03/11/2001
+	 */
+
+	if (-1 != stat (dl_dest, &buf))
+	{
+		int destlen = strlen (dl_dest);
+		int i;
+
+		/*
+		 * There must be enough room for us to append the ".xx" extensions.
+		 * That's 3 chars, plus the trailing NUL.
+		 */
+
+		if (destlen >= sizeof (dl_dest) - 4)
+		{
+			fprintf(stderr, "Found '%s' in completed dir, and path already too long",
+				output_name);
+			return;
+		}
+
+		strncpy (dl_tmp, dl_dest, destlen);
+
+		for (i = 1; i < 100; i++)
+		{
+			char ext[4];
+
+			snprintf (ext, 4, ".%02d", i);
+			dl_tmp[destlen] = '\0';				/* Ignore prior attempt */
+			strncat (dl_tmp+destlen, ext, 3);	/* Append .01, .02, ...*/
+			if (-1 == stat (dl_tmp, &buf))
+				break;
+		}
+
+		if (i == 100)
+		{
+			fprintf (stderr, "Found '%s' in completed dir, "
+				"and was unable to find another unique name",
+				output_name);
+			return;
+		}
+
+		strncat (dl_dest+destlen, dl_tmp+destlen, 3);
+	}
+
+	/* First try and link it to the new locatation */
+
+	return_tmp = rename (dl_src, dl_dest);
+
+	if (return_tmp == -1 && (errno == EXDEV || errno == EPERM))
+	{
+		/* link failed becase either the two paths aren't on the */
+		/* same filesystem or the filesystem doesn't support hard */
+		/* links, so we have to do a copy. */
+
+		int tmp_src, tmp_dest;
+		gboolean ok = FALSE;
+
+		if ((tmp_src = open (dl_src, O_RDONLY | OFLAGS)) == -1)
+		{
+			fprintf (stderr, "Unable to open() file '%s' (%s) !", dl_src,
+					  g_strerror (errno));
+			return;
+		}
+
+		if ((tmp_dest =
+			 open (dl_dest, O_WRONLY | O_CREAT | O_TRUNC | OFLAGS, dccpermissions)) < 0)
+		{
+			close (tmp_src);
+			fprintf (stderr, "Unable to create file '%s' (%s) !", dl_src,
+					  strerror (errno));
+			return;
+		}
+
+		for (;;)
+		{
+			return_tmp = read (tmp_src, dl_tmp, sizeof (dl_tmp));
+
+			if (!return_tmp)
+			{
+				ok = TRUE;
+				break;
+			}
+
+			if (return_tmp < 0)
+			{
+				fprintf (stderr, "download_move_to_completed_dir(): "
+					"error reading while moving file to save directory (%s)",
+					 strerror (errno));
+				break;
+			}
+
+			return_tmp2 = write (tmp_dest, dl_tmp, return_tmp);
+
+			if (return_tmp2 < 0)
+			{
+				fprintf (stderr, "download_move_to_completed_dir(): "
+					"error writing while moving file to save directory (%s)",
+					 strerror (errno));
+				break;
+			}
+
+			if (return_tmp < sizeof(dl_tmp))
+			{
+				ok = TRUE;
+				break;
+			}
+		}
+
+		close (tmp_dest);
+		close (tmp_src);
+		if (ok)
+			unlink (dl_src);
+	}
+}
+
+void
+play_wave (const char *file)
+{
+	char buf[512];
+
+	snprintf (buf, sizeof (buf), "%s/%s", prefs.sounddir, file);
+	if (access (buf, R_OK) == 0)
+	{
+		snprintf (buf, sizeof (buf), "%s %s/%s", prefs.soundcmd, prefs.sounddir, file);
+		buf[sizeof (buf) - 1] = '\0';
+		xchat_exec (buf);
+	} else
+	{
+		snprintf (buf, sizeof (buf), "Cannot read %s/%s", prefs.sounddir, file);
+		fe_message (buf, FALSE);
+	}
+}
