@@ -37,14 +37,15 @@
 #include "util.h"
 #include "cfgfiles.h"
 #include "ignore.h"
+#include "xchat-plugin.h"
 #include "plugin.h"
+#include "plugin-timer.h"
 #include "notify.h"
 #include "server.h"
 #include "servlist.h"
 #include "outbound.h"
 #include "text.h"
 #include "url.h"
-#include "plugin-timer.h"
 #include "xchatc.h"
 
 #ifdef USE_OPENSSL
@@ -77,7 +78,6 @@ struct xchatprefs prefs;
 
 #ifdef USE_OPENSSL
 SSL_CTX *ctx = NULL;
-void ssl_cb_info (SSL * s, int where, int ret);
 #endif
 
 
@@ -104,7 +104,7 @@ find_dialog (server *serv, char *nick)
 		sess = list->data;
 		if (sess->server == serv && sess->type == SESS_DIALOG)
 		{
-			if (!rfc_casecmp (nick, sess->channel))
+			if (!serv->p_cmp (nick, sess->channel))
 				return (sess);
 		}
 		list = list->next;
@@ -122,7 +122,7 @@ find_channel (server *serv, char *chan)
 		sess = list->data;
 		if ((!serv || serv == sess->server) && sess->type != SESS_DIALOG)
 		{
-			if (!rfc_casecmp (chan, sess->channel))
+			if (!serv->p_cmp (chan, sess->channel))
 				return sess;
 		}
 		list = list->next;
@@ -272,6 +272,11 @@ set_server_defaults (server *serv)
 		free (serv->nick_prefixes);
 	if (serv->nick_modes)
 		free (serv->nick_modes);
+	if (serv->encoding)
+	{
+		free (serv->encoding);
+		serv->encoding = NULL;
+	}
 
 	serv->chantypes = strdup ("#&!+");
 	serv->chanmodes = strdup ("beI,k,l");
@@ -329,9 +334,13 @@ new_ircwindow (server *serv, char *name, int type)
 		serv->server_session = sess;
 		serv->front_session = sess;
 		break;
+	case SESS_DIALOG:
+		sess = new_session (serv, name, type);
+		if (prefs.logging)
+			log_open (sess);
+		break;
 	default:
 /*	case SESS_CHANNEL:
-	case SESS_DIALOG:
 	case SESS_NOTICES:
 	case SESS_SNOTICES:*/
 		sess = new_session (serv, name, type);
@@ -339,6 +348,7 @@ new_ircwindow (server *serv, char *name, int type)
 	}
 
 	irc_init (sess);
+	plugin_emit_dummy_print (sess, "Open Context");
 
 	return sess;
 }
@@ -395,8 +405,6 @@ kill_server_callback (server * serv)
 {
 	serv->cleanup (serv);
 
-/*	fe_server_callback (serv);*/
-
 	serv_list = g_slist_remove (serv_list, serv);
 
 	dcc_notify_kill (serv);
@@ -413,7 +421,17 @@ kill_server_callback (server * serv)
 		free (serv->last_away_reason);
 	if (serv->eom_cmd)
 		free (serv->eom_cmd);
-	free (serv->gui);
+	if (serv->username)
+		free (serv->username);
+	if (serv->realname)
+		free (serv->realname);
+	if (serv->networkname)
+		free (serv->networkname);
+	if (serv->encoding)
+		free (serv->encoding);
+
+	fe_server_callback (serv);
+
 	free (serv);
 
 	notify_cleanup ();
@@ -525,8 +543,6 @@ kill_session_callback (session * killsess)
 	if (killsess->type == SESS_CHANNEL)
 		free_userlist (killsess);
 
-	fe_session_callback (killsess);
-
 	exec_notify_kill (killsess);
 
 	log_close (killsess);
@@ -534,12 +550,16 @@ kill_session_callback (session * killsess)
 	send_quit_or_part (killsess);
 
 	history_free (&killsess->history);
-	free (killsess->gui);
-	free (killsess->res);
 	if (killsess->topic)
 		free (killsess->topic);
 	if (killsess->current_modes)
 		free (killsess->current_modes);
+
+	fe_session_callback (killsess);
+
+	if (current_sess == killsess && sess_list)
+		current_sess = sess_list->data;
+
 	free (killsess);
 
 	if (!sess_list && !in_xchat_exit)
@@ -577,14 +597,14 @@ find_away_message (struct server *serv, char *nick)
 	while (list)
 	{
 		away = (struct away_msg *) list->data;
-		if (away->server == serv && !rfc_casecmp (nick, away->nick))
+		if (away->server == serv && !serv->p_cmp (nick, away->nick))
 			return away;
 		list = list->next;
 	}
 	return 0;
 }
 
-#define XTERM "rxvt"
+#define XTERM "gnome-terminal"
 
 #define defaultconf_ctcp \
 	"NAME TIME\n"				"CMD nctcp %s TIME %t\n\n"\
@@ -646,15 +666,7 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Ping\n"			"CMD !"XTERM" -e /bin/sh -c \"ping -c 4 %h ; sleep 30\"\n\n"\
 		"NAME Telnet\n"		"CMD !"XTERM" -e telnet %h\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
-	"NAME Open Query\n"		"CMD query %s\n\n"
-
-#define defaultconf_buttons \
-	"NAME Op\n"					"CMD op %a\n\n"\
-	"NAME DeOp\n"				"CMD deop %a\n\n"\
-	"NAME Ban\n"				"CMD ban %s\n\n"\
-	"NAME Kick\n"				"CMD kick %s\n\n"\
-	"NAME Sendfile\n"			"CMD dcc send %s\n\n"\
-	"NAME Dialog\n"			"CMD query %s\n\n"
+	"NAME Open Dialog Window\n"		"CMD query %s\n\n"
 
 #define defaultconf_dlgbuttons \
 	"NAME Whois\n"				"CMD whois %s\n\n"\
@@ -696,34 +708,6 @@ find_away_message (struct server *serv, char *nick)
 	"NAME WALLOPS\n"		"CMD quote WALLOPS :&2\n\n"\
 	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n"
 
-#define defaultconf_usermenu \
-	"NAME SUB\n"						"CMD IRC Stuff\n\n"\
-		"NAME Disconnect\n"			"CMD discon\n\n"\
-		"NAME Reconnect\n"			"CMD reconnect\n\n"\
-		"NAME Part Channel\n"		"CMD part\n\n"\
-		"NAME Cycle Channel\n"		"CMD cycle\n\n"\
-		"NAME Server Map\n"			"CMD quote MAP\n\n"\
-		"NAME Server Links\n"		"CMD quote LINKS\n\n"\
-		"NAME Ping Server\n"			"CMD ping\n\n"\
-	"NAME ENDSUB\n"					"CMD \n\n"\
-	"NAME SUB\n"						"CMD Connect\n\n"\
-		"NAME irc.xchat.org #Linux\n""CMD servchan irc.xchat.org 6667 #linux\n\n"\
- 		"NAME Go to EFNet\n"			"CMD newserver irc.efnet.net\n\n"\
-	"NAME ENDSUB\n"					"CMD \n\n"\
-	"NAME SUB\n"						"CMD Settings\n\n"\
-		"NAME TOGGLE Hide Version\n""CMD irc_hide_version\n\n"\
-		"NAME TOGGLE Colored Nicks\n""CMD text_color_nicks\n\n"\
-		"NAME TOGGLE 1.4.x Nick Comp.\n""CMD completion_old\n\n"\
-		"NAME TOGGLE Strip mIRC color\n""CMD text_stripcolor\n\n"\
-		"NAME TOGGLE Filter Beeps\n""CMD input_filter_beep\n\n"\
-		"NAME TOGGLE Raw MODE Display\n""CMD irc_raw_modes\n\n"\
-		"NAME TOGGLE Perl Warnings\n""CMD perl_warnings\n\n"\
-	"NAME ENDSUB\n"					"CMD \n\n"\
-	"NAME SUB\n"						"CMD External\n\n"\
-		"NAME Run XMMS\n"				"CMD !xmms\n\n"\
-		"NAME Run RXVT\n"				"CMD !"XTERM"\n\n"\
-	"NAME ENDSUB\n"					"CMD \n\n"
-
 #ifdef WIN32
 #define defaultconf_urlhandlers \
 	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
@@ -787,9 +771,19 @@ sighup_handler (int signal)
 
 #endif
 
+static gint
+xchat_auto_connect (gpointer userdata)
+{
+	servlist_auto_connect (NULL);
+	return 0;
+}
+
 static void
 xchat_init (void)
 {
+	char buf[512];
+	const char *cs = NULL;
+
 #ifdef WIN32
 	WSADATA wsadata;
 
@@ -807,38 +801,78 @@ xchat_init (void)
 	signal (SIGUSR1, sighup_handler);
 #endif
 
+	if (g_get_charset (&cs))
+		prefs.utf8_locale = TRUE;
+
 	load_text_events ();
 	notify_load ();
 	ignore_load ();
+
+	snprintf (buf, sizeof (buf),
+		"NAME %s\n"				"CMD discon\n\n"
+		"NAME %s\n"				"CMD reconnect\n\n"
+		"NAME %s\n"				"CMD part\n\n"
+		"NAME %s\n"				"CMD getstr # join \"%s\"\n\n"
+		"NAME %s\n"				"CMD quote LINKS\n\n"
+		"NAME %s\n"				"CMD ping\n\n"
+		"NAME TOGGLE %s\n"	"CMD irc_hide_version\n\n",
+				_("Disconnect"),
+				_("Reconnect"),
+				_("Leave Channel"),
+				_("Join Channel..."),
+				_("Enter Channel to Join:"),
+				_("Server Links"),
+				_("Ping Server"),
+				_("Hide Version"));
+	list_loadconf ("usermenu.conf", &usermenu_list, buf);
+
+	snprintf (buf, sizeof (buf),
+		"NAME %s\n"		"CMD op %%a\n\n"
+		"NAME %s\n"		"CMD deop %%a\n\n"
+		"NAME %s\n"		"CMD ban %%s\n\n"
+		"NAME %s\n"		"CMD getstr %s \"kick %%s\" \"%s\"\n\n"
+		"NAME %s\n"		"CMD dcc send %%s\n\n"
+		"NAME %s\n"		"CMD query %%s\n\n",
+				_("Op"),
+				_("DeOp"),
+				_("Ban"),
+				_("Kick"),
+				_("bye"),
+				_("Enter reason to kick %s:"),
+				_("Sendfile"),
+				_("Dialog"));
+	list_loadconf ("buttons.conf", &button_list, buf);
+
 	list_loadconf ("popup.conf", &popup_list, defaultconf_popup);
 	list_loadconf ("ctcpreply.conf", &ctcp_list, defaultconf_ctcp);
-	list_loadconf ("buttons.conf", &button_list, defaultconf_buttons);
 	list_loadconf ("dlgbuttons.conf", &dlgbutton_list, defaultconf_dlgbuttons);
 	list_loadconf ("commands.conf", &command_list, defaultconf_commands);
 	list_loadconf ("replace.conf", &replace_list, defaultconf_replace);
-	list_loadconf ("usermenu.conf", &usermenu_list, defaultconf_usermenu);
 	list_loadconf ("urlhandlers.conf", &urlhandler_list,
 						defaultconf_urlhandlers);
 
-#ifdef USE_TRANS
-	if (prefs.use_trans)
-	{
-		if (load_trans_table (prefs.trans_file) == 0)
-			prefs.use_trans = 0;
-	}
-#endif
-
 	servlist_init ();							/* load server list */
 
 	if (!prefs.slist_skip)
 		fe_serverlist_open (NULL);
 
-	/* do any auto connects */
-	if (!servlist_auto_connect (NULL))	/* if no new windows open .. */
+	/* turned OFF via -a arg */
+	if (auto_connect)
+	{
+		/* do any auto connects */
+		if (!servlist_have_auto ())	/* if no new windows open .. */
+		{
+			/* and no serverlist gui ... */
+			if (prefs.slist_skip)
+				/* we'll have to open one. */
+				new_ircwindow (NULL, NULL, SESS_SERVER);
+		} else
+		{
+			fe_idle_add (xchat_auto_connect, NULL);
+		}
+	} else
 	{
-		/* and no serverlist gui ... */
 		if (prefs.slist_skip)
-			/* we'll have to open one. */
 			new_ircwindow (NULL, NULL, SESS_SERVER);
 	}
 }
