@@ -50,6 +50,8 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #ifdef WIN32						/* for win32 */
 #define OFLAGS O_BINARY
 #define sleep(t) _sleep(t*1000)
+#define strcasecmp stricmp
+#define strncasecmp strnicmp
 #include <direct.h>
 #define	F_OK	0
 #define	X_OK	1
@@ -109,7 +111,6 @@ struct xchatprefs
 	char quitreason[256];
 	char partreason[256];
 	char font_normal[FONTNAMELEN + 1];
-	char dialog_font_normal[FONTNAMELEN + 1];
 #ifdef USE_ZVT
 	char font_shell[FONTNAMELEN + 1];
 #endif
@@ -117,8 +118,8 @@ struct xchatprefs
 	char sounddir[PATHLEN + 1];
 	char soundcmd[PATHLEN + 1];
 	char background[PATHLEN + 1];
-	char background_dialog[PATHLEN + 1];
 	char dccdir[PATHLEN + 1];
+	char dcc_completed_dir[PATHLEN + 1];
 	char bluestring[300];
 	char dnsprogram[72];
 	char hostname[127];
@@ -141,10 +142,6 @@ struct xchatprefs
 	int tint_green;
 	int tint_blue;
 
-	int dialog_tint_red;
-	int dialog_tint_green;
-	int dialog_tint_blue;
-
 	int tabs_position;
 	int max_auto_indent;
 	int dcc_blocksize;
@@ -152,6 +149,10 @@ struct xchatprefs
 	int notify_timeout;
 	int dcctimeout;
 	int dccstalltimeout;
+	int dcc_global_max_get_cps;
+	int dcc_global_max_send_cps;
+	int dcc_max_get_cps;
+	int dcc_max_send_cps;
 	int mainwindow_left;
 	int mainwindow_top;
 	int mainwindow_width;
@@ -169,7 +170,6 @@ struct xchatprefs
 	unsigned int perc_color;
 	unsigned int perc_ascii;
 	unsigned int use_trans;
-	unsigned int nopanel;
 	unsigned int autosave;
 	unsigned int autodialog;
 	unsigned int autosave_url;
@@ -196,10 +196,6 @@ struct xchatprefs
 	unsigned int autoopendccchatwindow;
 	unsigned int transparent;
 	unsigned int tint;
-	unsigned int dialog_transparent;
-	unsigned int dialog_tint;
-	unsigned int dialog_width;
-	unsigned int dialog_height;
 	unsigned int stripcolor;
 	unsigned int timestamp;
 	unsigned int fastdccsend;
@@ -207,6 +203,7 @@ struct xchatprefs
 	unsigned int dcc_remove;
 	unsigned int slist_skip;
 	unsigned int slist_edit;
+	unsigned int slist_select;
 	unsigned int filterbeep;
 	unsigned int beepmsg;
 	unsigned int beepchans;
@@ -219,8 +216,6 @@ struct xchatprefs
 	unsigned int hilitenotify;
 	unsigned int hidever;
 	unsigned int ip_from_server;
-	unsigned int panelize_hide;
-	unsigned int panel_vbox;
 	unsigned int raw_modes;
 	unsigned int show_away_once;
 	unsigned int show_away_message;
@@ -231,18 +226,11 @@ struct xchatprefs
 	unsigned int style_namelistgad;
 	unsigned int style_inputbox;
 	unsigned int windows_as_tabs;
-#ifdef USE_JCODE
-	unsigned int kanji_conv;
-#endif
 	unsigned int indent_nicks;
 	unsigned int show_separator;
 	unsigned int thin_separator;
-	unsigned int dialog_indent_nicks;
-	unsigned int dialog_show_separator;
-	unsigned int treeview;
 	unsigned int auto_indent;
 	unsigned int wordwrap;
-	unsigned int dialog_wordwrap;
 	unsigned int throttle;
 	unsigned int fudgeservernotice;
 	unsigned int topicbar;
@@ -257,6 +245,7 @@ struct xchatprefs
 	unsigned int whois_on_notifyonline;
 	unsigned int persist_chans;
 	unsigned int wait_on_exit;
+	unsigned int utf8_locale;
 
 	unsigned int ctcp_number_limit;	/*flood */
 	unsigned int ctcp_time_limit;	/*seconds of floods */
@@ -306,8 +295,9 @@ typedef struct session
 
 	int userlisthidden;
 	int type;
-	int new_data:1;		/* new data avail? (red tab) */
+	int new_data:1;			/* new data avail? (purple tab) */
 	int nick_said:1;		/* your nick mentioned? (blue tab) */
+	int msg_said:1;			/* new msg available? (red tab) */
 	int ignore_date:1;
 	int ignore_mode:1;
 	int ignore_names:1;
@@ -329,7 +319,7 @@ typedef struct server
 	void (*flush_queue)(struct server *);
 	void (*auto_reconnect)(struct server *, int send_quit, int err);
 	/* irc protocol functions (in proto*.c) */
-	void (*p_inline)(struct server *, char *buf);
+	void (*p_inline)(struct server *, char *buf, int len);
 	void (*p_invite)(struct server *, char *channel, char *nick);
 	void (*p_cycle)(struct server *, char *channel, char *key);
 	void (*p_ctcp)(struct server *, char *to, char *msg);
@@ -357,6 +347,7 @@ typedef struct server
 	void (*p_ping)(struct server *, char *to, char *timestring);
 /*	void (*p_set_away)(struct server *);*/
 	int (*p_raw)(struct server *, char *raw);
+	int (*p_cmp)(char *s1, char *s2);
 
 	int port;
 	int sok;					/* is equal to sok4 or sok6 (the one we are using) */
@@ -373,7 +364,6 @@ typedef struct server
 	int childhandle;
 #endif
 	int iotag;
-	int bartag;
 	int recondelay_tag;				/* reconnect delay timeout */
 	char hostname[128];				/* real ip number */
 	char servername[128];			/* what the server says is its name */
@@ -384,6 +374,10 @@ typedef struct server
 	int pos;								/* current position in linebuf */
 	int nickcount;
 
+	char *username;
+	char *realname;
+	char *networkname;
+
 	char *chantypes;					/* for 005 numeric - free me */
 	char *chanmodes;					/* for 005 numeric - free me */
 	char *nick_prefixes;				/* e.g. "*@%+" */
@@ -414,6 +408,8 @@ typedef struct server
 	time_t ping_recv;					/* when we last got a ping reply */
 	time_t away_time;					/* when we were marked away */
 
+	char *encoding;					/* NULL for system */
+
 	int motd_skipped:1;
 	int connected:1;
 	int connecting:1;
@@ -429,7 +425,6 @@ typedef struct server
 	int dont_use_proxy:1;		/* to proxy or not to proxy */
 	int supports_watch:1;		/* supports the WATCH command */
 	int bad_prefix:1;				/* gave us a bad PREFIX= 005 number */
-	int use_utf8:1;				/* send and receive utf8? */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
 	int accept_invalid_cert:1;	  /* ignore result of server's cert. verify */
