@@ -54,11 +54,6 @@
 #include "tabs.h"
 #include "urlgrab.h"
 
-#ifdef USE_PANEL
-#include "panel.h"
-#include <applet-widget.h>
-#endif
-
 #ifdef USE_XLIB
 #include <gdk/gdkx.h>
 #include <gtk/gtkinvisible.h>
@@ -66,11 +61,6 @@
 
 
 GdkPixmap *channelwin_pix;
-GdkPixmap *dialogwin_pix;
-
-
-extern void open_server_list (GtkWidget *wid, gpointer sess, int dontshow);
-extern void serverlist_autoconnect (struct session *sess, int);
 
 
 #ifdef USE_XLIB
@@ -157,13 +147,6 @@ fe_args (int argc, char *argv[])
 			auto_connect = 0;
 			offset++;
 		}
-#ifdef USE_PANEL
-		if (!strcasecmp (argv[1+offset], "-n") || !strcasecmp (argv[1+offset], "--no-panel"))
-		{
-			nopanel = TRUE;
-			offset++;
-		}
-#endif
 	}
 
 	if (argc > 2 + offset)
@@ -182,22 +165,7 @@ fe_args (int argc, char *argv[])
 
 	gtk_set_locale ();
 
-#ifdef USE_PANEL
-	if (nopanel)
-		gnome_init_with_popt_table (argv[0], VERSION, argc, argv, options, 0, 0);
-	else
-	{
-		CORBA_Environment ev;
-		CORBA_exception_init (&ev);
-
-		gnome_CORBA_init_with_popt_table (argv[0], VERSION,
-						  &argc, argv, options, 0, 0,
-						  GNORBA_INIT_SERVER_FUNC, &ev);
-		CORBA_exception_free (&ev);
-	}
-#else
 	gtk_init (&argc, &argv);
-#endif
 
 #ifdef USE_XLIB
 /*	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);*/
@@ -213,11 +181,23 @@ GtkStyle *
 create_input_style (void)
 {
 	GtkStyle *style;
+	char buf[256];
 
 	style = gtk_style_new ();
-	/* FIXME: fontd leaked, bg/fg hide cursor */
+	/* FIXME: bg/fg causes hide cursor */
+	pango_font_description_free (style->font_desc);
 	style->font_desc = pango_font_description_from_string (prefs.font_normal);
-/*	style->text[GTK_STATE_NORMAL] = colors[18];
+
+	/* fall back */
+	if (pango_font_description_get_size (style->font_desc) == 0)
+	{
+		snprintf (buf, sizeof (buf), _("Failed to open font:\n\n%s"), prefs.font_normal);
+		gtkutil_simpledialog (buf);
+		pango_font_description_free (style->font_desc);
+		style->font_desc = pango_font_description_from_string ("sans 11");
+	}
+
+	/*style->text[GTK_STATE_NORMAL] = colors[18];
 	style->base[GTK_STATE_NORMAL] = colors[19];*/
 
 	return style;
@@ -231,21 +211,13 @@ fe_init (void)
 	pixmaps_init ();
 
 	channelwin_pix = pixmap_load_from_file (prefs.background);
-	dialogwin_pix = pixmap_load_from_file (prefs.background_dialog);
 	input_style = create_input_style ();
 }
 
 void
 fe_main (void)
 {
-#ifdef USE_PANEL
-	if (nopanel)
-		gtk_main ();
-	else
-		applet_widget_gtk_main ();
-#else
 	gtk_main ();
-#endif
 
 	/* sleep for 3 seconds so any QUIT messages are not lost. The  */
 	/* GUI is closed at this point, so the user doesn't even know! */
@@ -257,10 +229,6 @@ void
 fe_cleanup (void)
 {
 	palette_save ();
-
-#ifdef USE_PANEL
-	panel_cleanup ();
-#endif
 }
 
 void
@@ -284,7 +252,19 @@ fe_timeout_remove (int tag)
 void
 fe_new_window (session *sess)
 {
-	mg_changui_new (sess, NULL);
+	int tab = FALSE;
+
+	if (sess->type == SESS_DIALOG)
+	{
+		if (prefs.privmsgtab)
+			tab = TRUE;
+	} else
+	{
+		if (prefs.tabchannels)
+			tab = TRUE;
+	}
+
+	mg_changui_new (sess, NULL, tab);
 }
 
 void
@@ -315,6 +295,12 @@ fe_message (char *msg, int wait)
 	}
 }
 
+void
+fe_idle_add (void *func, void *data)
+{
+	g_idle_add (func, data);
+}
+
 void
 fe_input_remove (int tag)
 {
@@ -322,13 +308,13 @@ fe_input_remove (int tag)
 }
 
 int
-fe_input_add (int sok, int read, int write, int ex, void *func, void *data)
+fe_input_add (int sok, int flags, void *func, void *data)
 {
 	int tag, type = 0;
 	GIOChannel *channel;
 
 #ifdef WIN32
-	if (read == 3)
+	if (flags & FIA_FD)
 		channel = g_io_channel_win32_new_fd (sok);
 	else
 		channel = g_io_channel_win32_new_socket (sok);
@@ -336,11 +322,11 @@ fe_input_add (int sok, int read, int write, int ex, void *func, void *data)
 	channel = g_io_channel_unix_new (sok);
 #endif
 
-	if (read)
+	if (flags & FIA_READ)
 		type |= G_IO_IN | G_IO_HUP | G_IO_ERR;
-	if (write)
+	if (flags & FIA_WRITE)
 		type |= G_IO_OUT | G_IO_ERR;
-	if (ex)
+	if (flags & FIA_EX)
 		type |= G_IO_PRI;
 
 	tag = g_io_add_watch (channel, type, (GIOFunc) func, data);
@@ -354,7 +340,7 @@ fe_set_topic (session *sess, char *topic)
 {
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
-		gtk_entry_set_text (GTK_ENTRY (sess->gui->topic), topic);
+		gtk_entry_set_text (GTK_ENTRY (sess->gui->topic_entry), topic);
 		mg_set_topic_tip (sess);
 	} else
 	{
@@ -370,12 +356,7 @@ fe_set_hilight (struct session *sess)
 	if (sess->gui->is_tab)
 	{
 		sess->nick_said = TRUE;
-#ifdef USE_PANEL
-		if (sess->gui->panel_button)
-			gtk_widget_set_style (GTK_BIN (sess->gui->panel_button)->child,
-									 blue_style);
-#endif
-		tab_style (sess->gui->tab, blue_style);
+		tab_set_attrlist (sess->res->tab, nickseen_list);
 	}
 }
 
@@ -413,6 +394,7 @@ fe_update_channel_limit (struct session *sess)
 	sprintf (tmp, "%d", sess->limit);
 	fe_update_mode_entry (sess, sess->gui->limit_entry,
 								 &sess->res->limit_text, tmp);
+	fe_set_title (sess);
 }
 
 int
@@ -426,7 +408,7 @@ fe_is_chanwindow (struct server *serv)
 int
 fe_is_banwindow (struct session *sess)
 {
-   if (!sess->gui->banlist_window)
+   if (!sess->res->banlist_window)
      return 0;
    return 1;
 }
@@ -456,63 +438,20 @@ void
 fe_close_window (struct session *sess)
 {
 	if (sess->gui->is_tab)
-		gtk_widget_destroy (sess->gui->tab);
+		gtk_widget_destroy (sess->res->tab);
 	else
 		gtk_widget_destroy (sess->gui->window);
 }
 
-static int
-updatedate_bar (struct session *sess)
-{
-	static int type = 0;
-	static float pos = 0;
-
-	if (!is_session (sess))
-		return 0;
-
-	pos += 0.05;
-	if (pos >= 0.99)
-	{
-		if (type == 0)
-		{
-			type = 1;
-			gtk_progress_bar_set_orientation ((GtkProgressBar *) sess->gui->bar,
-														 GTK_PROGRESS_RIGHT_TO_LEFT);
-		} else
-		{
-			type = 0;
-			gtk_progress_bar_set_orientation ((GtkProgressBar *) sess->gui->bar,
-														 GTK_PROGRESS_LEFT_TO_RIGHT);
-		}
-		pos = 0.05;
-	}
-	gtk_progress_bar_set_fraction ((GtkProgressBar *) sess->gui->bar, pos);
-	return 1;
-}
-
 void
 fe_progressbar_start (session *sess)
 {
 	if (!sess->gui->is_tab || current_tab == sess)
-	{
-		sess->gui->bar = gtk_progress_bar_new ();
-		gtk_box_pack_start (GTK_BOX (sess->gui->nick_box), sess->gui->bar, 0, 0,
-								  0);
-		gtk_widget_show (sess->gui->bar);
-		sess->server->bartag = fe_timeout_add (50, updatedate_bar, sess);
-	} else
-	{
+	/* if it's the focused tab, create it for real! */
+		mg_progressbar_create (sess->gui);
+	else
+	/* otherwise just remember to create on when it gets focused */
 		sess->res->c_graph = TRUE;
-	}
-}
-
-void
-fe_progressbar_destroy (session *sess)
-{
-	fe_timeout_remove (sess->server->bartag);
-	if (GTK_IS_WIDGET (sess->gui->bar))
-		gtk_widget_destroy (sess->gui->bar);
-	sess->gui->bar = 0;
 }
 
 void
@@ -528,7 +467,7 @@ fe_progressbar_end (server *serv)
 		if (sess->server == serv)
 		{
 			if (sess->gui->bar)
-				fe_progressbar_destroy (sess);
+				mg_progressbar_destroy (sess->gui);
 			sess->res->c_graph = FALSE;
 		}
 		list = list->next;
@@ -538,34 +477,21 @@ fe_progressbar_end (server *serv)
 void
 fe_print_text (struct session *sess, char *text)
 {
-	int indent;
-
-	/*if (sess->type == SESS_DIALOG)
-		indent = prefs.dialog_indent_nicks;
-	else*/
-		indent = prefs.indent_nicks;
-
-	PrintTextRaw (sess->res->buffer, text, indent);
+	PrintTextRaw (sess->res->buffer, text, prefs.indent_nicks);
 
 	if (prefs.limitedtabhighlight && !sess->highlight_tab)
 		return;
 
 	sess->highlight_tab = FALSE;
 
-	if (!sess->new_data && !sess->nick_said && sess != current_tab)
-	{
-#ifdef USE_PANEL
-		if (sess->gui->panel_button)
-			gtk_widget_set_style (GTK_BIN (sess->gui->panel_button)->child,
-										 red_style);
-#endif
-	}
-
 	if (!sess->new_data && sess != current_tab &&
 		 sess->gui->is_tab && !sess->nick_said)
 	{
 		sess->new_data = TRUE;
-		tab_style (sess->gui->tab, red_style);
+		if (sess->msg_said)
+			tab_set_attrlist (sess->res->tab, newmsg_list);
+		else
+			tab_set_attrlist (sess->res->tab, newdata_list);
 	}
 }
 
@@ -584,7 +510,7 @@ static void
 fe_lastlog_foreach (GtkXText *xtext, unsigned char *text, void *data)
 {
 	fe_lastlog_info *info = data;
-	 
+
 	if (nocasestrstr (text, info->sstr))
 		PrintText (info->sess, text);
 }
@@ -603,7 +529,7 @@ fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
 		info.sess = lastlog_sess;
 		info.sstr = sstr;
 		
-		gtk_xtext_foreach (GTK_XTEXT (sess->gui->xtext), fe_lastlog_foreach, &info);
+		gtk_xtext_foreach (sess->res->buffer, fe_lastlog_foreach, &info);
 	}
 }
 
@@ -628,6 +554,9 @@ fe_set_lag (server *serv, int lag)
 	if (per > 1.0)
 		per = 1.0;
 
+	snprintf (tip, sizeof (tip) - 1, "%s%d.%ds",
+				 serv->lag_sent ? "+" : "", lag / 10, lag % 10);
+
 	while (list)
 	{
 		sess = list->data;
@@ -636,18 +565,15 @@ fe_set_lag (server *serv, int lag)
 			if (!sess->gui->is_tab || current_tab == sess)
 			{
 				if (sess->gui->lagometer)
-				{
 					gtk_progress_bar_set_fraction ((GtkProgressBar *) sess->gui->lagometer, per);
-				}
 				if (sess->gui->laginfo)
-				{
-					snprintf (tip, sizeof(tip) - 1, "%s%d.%ds",
-						serv->lag_sent ? "+" : "", lag / 10, lag % 10);
 					gtk_label_set_text ((GtkLabel *) sess->gui->laginfo, tip);
-				}
 			} else
 			{
 				sess->res->lag_value = per;
+				if (sess->res->lag_text)
+					free (sess->res->lag_text);
+				sess->res->lag_text = strdup (tip);
 			}
 		}
 		list = list->next;
@@ -671,22 +597,28 @@ fe_set_throttle (server *serv)
 		sess = list->data;
 		if (sess->server == serv)
 		{
+			snprintf (tbuf, sizeof (tbuf) - 1, "%d bytes", serv->sendq_len);
+
 			if (!sess->gui->is_tab || current_tab == sess)
 			{
 				if (sess->gui->throttlemeter)
-				{
 					gtk_progress_bar_set_fraction ((GtkProgressBar *) sess->gui->throttlemeter, per);
-				}
 				if (sess->gui->throttleinfo)
-				{
-					snprintf (tbuf, sizeof(tbuf) - 1, "%d bytes", serv->sendq_len);
 					gtk_label_set_text ((GtkLabel *) sess->gui->throttleinfo, tbuf);
-				}
 			} else
 			{
 				sess->res->queue_value = per;
+				if (sess->res->queue_text)
+					free (sess->res->queue_text);
+				sess->res->queue_text = strdup (tbuf);
 			}
 		}
 		list = list->next;
 	}
 }
+
+void
+fe_play_wave (const char *file)
+{
+    play_wave (file);
+}
