@@ -81,6 +81,13 @@ struct server_gui
 
 typedef struct restore_gui
 {
+	/* banlist stuff */
+	GtkWidget *banlist_window;
+	GtkWidget *banlist_clistBan;
+	GtkWidget *banlist_butRefresh;
+
+	GtkWidget *tab;		/* toggleButton */
+
 	/* information stored when this tab isn't front-most */
 	void *user_model;	/* for filling the GtkTreeView */
 	void *buffer;		/* xtext_Buffer */
@@ -90,26 +97,28 @@ typedef struct restore_gui
 	char *limit_text;
 	gfloat old_ul_value;	/* old userlist value (for adj) */
 	gfloat lag_value;	/* lag-o-meter */
+	char *lag_text;	/* lag-o-meter text */
 	gfloat queue_value; /* outbound queue meter */
+	char *queue_text;		/* outbound queue text */
+	struct User *myself;	/* it's me in the Userlist */
 	short flag_wid_state[NUM_FLAG_WIDS];
-	char prefix;	/* my prefix char. e.g. @ for ops */
 	unsigned int c_graph:1;	/* connecting graph, is there one? */
 } restore_gui;
 
 typedef struct session_gui
 {
 	GtkWidget
-		*tab,		/* toggleButton */
 		*xtext,
 		*vscrollbar,
 		*window,	/* toplevel */
-		*topic,
+		*topic_entry,
 		*note_book,
 		*main_vbox,	/* container in TOPLEVEL window */
 		*user_tree,	/* GtkTreeView */
 		*user_box,
 		*button_box_parent,
 		*button_box,	/* userlist buttons' box */
+		*dialogbutton_box,
 		*topicbutton_box,
 		*lagometer,
 		*laginfo,
@@ -130,10 +139,7 @@ typedef struct session_gui
 		*limit_entry,		  /* +l */
 		*key_entry;		  /* +k */
 
-	/* banlist stuff */
-	GtkWidget *banlist_window;
-	GtkWidget *banlist_clistBan;
-	GtkWidget *banlist_butRefresh;
+	int bartag;		/*connecting progressbar timeout */
 
 	unsigned int is_tab:1;	/* is tab or toplevel? */
 
