@@ -271,11 +271,13 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 	}
 
 	/* check if it's a return or enter */
-	if ((evt->keyval == GDK_Return) || (evt->keyval == GDK_KP_Enter))
+	/* ---handled by the "activate" signal in maingui.c */
+/*	if ((evt->keyval == GDK_Return) || (evt->keyval == GDK_KP_Enter))
 		mg_inputbox_cb (wid, sess);
 
 	if (evt->keyval == GDK_KP_Enter)
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (wid), "key_press_event");
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (wid), "key_press_event");*/
+
 	return 0;
 }
 
@@ -644,7 +646,7 @@ key_dialog_show ()
 
 	key_dialog =
 			  mg_create_generic_tab ("editkeys", _("X-Chat: Edit Key Bindings"),
-								TRUE, FALSE, key_dialog_close, NULL, 560, 330, &vbox);
+							TRUE, FALSE, key_dialog_close, NULL, 560, 330, &vbox, 0);
 
 	hbox = gtk_hbox_new (0, 2);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, 1, 1, 0);
@@ -744,6 +746,7 @@ key_dialog_show ()
 
 	wid = gtk_xtext_new (colors, 0);
 	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
+	gtk_xtext_set_tint (GTK_XTEXT (wid), prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (GTK_XTEXT (wid),
 									  channelwin_pix,
 									  prefs.transparent, prefs.tint);
@@ -1293,7 +1296,7 @@ static int
 key_action_move_tab_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	mg_move_tab (sess->gui->window, +1);
+	mg_move_tab (sess->res->tab, +1);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1301,7 +1304,7 @@ static int
 key_action_move_tab_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	mg_move_tab (sess->gui->window, -1);
+	mg_move_tab (sess->res->tab, -1);
 	return 2;						  /* -''- */
 }
 
