@@ -35,7 +35,6 @@
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtktooltips.h>
 #include <gtk/gtklabel.h>
-#include <gtk/gtkpixmap.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkspinbutton.h>
@@ -124,16 +123,21 @@ gtkutil_file_req_done (GtkWidget * wid, struct file_req *freq)
 
 	if (axs)
 	{
-		freq->callback (freq->userdata, freq->userdata2, file);
+		char *utf8_file;
+		/* convert to UTF8. It might be converted back to locale by
+			server.c's g_convert */
+		utf8_file = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+		freq->callback (freq->userdata, freq->userdata2, utf8_file);
+		g_free (utf8_file);
 	} else
 	{
-		free (file);
 		if (freq->write)
 			gtkutil_simpledialog (_("Cannot write to that file."));
 		else
 			gtkutil_simpledialog (_("Cannot read that file."));
 	}
 
+	free (file);
 	gtk_widget_destroy (freq->dialog);
 	free (freq);
 }
@@ -175,20 +179,22 @@ gtkutil_destroy (GtkWidget * igad, GtkWidget * dgad)
 static void
 gtkutil_get_str_response (GtkDialog *dialog, gint arg1, gpointer entry)
 {
-	void (*callback) (char *text, void *user_data);
+	void (*callback) (int cancel, char *text, void *user_data);
 	char *text;
 	void *user_data;
 
+	text = (char *) gtk_entry_get_text (GTK_ENTRY (entry));
+	callback = g_object_get_data (G_OBJECT (dialog), "cb");
+	user_data = g_object_get_data (G_OBJECT (dialog), "ud");
+
 	switch (arg1)
 	{
 	case GTK_RESPONSE_REJECT:
+		callback (TRUE, text, user_data);
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
 	case GTK_RESPONSE_ACCEPT:
-		text = (char *) gtk_entry_get_text (GTK_ENTRY (entry));
-		callback = g_object_get_data (G_OBJECT (dialog), "cb");
-		user_data = g_object_get_data (G_OBJECT (dialog), "ud");
-		callback (text, user_data);
+		callback (FALSE, text, user_data);
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
 	}
@@ -201,7 +207,7 @@ gtkutil_str_enter (GtkWidget *entry, GtkWidget *dialog)
 }
 
 void
-gtkutil_get_str (char *msg, char *def, void *callback, void *userdata)
+fe_get_str (char *msg, char *def, void *callback, void *userdata)
 {
 	GtkWidget *dialog;
 	GtkWidget *entry;
@@ -209,8 +215,8 @@ gtkutil_get_str (char *msg, char *def, void *callback, void *userdata)
 	GtkWidget *label;
 
 	dialog = gtk_dialog_new_with_buttons (msg, NULL, 0,
-										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 										GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 										NULL);
 	gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->vbox), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
@@ -236,33 +242,32 @@ gtkutil_get_str (char *msg, char *def, void *callback, void *userdata)
 	gtk_widget_show_all (dialog);
 }
 
-
-#if 0
-
 static void
 gtkutil_get_number_response (GtkDialog *dialog, gint arg1, gpointer spin)
 {
-	void (*callback) (int value, void *user_data);
+	void (*callback) (int cancel, int value, void *user_data);
 	int num;
 	void *user_data;
 
+	num = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
+	callback = g_object_get_data (G_OBJECT (dialog), "cb");
+	user_data = g_object_get_data (G_OBJECT (dialog), "ud");
+
 	switch (arg1)
 	{
 	case GTK_RESPONSE_REJECT:
+		callback (TRUE, num, user_data);
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
 	case GTK_RESPONSE_ACCEPT:
-		num = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
-		callback = g_object_get_data (G_OBJECT (dialog), "cb");
-		user_data = g_object_get_data (G_OBJECT (dialog), "ud");
-		callback (num, user_data);
+		callback (FALSE, num, user_data);
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
 	}
 }
 
 void
-gtkutil_get_number (char *title, char *msg, void *callback, void *userdata)
+fe_get_int (char *msg, int def, void *callback, void *userdata)
 {
 	GtkWidget *dialog;
 	GtkWidget *spin;
@@ -270,10 +275,12 @@ gtkutil_get_number (char *title, char *msg, void *callback, void *userdata)
 	GtkWidget *label;
 	GtkAdjustment *adj;
 
-	dialog = gtk_dialog_new_with_buttons (title, NULL, 0,
-										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+	dialog = gtk_dialog_new_with_buttons (msg, NULL, 0,
 										GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 										NULL);
+	gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->vbox), TRUE);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 	hbox = gtk_hbox_new (TRUE, 0);
 
 	g_object_set_data (G_OBJECT (dialog), "cb", callback);
@@ -285,6 +292,7 @@ gtkutil_get_number (char *title, char *msg, void *callback, void *userdata)
 	adj->upper = 1024;
 	adj->step_increment = 1;
 	gtk_adjustment_changed (adj);
+	gtk_spin_button_set_value ((GtkSpinButton*)spin, def);
 	gtk_box_pack_end (GTK_BOX (hbox), spin, 0, 0, 0);
 
 	label = gtk_label_new (msg);
@@ -298,11 +306,9 @@ gtkutil_get_number (char *title, char *msg, void *callback, void *userdata)
 	gtk_widget_show_all (dialog);
 }
 
-#endif
-
 GtkWidget *
 gtkutil_button (GtkWidget *box, char *stock, char *tip, void *callback,
-				 void *userdata, char *labeltext, GdkPixmap *pix, GdkBitmap *mask)
+				 void *userdata, char *labeltext)
 {
 	GtkWidget *wid, *img, *label, *bbox;
 
@@ -322,13 +328,6 @@ gtkutil_button (GtkWidget *box, char *stock, char *tip, void *callback,
 		gtk_widget_show (img);
 	}
 
-	if (pix)
-	{
-		img = gtk_pixmap_new (pix, mask);
-		gtk_container_add (GTK_CONTAINER (bbox), img);
-		gtk_widget_show (img);
-	}
-
 	if (labeltext)
 	{
 		label = gtk_label_new_with_mnemonic (labeltext);
