@@ -38,6 +38,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenubar.h>
 #include <gtk/gtkstock.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
@@ -67,10 +68,6 @@
 #include "userlistgui.h"
 #include "menu.h"
 
-#ifdef USE_ZVT
-#include <zvt/zvtterm.h>
-#endif
-
 static GSList *submenu_list;
 
 enum
@@ -94,6 +91,7 @@ struct mymenu
 	short type;
 	unsigned int state:1;
 	unsigned int activate:1;
+	guint key;
 };
 
 
@@ -115,7 +113,7 @@ goto_url (char *url)
 		g_free (moz);
 	} else
 	{
-		snprintf (tbuf, sizeof (tbuf), "netscape -remote 'openURL(%s)'", url);
+		snprintf (tbuf, sizeof (tbuf), "mozilla -remote 'openURL(%s)'", url);
 	}
 	xchat_exec (tbuf);
 #endif
@@ -124,17 +122,9 @@ goto_url (char *url)
 
 /* execute a userlistbutton/popupmenu command */
 
-/*static void
-shit (int val, char *ud)
-{
-	printf("%d: %s\n", val, ud);
-}*/
-
 static void
 nick_command (session * sess, char *cmd)
 {
-/*	gtkutil_get_number ("title", "Number to kill:", shit, "hi");*/
-  
 	if (*cmd == '!')
 		xchat_exec (cmd + 1);
 	else
@@ -152,7 +142,7 @@ nick_command_parse (session *sess, char *cmd, char *nick, char *allnick)
 
 	if (sess->type == SESS_DIALOG)
 	{
-		buf = (char *)gtk_entry_get_text (GTK_ENTRY (sess->gui->topic));
+		buf = (char *)(GTK_ENTRY (sess->gui->topic_entry)->text);
 		buf = strrchr (buf, '@');
 		if (buf)
 			host = buf + 1;
@@ -189,13 +179,21 @@ userlist_button_cb (GtkWidget * button, char *cmd)
 	if (strstr (cmd, "%a"))
 		using_allnicks = TRUE;
 
-	/* find number of selected rows */
-	nicks = userlist_selection_list (sess->gui->user_tree, &num_sel);
-
-	if (num_sel < 1)
+	if (sess->type == SESS_DIALOG)
 	{
-		nick_command_parse (sess, cmd, "", "");
-		return;
+		/* fake a selection */
+		nicks = malloc (sizeof (char *) * 2);
+		nicks[0] = sess->channel;
+		nicks[1] = NULL;
+	} else
+	{
+		/* find number of selected rows */
+		nicks = userlist_selection_list (sess->gui->user_tree, &num_sel);
+		if (num_sel < 1)
+		{
+			nick_command_parse (sess, cmd, "", "");
+			return;
+		}
 	}
 
 	/* create "allnicks" string */
@@ -290,7 +288,7 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 	return item;
 }
 
-void
+static void
 menu_quick_item_with_callback (void *callback, char *label, GtkWidget * menu,
 										 void *arg)
 {
@@ -389,21 +387,23 @@ menu_create (GtkWidget *menu, GSList *list, char *target)
 }
 
 static void
-menu_destroy (GtkObject *object, gpointer unused)
+menu_destroy (GtkObject *object, gpointer objtounref)
 {
 	gtk_widget_destroy (GTK_WIDGET (object));
+	if (objtounref)
+		g_object_unref (G_OBJECT (objtounref));
 }
 
 static void
-menu_popup (GtkWidget *menu, GdkEventButton *event)
+menu_popup (GtkWidget *menu, GdkEventButton *event, gpointer objtounref)
 {
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-							G_CALLBACK (menu_destroy), NULL);
+							G_CALLBACK (menu_destroy), objtounref);
 	if (event == NULL)
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 0);
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, 0);
 	else
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-							 event->button, event->time);
+							 0, event->time);
 }
 
 static char *str_copy = 0;		/* for all pop-up menus */
@@ -466,36 +466,28 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 		menu_create (menu, popup_list, NULL);
 	else
 		menu_create (menu, popup_list, str_copy);
-	menu_popup (menu, event);
+	menu_popup (menu, event, NULL);
 }
 
-void
+static void
 menu_showhide (void)
 {
 	session *sess;
 	GSList *list;
-	GtkWidget *menu;
 
 	list = sess_list;
 	while (list)
 	{
 		sess = list->data;
 
-		if (sess->gui->is_tab)
+		if (sess->gui->menu)
 		{
-			menu = main_menu;
-		} else
-		{
-			if (!sess->gui->menu)
-				goto cont;
-			menu = sess->gui->menu;
+			if (prefs.hidemenu)
+				gtk_widget_hide (sess->gui->menu);
+			else
+				gtk_widget_show (sess->gui->menu);
 		}
 
-		if (prefs.hidemenu)
-			gtk_widget_hide (menu);
-		else
-			gtk_widget_show (menu);
-cont:
 		list = list->next;
 	}
 }
@@ -525,14 +517,18 @@ void
 menu_middlemenu (session *sess, GdkEventButton *event)
 {
 	GtkWidget *menu, *away, *user;
+	GtkAccelGroup *accel_group;
 
-	menu = menu_create_main (FALSE, sess->server->is_away, &away, &user);
+	accel_group = gtk_accel_group_new ();
+	menu = menu_create_main (accel_group, FALSE, sess->server->is_away, &away, &user);
+
+	menu_quick_item (0, 0, menu, 1, 0);	/* sep */
 
 	menu_toggle_item (_("Menu Bar"), menu, menu_middle_cb, 0, !prefs.hidemenu);
 	menu_toggle_item (_("Topic Bar"), menu, menu_middle_cb, (void*)1, prefs.topicbar);
 	menu_toggle_item (_("User List"), menu, menu_middle_cb, (void*)2, !prefs.hideuserlist);
 
-	menu_popup (menu, event);
+	menu_popup (menu, event, accel_group);
 }
 
 #ifdef WIN32
@@ -547,12 +543,12 @@ void
 menu_urlmenu (GdkEventButton *event, char *url)
 {
 	GtkWidget *menu;
-	int i;
 	char *tmp;
 
 	if (str_copy)
 		free (str_copy);
 
+#if 0
 	/* replace all commas with %2c */
 	str_copy = malloc ((strlen (url) * 3) + 1);
 	i = 0;
@@ -575,6 +571,9 @@ menu_urlmenu (GdkEventButton *event, char *url)
 		url++;
 	}
 	str_copy[i] = 0;
+#else
+	str_copy = strdup (url);
+#endif
 
 	menu = gtk_menu_new ();
 	if (strlen (str_copy) > 51)
@@ -595,7 +594,7 @@ menu_urlmenu (GdkEventButton *event, char *url)
 #endif
 
 	menu_create (menu, urlhandler_list, str_copy);
-	menu_popup (menu, event);
+	menu_popup (menu, event, NULL);
 }
 
 static void
@@ -658,7 +657,7 @@ menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
 												 str_copy);
 	}
 
-	menu_popup (menu, event);
+	menu_popup (menu, event, NULL);
 }
 
 static void
@@ -670,164 +669,23 @@ menu_open_server_list (GtkWidget *wid, gpointer none)
 static void
 menu_settings (GtkWidget * wid, gpointer none)
 {
+	extern void setup_open (void);
 	setup_open ();
 }
 
-#ifdef USE_ZVT
-
-static int
-shell_exit (ZvtTerm * term, struct session *sess)
-{
-	gtk_widget_destroy (sess->gui->window);
-	return 0;
-}
-
 static void
-menu_shell_title (ZvtTerm * term, int type, char *newtitle, session * sess)
+menu_usermenu (void)
 {
-	switch (type)
-	{
-	case VTTITLE_WINDOW:
-	case VTTITLE_WINDOWICON:
-		gtk_entry_set_text (GTK_ENTRY (sess->gui->topic), newtitle);
-	}
+	editlist_gui_open (usermenu_list, _("X-Chat: User menu"), "usermenu",
+							 "usermenu.conf", 0);
 }
 
-#ifdef USE_XLIB
-#include <gdk/gdkx.h>
-#endif
-
-/*   30 31 32 33 34 35 36  37  30  31 32 33 34  35  36  37 */
-static const short color_conv[] =
-	{ 1, 4, 3, 5, 2, 6, 10, 15, 14, 7, 9, 8, 12, 13, 11, 0, 18, 19 };
-
-static gushort zvt_red[18];
-static gushort zvt_grn[18];
-static gushort zvt_blu[18];
-
-void
-menu_newshell_set_palette (session *sess)
-{
-	int i;
-
-	for (i = 17;; i--)
-	{
-		zvt_red[i] = colors[color_conv[i]].red;
-		zvt_grn[i] = colors[color_conv[i]].green;
-		zvt_blu[i] = colors[color_conv[i]].blue;
-		if (!i)
-			break;
-	}
-	zvt_term_set_color_scheme ((ZvtTerm *) sess->gui->xtext,
-										zvt_red, zvt_grn, zvt_blu);
-}
-
-static void
-menu_newshell_tab (GtkWidget * wid, struct session *sess)
-{
-	int oldb = prefs.chanmodebuttons;
-	int oldp = prefs.paned_userlist;
-	int oldu = prefs.use_server_tab;
-	char *shell, *name;
-	char buf[16];
-	ZvtTerm *zvt;
-
-	prefs.chanmodebuttons = prefs.paned_userlist = prefs.use_server_tab = 0;
-	sess = new_ircwindow (NULL, NULL, SESS_SHELL);
-	prefs.chanmodebuttons = oldb;
-	prefs.paned_userlist = oldp;
-	prefs.use_server_tab = oldu;
-
-	fe_set_title (sess);
-
-#ifdef USE_GNOME
-	shell = gnome_util_user_shell ();
-#else
-	shell = strdup (getenv ("SHELL"));
-#endif
-	name = strrchr (shell, '/');
-	if (!name)
-		name = shell;
-	else
-		name++;
-	safe_strcpy (sess->channel, name, CHANLEN);
-	fe_set_channel (sess);
-
-	gtk_widget_destroy (sess->gui->xtext);
-	gtk_widget_destroy (sess->gui->vscrollbar);
-
-	sess->gui->xtext = zvt_term_new ();
-	zvt = (ZvtTerm *) sess->gui->xtext;
-
-	zvt_term_set_blink (zvt, FALSE);
-	if (prefs.font_shell[0] == 0)
-		zvt_term_set_fonts (zvt, font_normal, NULL);
-	else
-		zvt_term_set_font_name (zvt, prefs.font_shell);
-	zvt_term_set_scrollback (zvt, 720);
-	zvt_term_set_scroll_on_keystroke (zvt, TRUE);
-	zvt_term_set_background (zvt, prefs.background, prefs.transparent, 0);
-	gtk_container_add (GTK_CONTAINER (sess->gui->leftpane), GTK_WIDGET (zvt));
-	gtk_widget_show (GTK_WIDGET (zvt));
-	menu_newshell_set_palette (sess);
-
-	sess->gui->vscrollbar = gtk_vscrollbar_new (zvt->adjustment);
-	gtk_box_pack_start (GTK_BOX (sess->gui->leftpane), sess->gui->vscrollbar,
-							  0, 0, 1);
-	show_and_unfocus (sess->gui->vscrollbar);
-
-	GTK_WIDGET_UNSET_FLAGS (sess->gui->topic, GTK_CAN_FOCUS);
-
-	if (sess->gui->userlistbox)
-	{
-		gtk_widget_destroy (sess->gui->userlistbox);
-		sess->gui->userlistbox = 0;
-	}
-
-	g_signal_connect (G_OBJECT (sess->gui->xtext), "child_died",
-							G_CALLBACK (shell_exit), sess);
-	g_signal_connect (G_OBJECT (sess->gui->xtext), "title_changed",
-							G_CALLBACK (menu_shell_title), sess);
-	gtk_widget_destroy (sess->gui->inputgad->parent);
-
-	sess->gui->inputgad = 0;
-	sess->gui->toolbox = 0;
-	gtk_widget_grab_focus (sess->gui->xtext);
-
-	switch (zvt_term_forkpty ((ZvtTerm *) sess->gui->xtext, 0))
-	{
-	case -1:
-		break;
-
-	case 0:
-#ifdef USE_XLIB
-		sprintf (buf, "%d",
-					(int) GDK_WINDOW_XWINDOW (GTK_WIDGET (sess->gui->xtext)->
-													  window));
-#endif
-#ifdef HAVE_SETENV				  /* solaris is lame */
-		setenv ("WINDOWID", buf, 1);
-		setenv ("COLORTERM", "zterm", 0);
-#endif
-		execl (shell, name, NULL);
-		_exit (127);
-	}
-
-	if (main_window)
-		gdk_window_set_back_pixmap (main_book->window, 0, 0);
-
-#ifdef USE_GNOME
-	g_free (shell);				/* FIXME: is this a race condition ???? */
-#else
-	free (shell);
-#endif
-}
-#endif
-
 static void
 usermenu_create (GtkWidget *menu)
 {
 	menu_create (menu, usermenu_list, "");
+	menu_quick_item (0, 0, menu, 1, 0);	/* sep */
+	menu_quick_item_with_callback (menu_usermenu, _("Edit This Menu..."), menu, 0);
 }
 
 static void
@@ -836,13 +694,6 @@ usermenu_destroy (GtkWidget * menu)
 	GList *items = ((GtkMenuShell *) menu)->children;
 	GList *next;
 
-	/* under gnome, the tearoff is also an item */
-#ifdef USE_GNOME
-	items = items->next->next->next;	/* don't destroy the 1st 3 items */
-#else
-	items = items->next->next;	  /* don't destroy the 1st 2 items */
-#endif
-
 	while (items)
 	{
 		next = items->next;
@@ -934,7 +785,7 @@ menu_autodccsend (GtkWidget * wid, gpointer none)
 #ifndef WIN32
 	if (prefs.autodccsend)
 	{
-		if (!strcasecmp ((char *)g_get_home_dir (), prefs.dccdir))
+		if (!strcmp ((char *)g_get_home_dir (), prefs.dccdir))
 		{
 			gtkutil_simpledialog (_("*WARNING*\n"
 										 "Auto accepting DCC to your home directory\n"
@@ -945,10 +796,16 @@ menu_autodccsend (GtkWidget * wid, gpointer none)
 #endif
 }
 
+static void
+menu_detach (GtkWidget * wid, gpointer none)
+{
+	mg_link_cb (NULL, NULL);
+}
+
 static void
 menu_close (GtkWidget * wid, gpointer none)
 {
-	fe_close_window (current_sess);
+	mg_x_click_cb (NULL, NULL);
 }
 
 static void
@@ -977,7 +834,6 @@ savebuffer_req_done (session *sess, void *arg2, char *file)
 		gtk_xtext_save (GTK_XTEXT (sess->gui->xtext), fh);
 		close (fh);
 	}
-	free (file);
 }
 
 static void
@@ -1047,6 +903,7 @@ menu_invisible (GtkWidget * wid, gpointer none)
 	}
 }
 
+#if 0
 static void
 menu_savedefault (GtkWidget * wid, gpointer none)
 {
@@ -1054,6 +911,7 @@ menu_savedefault (GtkWidget * wid, gpointer none)
 	if (save_config ())
 		gtkutil_simpledialog (_("Settings saved."));
 }
+#endif
 
 static void
 menu_chanlist (GtkWidget * wid, gpointer none)
@@ -1083,7 +941,6 @@ menu_pluginlist (void)
 
 #else
 
-#define menu_unloadallplugins 0
 #define menu_pluginlist 0
 #define menu_loadplugin 0
 
@@ -1159,13 +1016,6 @@ menu_rpopup (void)
 							 "replace.conf", 0);
 }
 
-static void
-menu_usermenu (void)
-{
-	editlist_gui_open (usermenu_list, _("X-Chat: User menu"), "usermenu",
-							 "usermenu.conf", 0);
-}
-
 static void
 menu_urlhandlers (void)
 {
@@ -1206,6 +1056,7 @@ menu_ctcpguiopen (void)
 							 "ctcpreply.conf", ctcp_help);
 }
 
+#if 0
 static void
 menu_reload (void)
 {
@@ -1215,6 +1066,7 @@ menu_reload (void)
 	fe_message (buf, FALSE);
 	free (buf);
 }
+#endif
 
 static void
 menu_autorejoin (GtkWidget *wid, gpointer none)
@@ -1249,11 +1101,13 @@ menu_autodccchat (GtkWidget *wid, gpointer none)
 	prefs.autodccchat = !prefs.autodccchat;
 }
 
+#if 0
 static void
 menu_saveexit (GtkWidget *wid, gpointer none)
 {
 	prefs.autosave = !prefs.autosave;
 }
+#endif
 
 static void
 menu_docs (GtkWidget *wid, gpointer none)
@@ -1261,11 +1115,11 @@ menu_docs (GtkWidget *wid, gpointer none)
 	goto_url ("http://xchat.org/docs.html");
 }
 
-static void
+/*static void
 menu_webpage (GtkWidget *wid, gpointer none)
 {
 	goto_url ("http://xchat.org");
-}
+}*/
 
 static void
 menu_dcc_recv_win (GtkWidget *wid, gpointer none)
@@ -1286,15 +1140,24 @@ menu_dcc_chat_win (GtkWidget *wid, gpointer none)
 }
 
 static struct mymenu mymenu[] = {
-	{N_("X-Chat"), 0, 0, M_NEWMENU, 0, 1},
-	{N_("Server List..."), menu_open_server_list, (char *)&pix_book, M_MENUPIX, 0, 1},
+	{N_("_X-Chat"), 0, 0, M_NEWMENU, 0, 1},
+	{N_("Server List..."), menu_open_server_list, (char *)&pix_book, M_MENUPIX, 0, 1, GDK_s},
 	{0, 0, 0, M_SEP, 0, 0},
-	{N_("New Server Tab..."), menu_newserver_tab, 0, M_MENU, 0, 1},
-	
-	{N_("New Server Window..."), menu_newserver_window, 0, M_MENU, 0, 1},
-	{N_("New Channel Tab..."), menu_newchannel_tab, 0, M_MENU, 0, 1},
-	{N_("New Channel Window..."), menu_newchannel_window, 0, M_MENU, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},	/* 7 */
+
+	{N_("New"), 0, GTK_STOCK_NEW, M_MENUSUB, 0, 1},
+	{N_("Server Tab..."), menu_newserver_tab, 0, M_MENU, 0, 1, GDK_t},
+	{N_("Channel Tab..."), menu_newchannel_tab, 0, M_MENU, 0, 1},
+	{N_("Server Window..."), menu_newserver_window, 0, M_MENU, 0, 1},
+	{N_("Channel Window..."), menu_newchannel_window, 0, M_MENU, 0, 1},
+	{0, 0, 0, M_END, 0, 0},
+	{0, 0, 0, M_SEP, 0, 0},
+
+#ifdef USE_PLUGIN
+	{N_("Load Plugin..."), menu_loadplugin, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 1},
+#else
+	{N_("Load Plugin..."), 0, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 0},
+#endif
+	{0, 0, 0, M_SEP, 0, 0},	/* 11 */
 #ifdef USE_ZVT
 	{N_("New Shell Tab..."), menu_newshell_tab, 0, M_MENU, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0},
@@ -1302,42 +1165,29 @@ static struct mymenu mymenu[] = {
 #else
 #define menuoffset 2
 #endif
-	{N_("Close"), menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 1},
+	{N_("Detach Tab"), menu_detach, GTK_STOCK_REDO, M_MENUSTOCK, 0, 1, GDK_I},
+	{N_("Close Tab"), menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 1, GDK_w},
 	{0, 0, 0, M_SEP, 0, 0},
-	{N_("Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 1},	/* 12 */
+	{N_("Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 1, GDK_q},	/* 17 */
 
-	{N_("Windows"), 0, 0, M_NEWMENU, 0, 1},
-	{N_("Channel List Window..."), menu_chanlist, 0, M_MENU, 0, 1},
-	{N_("File Send Window..."), menu_dcc_send_win, 0, M_MENU, 0, 1},
-	{N_("File Receive Window..."), menu_dcc_recv_win, 0, M_MENU, 0, 1},
-	{N_("Direct Chat Window..."), menu_dcc_chat_win, 0, M_MENU, 0, 1},
-	{N_("Raw Log Window..."), menu_rawlog, 0, M_MENU, 0, 1},
-	{N_("URL Grabber Window..."), url_opengui, 0, M_MENU, 0, 1},
-	{N_("Notify List Window..."), notify_opengui, 0, M_MENU, 0, 1},
-	{N_("Ignore Window..."), ignore_gui_open, 0, M_MENU, 0, 1},
-	{N_("Ban List Window..."), menu_banlist, 0, M_MENU, 0, 1},
-	{N_("ASCII Window..."), ascii_open, 0, M_MENU, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},
-	{N_("F_lush Buffer"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 1},
-	{N_("Search Buffer..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 1},
-	{N_("Save Buffer..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 1},	/* 27 */
-
-	{N_("User Modes"), 0, 0, M_NEWMENU, 0, 1},
+	{N_("_IRC"), 0, 0, M_NEWMENU, 0, 1},
 	{N_("Invisible"), menu_invisible, 0, M_MENUTOG, 1, 1},
 	{N_("Receive Wallops"), menu_wallops, 0, M_MENUTOG, 1, 1},
 	{N_("Receive Server Notices"), menu_servernotice, 0, M_MENUTOG, 1, 1},
 	{0, 0, 0, M_SEP, 0, 0},
-	{N_("Marked Away"), menu_away, 0, M_MENUTOG, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},														/* 34 */
-	{N_("Auto ReJoin on Kick"), menu_autorejoin, 0, M_MENUTOG, 0, 1},
-	{N_("Auto ReConnect to Server"), menu_autoreconnect, 0, M_MENUTOG, 0, 1},
+	{N_("Marked Away"), menu_away, 0, M_MENUTOG, 0, 1, GDK_a},
+	{0, 0, 0, M_SEP, 0, 0},														/* 24 */
+	{N_("Auto Rejoin when Kicked"), menu_autorejoin, 0, M_MENUTOG, 0, 1},
+	{N_("Auto Reconnect to Server"), menu_autoreconnect, 0, M_MENUTOG, 0, 1},
 	{N_("Never-give-up ReConnect"), menu_autoreconnectonfail, 0, M_MENUTOG, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},														/* 38 */
+	{0, 0, 0, M_SEP, 0, 0},														/* 28 */
 	{N_("Auto Open Dialog Windows"), menu_autodialog, 0, M_MENUTOG, 0, 1},
-	{N_("Auto Accept DCC Chat"), menu_autodccchat, 0, M_MENUTOG, 0, 1},
-	{N_("Auto Accept DCC Send"), menu_autodccsend, 0, M_MENUTOG, 0, 1},
+	{N_("Auto Accept Direct Chat"), menu_autodccchat, 0, M_MENUTOG, 0, 1},
+	{N_("Auto Accept Files"), menu_autodccsend, 0, M_MENUTOG, 0, 1},
 
-	{N_("Settings"), 0, 0, M_NEWMENU, 0, 1},	/* 43*/
+	{N_("_Server"), (void *) -1, 0, M_NEWMENU, 0, 1},	/* 32 */
+
+	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 1},	/* 33 */
 	{N_("Preferences..."), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 1},
 
 	{N_("Lists"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 1},
@@ -1349,37 +1199,42 @@ static struct mymenu mymenu[] = {
 	{N_("Replace Popup..."), menu_rpopup, 0, M_MENU, 0, 1},
 	{N_("URL Handlers..."), menu_urlhandlers, 0, M_MENU, 0, 1},
 	{N_("Event Texts..."), menu_evtpopup, 0, M_MENU, 0, 1},
-	{N_("Key Bindings..."), menu_keypopup, 0, M_MENU, 0, 1},
+	{N_("Key Bindings..."), menu_keypopup, 0, M_MENU, 0, 1},	/* 44 */
 	{0, 0, 0, M_END, 0, 0},
 
-	{0, 0, 0, M_SEP, 0, 0},	/* 55 */
+#if 0
+	{0, 0, 0, M_SEP, 0, 0},	/* 56 */
 	{N_("Reload Settings"), menu_reload, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0},
 	{N_("Save Settings now"), menu_savedefault, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 1},
 	{N_("Save Settings on exit"), menu_saveexit, 0, M_MENUTOG, 1, 1},
-	{N_("Plugins"), 0, 0, M_NEWMENU, 0, 1},	/* 60 */
-#ifdef USE_PLUGIN
-	{N_("Load Plugin..."), menu_loadplugin, 0, M_MENU, 0, 1},
-	{N_("Plugin List"), menu_pluginlist, 0, M_MENU, 0, 1},
-#else
-	{N_("Load Plugin..."), 0, 0, M_MENU, 0, 0},
-	{N_("Plugin List"), 0, 0, M_MENU, 0, 0},
 #endif
 
-	{N_("User Menu"), (void *) -1, 0, M_NEWMENU, 0, 1},
-	{N_("Edit User Menu"), menu_usermenu, 0, M_MENU, 0, 1},
+	{N_("_Window"), 0, 0, M_NEWMENU, 0, 1},
+	{N_("Channel List..."), menu_chanlist, 0, M_MENU, 0, 1},
+	{N_("File Send..."), menu_dcc_send_win, 0, M_MENU, 0, 1},
+	{N_("File Receive..."), menu_dcc_recv_win, 0, M_MENU, 0, 1},
+	{N_("Direct Chat..."), menu_dcc_chat_win, 0, M_MENU, 0, 1},
+	{N_("Raw Log..."), menu_rawlog, 0, M_MENU, 0, 1},	/* 51 */
+	{N_("URL Grabber..."), url_opengui, 0, M_MENU, 0, 1},
+	{N_("Notify List..."), notify_opengui, 0, M_MENU, 0, 1},
+	{N_("Ignore List..."), ignore_gui_open, 0, M_MENU, 0, 1},
+	{N_("Ban List..."), menu_banlist, 0, M_MENU, 0, 1},
+	{N_("Character Chart..."), ascii_open, 0, M_MENU, 0, 1},
+	{N_("Plugin List..."), menu_pluginlist, 0, M_MENU, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0},
+	{N_("C_lear Text"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 1, GDK_l},
+	{N_("Search Text..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 1, GDK_f},
+	{N_("Save Text..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 1},
 
-	{N_("Help"), 0, 0, M_NEWMENU, 0, 1},
-	{N_("X-Chat Homepage..."), menu_webpage, GTK_STOCK_HOME, M_MENUSTOCK, 0, 1},
-	{N_("Online Docs..."), menu_docs, GTK_STOCK_DND_MULTIPLE, M_MENUSTOCK, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},
-	{N_("About X-Chat..."), menu_about, GTK_STOCK_HELP, M_MENUSTOCK, 0, 1},
+	{N_("_Help"), 0, 0, M_NEWMENU, 0, 1},	/* 62 */
+	{N_("Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 1, GDK_F1},
+	{N_("About"), menu_about, 0, M_MENU, 0, 1},
 
 	{0, 0, 0, M_END, 0, 0},
 };
 
-static GtkWidget *
+GtkWidget *
 create_icon_menu (char *labeltext, void *stock_name, int is_stock)
 {
 	GtkWidget *item, *img;
@@ -1396,7 +1251,7 @@ create_icon_menu (char *labeltext, void *stock_name, int is_stock)
 }
 
 GtkWidget *
-menu_create_main (int bar, int away,
+menu_create_main (void *accel_group, int bar, int away,
 						GtkWidget **away_item_ret, GtkWidget **user_menu_ret)
 {
 	int i = 0;
@@ -1412,17 +1267,17 @@ menu_create_main (int bar, int away,
 	else
 		menu_bar = gtk_menu_new ();
 
-	mymenu[29-menuoffset].state = prefs.invisible;
-	mymenu[30-menuoffset].state = prefs.wallops;
-	mymenu[31-menuoffset].state = prefs.servernotice;
-	mymenu[33-menuoffset].state = away;
-	mymenu[35-menuoffset].state = prefs.autorejoin;
-	mymenu[36-menuoffset].state = prefs.autoreconnect;
-	mymenu[37-menuoffset].state = prefs.autoreconnectonfail;
-	mymenu[39-menuoffset].state = prefs.autodialog;
-	mymenu[40-menuoffset].state = prefs.autodccchat;
-	mymenu[41-menuoffset].state = prefs.autodccsend;
-	mymenu[59-menuoffset].state = prefs.autosave;
+	mymenu[19-menuoffset].state = prefs.invisible;
+	mymenu[20-menuoffset].state = prefs.wallops;
+	mymenu[21-menuoffset].state = prefs.servernotice;
+	mymenu[23-menuoffset].state = away;
+	mymenu[25-menuoffset].state = prefs.autorejoin;
+	mymenu[26-menuoffset].state = prefs.autoreconnect;
+	mymenu[27-menuoffset].state = prefs.autoreconnectonfail;
+	mymenu[29-menuoffset].state = prefs.autodialog;
+	mymenu[30-menuoffset].state = prefs.autodccchat;
+	mymenu[31-menuoffset].state = prefs.autodccsend;
+	/*mymenu[60-menuoffset].state = prefs.autosave;*/
 
 	while (1)
 	{
@@ -1434,7 +1289,7 @@ menu_create_main (int bar, int away,
 			menu = gtk_menu_new ();
 			if (mymenu[i].callback == (void *) -1)
 				usermenu = menu;
-			menu_item = gtk_menu_item_new_with_label (_(mymenu[i].text));
+			menu_item = gtk_menu_item_new_with_mnemonic (_(mymenu[i].text));
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);
 			gtk_widget_show (menu_item);
 			break;
@@ -1450,6 +1305,11 @@ menu_create_main (int bar, int away,
 		case M_MENU:
 			item = gtk_menu_item_new_with_label (_(mymenu[i].text));
 normalitem:
+			if (mymenu[i].key != 0)
+				gtk_widget_add_accelerator (item, "activate", accel_group,
+										mymenu[i].key,
+										mymenu[i].key == GDK_F1 ? 0 : GDK_CONTROL_MASK,
+										GTK_ACCEL_VISIBLE);
 			if (mymenu[i].callback)
 				g_signal_connect (G_OBJECT (item), "activate",
 										G_CALLBACK (mymenu[i].callback), 0);
@@ -1465,13 +1325,16 @@ normalitem:
 			item = gtk_check_menu_item_new_with_label (_(mymenu[i].text));
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
 													 mymenu[i].state);
+			if (mymenu[i].key != 0)
+				gtk_widget_add_accelerator (item, "activate", accel_group,
+									mymenu[i].key, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
 			if (mymenu[i].callback)
 				g_signal_connect (G_OBJECT (item), "toggled",
 										G_CALLBACK (mymenu[i].callback), 0);
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			gtk_widget_show (item);
 			gtk_widget_set_sensitive (item, mymenu[i].activate);
-			if (bar && i == 33 - menuoffset)
+			if (bar && i == 23 - menuoffset)
 				*away_item_ret = item;
 			break;
 
