@@ -1,15 +1,10 @@
 extern GdkPixbuf *pix_xchat;
 extern GdkPixbuf *pix_book;
-extern GdkPixbuf *pix_admin;
+extern GdkPixbuf *pix_purple;
+extern GdkPixbuf *pix_red;
 extern GdkPixbuf *pix_op;
 extern GdkPixbuf *pix_hop;
 extern GdkPixbuf *pix_voice;
 
-extern GdkPixmap *pix_globe;
-extern GdkPixmap *pix_server;
-
-extern GdkBitmap *mask_globe;
-extern GdkBitmap *mask_server;
-
 extern GdkPixmap *pixmap_load_from_file (char *file);
 extern void pixmaps_init (void);
