@@ -28,6 +28,9 @@
 #include <gtk/gtkbox.h>
 
 #include "../common/xchat.h"
+#define PLUGIN_C
+typedef struct session xchat_context;
+#include "../common/xchat-plugin.h"
 #include "../common/plugin.h"
 #include "../common/util.h"
 #include "../common/outbound.h"
@@ -51,6 +54,8 @@ plugingui_close (GtkWidget * wid, gpointer a)
 	plugin_window = NULL;
 }
 
+extern GSList *plugin_list;
+
 void
 fe_pluginlist_update (void)
 {
@@ -87,7 +92,6 @@ plugingui_load_cb (session *sess, void *data2, char *file)
 		char *buf = malloc (strlen (file) + 7);
 
 		sprintf (buf, "LOAD %s", file);
-		free (file);
 		handle_command (sess, buf, FALSE);
 		free (buf);
 	}
@@ -169,17 +173,17 @@ plugingui_open (void)
 
 	gtkutil_button (GTK_DIALOG (plugin_window)->action_area,
 						 GTK_STOCK_REVERT_TO_SAVED, NULL, plugingui_loadbutton_cb,
-						 NULL, _("_Load"), NULL, NULL);
+						 NULL, _("_Load..."));
 
 	gtkutil_button (GTK_DIALOG (plugin_window)->action_area,
 						 GTK_STOCK_DELETE, NULL, plugingui_unload,
-						 NULL, _("_UnLoad"), NULL, NULL);
+						 NULL, _("_UnLoad"));
 
 	okb = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
 	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (plugin_window)->action_area),
 							okb, 1, 1, 10);
-	gtk_signal_connect (GTK_OBJECT (okb), "clicked",
-							  GTK_SIGNAL_FUNC (plugingui_close_button), 0);
+	g_signal_connect (G_OBJECT (okb), "clicked",
+							G_CALLBACK (plugingui_close_button), 0);
 	gtk_widget_show (okb);
 
 	fe_pluginlist_update ();
