@@ -7,6 +7,7 @@
 
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcombo.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbox.h>
@@ -29,9 +30,10 @@
 
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
-#include "../pixmaps/xchat.h"
 #include "../common/servlist.h"
+#include "../common/cfgfiles.h"
 #include "gtkutil.h"
+#include "menu.h"
 #include "pixmaps.h"
 
 
@@ -42,6 +44,7 @@ static GtkWidget *servers_tree;	/* list of servers */
 static GtkWidget *networks_tree;	/* network TreeView */
 static GtkWidget *connect_button;
 static GtkWidget *connectnew_button;
+static int ignore_changed = FALSE;
 
 /* global user info */
 static GtkWidget *entry_nick1;
@@ -57,12 +60,14 @@ static GtkWidget *entry_real;
 static GtkWidget *entry_join;
 static GtkWidget *entry_pass;
 static GtkWidget *entry_cmd;
+static GtkWidget *combo_encoding;
 static GtkWidget *check_buttons[FLAG_COUNT];
 static GtkWidget *editbox;	/* shown/hidden by "Edit Mode" checkbutton */
 
 static ircnet *selected_net = NULL;
 static session *servlist_sess;
-static gboolean right_click = FALSE;
+
+static void servlist_network_row_cb (GtkTreeSelection *sel, gpointer user_data);
 
 
 static void
@@ -71,16 +76,35 @@ servlist_toggles_populate (ircnet *net)
 	int i;
 
 	for (i = 0; i < FLAG_COUNT; i++)
-	{
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_buttons[i]),
 												net->flags & (1 << i));
 
-	}
 }
 
+static const char *pages[]=
+{
+	"UTF-8",
+	"ISO-8859-1 (Western Europe)",
+	"ISO-8859-2 (Central Europe)",
+	"ISO-8859-7 (Greek)",
+	"ISO-8859-8 (Hebrew)",
+	"ISO-8859-9 (Turkish)",
+	"CP1251 (Cyrillic)",
+	"CP1256 (Arabic)",
+	"GB18030 (Chinese)",
+	"SJIS (Japanese)",
+	NULL
+};
+
 static void
 servlist_entries_populate (ircnet *net)
 {
+	static GList *cbitems = NULL;
+	int i;
+
+	/* avoid the "changed" callback */
+	ignore_changed = TRUE;
+
 	gtk_entry_set_text (GTK_ENTRY (entry_nick), net->nick ? net->nick : "");
 	gtk_entry_set_text (GTK_ENTRY (entry_user), net->user ? net->user : "");
 	gtk_entry_set_text (GTK_ENTRY (entry_real), net->real ? net->real : "");
@@ -88,6 +112,22 @@ servlist_entries_populate (ircnet *net)
 	gtk_entry_set_text (GTK_ENTRY (entry_join), net->autojoin ? net->autojoin : "");
 	gtk_entry_set_text (GTK_ENTRY (entry_cmd), net->command ? net->command : "");
 
+	if (cbitems == NULL)
+	{
+		cbitems = g_list_append (cbitems, "System default");
+		i = 0;
+		while (pages[i])
+		{
+			cbitems = g_list_append (cbitems, (char *)pages[i]);
+			i++;
+		}
+		gtk_combo_set_popdown_strings (GTK_COMBO (combo_encoding), cbitems);
+	}
+
+	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_encoding)->entry), net->encoding ? net->encoding : "System default");
+
+	ignore_changed = FALSE;
+
 	if (net->flags & FLAG_USE_GLOBAL)
 	{
 		gtk_widget_set_sensitive (entry_nick, FALSE);
@@ -106,6 +146,7 @@ servlist_servers_populate (ircnet *net, GtkWidget *treeview, gboolean edit)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
+	GtkTreePath *path;
 	int i;
 	ircserver *serv;
 	char buf[256];
@@ -114,7 +155,7 @@ servlist_servers_populate (ircnet *net, GtkWidget *treeview, gboolean edit)
 
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 
-	snprintf (buf, sizeof (buf), "Settings for %s", net->name);
+	snprintf (buf, sizeof (buf), _("Settings for %s"), net->name);
 	gtk_frame_set_label (GTK_FRAME (editbox), buf);
 
 	store = (GtkListStore *)gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
@@ -128,7 +169,15 @@ servlist_servers_populate (ircnet *net, GtkWidget *treeview, gboolean edit)
 		gtk_list_store_set (store, &iter, 0, serv->hostname, 1, edit, -1);
 
 		if (net->selected == i)
+		{
+			/* select this server */
 			gtk_tree_selection_select_iter (sel, &iter);
+			/* and make sure it's visible */
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+			gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path,
+													NULL, TRUE, 0.5, 0.5);
+			gtk_tree_path_free (path);
+		}
 		i++;
 
 		list = list->next;
@@ -140,11 +189,20 @@ servlist_networks_populate (GtkWidget *treeview, GSList *netlist, gboolean edit)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
+	GtkTreeSelection *sel;
+	GtkTreePath *path;
 	int i;
 	ircnet *net;
 
+	if (!netlist)
+	{
+		net = servlist_net_add (_("New Network"), "");
+		servlist_server_add (net, "newserver/6667");
+		netlist = network_list;
+	}
 	store = (GtkListStore *)gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
 	gtk_list_store_clear (store);
+	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 
 	i = 0;
 	while (netlist)
@@ -152,6 +210,16 @@ servlist_networks_populate (GtkWidget *treeview, GSList *netlist, gboolean edit)
 		net = netlist->data;
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter, 0, net->name, 1, edit, -1);
+		if (i == prefs.slist_select)
+		{
+			/* select this network */
+			gtk_tree_selection_select_iter (sel, &iter);
+			/* and make sure it's visible */
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+			gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path,
+													NULL, TRUE, 0.5, 0.5);
+			gtk_tree_path_free (path);
+		}
 		i++;
 		netlist = netlist->next;
 	}
@@ -164,7 +232,8 @@ servlist_populate (ircnet *net)
 	servlist_entries_populate (net);
 	servlist_toggles_populate (net);
 	gtk_widget_set_sensitive (connect_button, TRUE);
-	gtk_widget_set_sensitive (connectnew_button, TRUE);
+	if (connectnew_button)
+		gtk_widget_set_sensitive (connectnew_button, TRUE);
 	gtk_widget_set_sensitive (editbox, TRUE);
 }
 
@@ -184,25 +253,38 @@ servlist_addserver_cb (GtkWidget *item, GtkWidget *treeview)
 		return;
 
 	store = (GtkListStore *)gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
-	servlist_server_add (selected_net, _("NewServer"));
+	servlist_server_add (selected_net, "newserver/6667");
 
 	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("NewServer"), 1, 1, -1);
+	gtk_list_store_set (store, &iter, 0, "newserver/6667", 1, 1, -1);
 }
 
 static void
 servlist_addnet_cb (GtkWidget *item, GtkTreeView *treeview)
 {
+	GtkTreeSelection *sel;
 	GtkTreeIter iter;
 	GtkListStore *store;
+	GtkTreePath *path;
 	ircnet *net;
 
 	store = (GtkListStore *)gtk_tree_view_get_model (treeview);
 	net = servlist_net_add (_("New Network"), "");
-	servlist_server_add (net, _("NewServer"));
+	servlist_server_add (net, "newserver/6667");
 
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter, 0, net->name, 1, 1, -1);
+
+	/* select this server */
+	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (networks_tree));
+	gtk_tree_selection_select_iter (sel, &iter);
+	/* and make sure it's visible */
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path,
+											NULL, TRUE, 0.5, 0.5);
+	gtk_tree_path_free (path);
+
+	servlist_network_row_cb (gtk_tree_view_get_selection (GTK_TREE_VIEW (networks_tree)), NULL);
 }
 
 static void
@@ -223,7 +305,8 @@ servlist_deletenetdialog_cb (GtkDialog *dialog, gint arg1, ircnet *net)
 
 		/* don't allow user to play with freed memory */
 		gtk_widget_set_sensitive (connect_button, FALSE);
-		gtk_widget_set_sensitive (connectnew_button, FALSE);
+		if (connectnew_button)
+			gtk_widget_set_sensitive (connectnew_button, FALSE);
 		gtk_widget_set_sensitive (editbox, FALSE);
 		store = (GtkListStore *)gtk_tree_view_get_model (GTK_TREE_VIEW (servers_tree));
 		gtk_list_store_clear (store);
@@ -233,9 +316,45 @@ servlist_deletenetdialog_cb (GtkDialog *dialog, gint arg1, ircnet *net)
 
 		if (selected_net == net)
 			selected_net = NULL;
+
+		if (!network_list)
+		{
+			servlist_networks_populate (networks_tree, network_list, prefs.slist_edit);
+		}
 	}
 }
 
+static void
+servlist_move_network (ircnet *net, int delta)
+{
+	int pos;
+
+	pos = g_slist_index (network_list, net);
+	if (pos >= 0)
+	{
+		pos += delta;
+		if (pos >= 0)
+		{
+			prefs.slist_select += delta;
+			network_list = g_slist_remove (network_list, net);
+			network_list = g_slist_insert (network_list, net, pos);
+			servlist_networks_populate (networks_tree, network_list, FALSE);
+		}
+	}
+}
+
+static void
+servlist_movedownnet_cb (GtkWidget *item, ircnet *net)
+{
+	servlist_move_network (net, +1);
+}
+
+static void
+servlist_moveupnet_cb (GtkWidget *item, ircnet *net)
+{
+	servlist_move_network (net, -1);
+}
+
 static void
 servlist_deletenet_cb (GtkWidget *item, ircnet *net)
 {
@@ -275,7 +394,7 @@ servlist_deleteserver_cb (GtkWidget *item, ircserver *serv)
 }
 
 static void
-servlist_server_popmenu (ircserver *serv, GtkTreeView *treeview)
+servlist_server_popmenu (ircserver *serv, GtkTreeView *treeview, GdkEventButton *event)
 {
 	GtkWidget *item, *menu;
 	char buf[256];
@@ -283,13 +402,13 @@ servlist_server_popmenu (ircserver *serv, GtkTreeView *treeview)
 	menu = gtk_menu_new ();
 
 	snprintf (buf, sizeof (buf), _("_Remove \"%s\""), serv->hostname);
-	item = gtk_menu_item_new_with_mnemonic (buf);
+	item = create_icon_menu (buf, GTK_STOCK_DELETE, TRUE);
 	g_signal_connect (G_OBJECT (item), "activate",
 							G_CALLBACK (servlist_deleteserver_cb), serv);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
-	item = gtk_menu_item_new_with_mnemonic (_("_Add new server"));
+	item = create_icon_menu (_("_Add new server"), GTK_STOCK_NEW, TRUE);
 	g_signal_connect (G_OBJECT (item), "activate",
 							G_CALLBACK (servlist_addserver_cb), treeview);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
@@ -297,11 +416,11 @@ servlist_server_popmenu (ircserver *serv, GtkTreeView *treeview)
 
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (servlist_menu_destroy), menu);
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 0);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
 }
 
 static void
-servlist_network_popmenu (ircnet *net, GtkTreeView *treeview)
+servlist_network_popmenu (ircnet *net, GtkTreeView *treeview, GdkEventButton *event)
 {
 	GtkWidget *item, *menu;
 	char buf[256];
@@ -309,13 +428,31 @@ servlist_network_popmenu (ircnet *net, GtkTreeView *treeview)
 	menu = gtk_menu_new ();
 
 	snprintf (buf, sizeof (buf), _("_Remove \"%s\""), net->name);
-	item = gtk_menu_item_new_with_mnemonic (buf);
+	item = create_icon_menu (buf, GTK_STOCK_DELETE, TRUE);
 	g_signal_connect (G_OBJECT (item), "activate",
 							G_CALLBACK (servlist_deletenet_cb), net);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
-	item = gtk_menu_item_new_with_mnemonic (_("_Add new network"));
+	snprintf (buf, sizeof (buf), _("Move \"%s\" _down"), net->name);
+	item = create_icon_menu (buf, GTK_STOCK_GO_DOWN, TRUE);
+	g_signal_connect (G_OBJECT (item), "activate",
+							G_CALLBACK (servlist_movedownnet_cb), net);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	snprintf (buf, sizeof (buf), _("Move \"%s\" _up"), net->name);
+	item = create_icon_menu (buf, GTK_STOCK_GO_UP, TRUE);
+	g_signal_connect (G_OBJECT (item), "activate",
+							G_CALLBACK (servlist_moveupnet_cb), net);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	item = gtk_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	item = create_icon_menu (_("_Add new network"), GTK_STOCK_NEW, TRUE);
 	g_signal_connect (G_OBJECT (item), "activate",
 							G_CALLBACK (servlist_addnet_cb), treeview);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
@@ -323,49 +460,47 @@ servlist_network_popmenu (ircnet *net, GtkTreeView *treeview)
 
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (servlist_menu_destroy), menu);
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 0);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
 }
 
 static ircnet *
-servlist_find_selected_net (GtkTreeView *treeview)
+servlist_find_selected_net (GtkTreeSelection *sel, int *pos)
 {
-	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	char *netname;
 	ircnet *net = NULL;
 
-	sel = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_get_selected (sel, &model, &iter))
 	{
 		gtk_tree_model_get (model, &iter, 0, &netname, -1);
-		net = servlist_net_find (netname);
+		net = servlist_net_find (netname, pos);
+		if (net)
+			prefs.slist_select = *pos;
 	}
 
 	return net;
 }
 
 static void
-servlist_network_row_cb (GtkTreeView *treeview, gpointer user_data)
+servlist_network_row_cb (GtkTreeSelection *sel, gpointer user_data)
 {
 	ircnet *net;
+	int pos;
 
 	selected_net = NULL;
 
-	net = servlist_find_selected_net (treeview);
+	net = servlist_find_selected_net (sel, &pos);
 	if (net)
 	{
-		servlist_populate (net);
 		selected_net = net;
-		if (right_click)
-			servlist_network_popmenu (net, treeview);
+		servlist_populate (net);
 	}
 }
 
 static void
-servlist_server_row_cb (GtkTreeView *treeview, gpointer user_data)
+servlist_server_row_cb (GtkTreeSelection *sel, gpointer user_data)
 {
-	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	ircserver *serv;
@@ -375,17 +510,12 @@ servlist_server_row_cb (GtkTreeView *treeview, gpointer user_data)
 	if (!selected_net)
 		return;
 
-	sel = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_get_selected (sel, &model, &iter))
 	{
 		gtk_tree_model_get (model, &iter, 0, &servname, -1);
 		serv = servlist_server_find (selected_net, servname, &pos);
 		if (serv)
-		{
 			selected_net->selected = pos;
-			if (right_click)
-				servlist_server_popmenu (serv, treeview);
-		}
 	}
 }
 
@@ -425,16 +555,86 @@ servlist_connect_cb (GtkWidget *button, gpointer userdata)
 }
 
 static gboolean
-servlist_cellclick_cb (GtkWidget *widget, GdkEventButton *event,
+servlist_net_press_cb (GtkWidget *widget, GdkEventButton *event,
 							  gpointer user_data)
 {
-	right_click = FALSE;
+	GtkTreeSelection *sel;
+	GtkTreePath *path;
+	int pos;
+	ircnet *net;
 
 	if (event->type == GDK_2BUTTON_PRESS)
-		servlist_connect_cb (widget, user_data);
+	{
+		if (selected_net != NULL)
+			servlist_connect_cb (widget, user_data);
+	}
 
 	if (event->button == 3)
-		right_click = TRUE;
+	{
+		sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
+			 event->x, event->y, &path, 0, 0, 0))
+		{
+			gtk_tree_selection_unselect_all (sel);
+			gtk_tree_selection_select_path (sel, path);
+			gtk_tree_path_free (path);
+			net = servlist_find_selected_net (sel, &pos);
+			if (net)
+				servlist_network_popmenu (net, GTK_TREE_VIEW (widget), event);
+		} else
+		{
+			gtk_tree_selection_unselect_all (sel);
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+servlist_serv_press_cb (GtkWidget *widget, GdkEventButton *event,
+							   gpointer user_data)
+{
+	GtkTreeSelection *sel;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	int pos;
+	char *servname;
+	ircserver *serv;
+	GtkTreeModel *model;
+
+	if (event->type == GDK_2BUTTON_PRESS)
+	{
+		if (selected_net != NULL)
+			servlist_connect_cb (widget, user_data);
+	}
+
+	if (event->button == 3)
+	{
+		sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (servers_tree));
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (servers_tree),
+			 event->x, event->y, &path, 0, 0, 0))
+		{
+			gtk_tree_selection_unselect_all (sel);
+			gtk_tree_selection_select_path (sel, path);
+			gtk_tree_path_free (path);
+
+			if (gtk_tree_selection_get_selected (sel, &model, &iter))
+			{
+				gtk_tree_model_get (model, &iter, 0, &servname, -1);
+				serv = servlist_server_find (selected_net, servname, &pos);
+				if (serv)
+					servlist_server_popmenu (serv, GTK_TREE_VIEW (servers_tree), event);
+			}
+
+		} else
+		{
+			gtk_tree_selection_unselect_all (sel);
+		}
+
+		return TRUE;
+	}
 
 	return FALSE;
 }
@@ -452,9 +652,7 @@ servlist_celledit_cb (GtkCellRendererText *cell, gchar *arg1, gchar *arg2,
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get (model, &iter, 0, &netname, -1);
 
-//	printf("edited: %s -> %s\n", netname, arg2);
-
-	net = servlist_net_find (netname);
+	net = servlist_net_find (netname, NULL);
 	if (net)
 	{
 		netname = net->name;
@@ -491,8 +689,6 @@ gtkutil_create_list (GtkWidget *box, char *title, void *select_callback,
 	model = GTK_TREE_MODEL (store);
 
 	treeview = gtk_tree_view_new_with_model (model);
-	g_signal_connect (G_OBJECT (treeview), "cursor_changed",
-							G_CALLBACK (select_callback), NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1,
@@ -550,8 +746,11 @@ servlist_entry_cb (GtkWidget *entry, gpointer userdata)
 	if (!selected_net)
 		return;
 
-	free (*str);
-	*str = strdup (GTK_ENTRY (entry)->text);
+	if (!ignore_changed)
+	{
+		free (*str);
+		*str = strdup (GTK_ENTRY (entry)->text);
+	}
 }
 
 static GtkWidget *
@@ -581,7 +780,7 @@ servlist_create_infobox (GtkWidget *box)
 {
 	GtkWidget *frame, *table, *label, *entry, *img, *hbox;
 
-	frame = gtk_frame_new ("Global User Info");
+	frame = gtk_frame_new (_("Global User Info"));
 
 	table = gtk_table_new (2, 5, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
@@ -589,7 +788,7 @@ servlist_create_infobox (GtkWidget *box)
 	img = gtk_image_new_from_pixbuf (pix_xchat);
 	gtk_table_attach (GTK_TABLE (table), img, 4, 5, 0, 2, 0, 0, 0, 0);
 
-	label = gtk_label_new ("Nick Names:");
+	label = gtk_label_new (_("Nick Names:"));
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
 							GTK_FILL, 0, 1, 0);
 
@@ -608,7 +807,7 @@ servlist_create_infobox (GtkWidget *box)
 							GTK_FILL|GTK_EXPAND, 0, 1, 0);
 	gtk_entry_set_text (GTK_ENTRY (entry), prefs.nick3);
 
-	label = gtk_label_new ("User Name:");
+	label = gtk_label_new (_("User Name:"));
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
 							GTK_FILL, 0, 1, 0);
 
@@ -621,7 +820,7 @@ servlist_create_infobox (GtkWidget *box)
 	gtk_table_attach (GTK_TABLE (table), hbox, 2, 4, 1, 2,
 							GTK_FILL|GTK_EXPAND, 0, 1, 0);
 
-	label = gtk_label_new ("Real Name:");
+	label = gtk_label_new (_("Real Name:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, 0, 0, 10);
 
 	entry_greal = entry = gtk_entry_new ();
@@ -669,11 +868,14 @@ servlist_create_buttons (GtkWidget *box)
 	gtk_widget_set_sensitive (but, FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), but, 0, 0, 0);
 
-	connectnew_button = but = gtk_button_new_with_mnemonic (_("Connect in a _new tab"));
-	g_signal_connect (G_OBJECT (but), "clicked",
-							G_CALLBACK (servlist_connectnew_cb), 0);
-	gtk_widget_set_sensitive (but, FALSE);
-	gtk_box_pack_start (GTK_BOX (hbox), but, 0, 0, 0);
+	if (servlist_sess)
+	{
+		connectnew_button = but = gtk_button_new_with_mnemonic (_("Connect in a _new tab"));
+		g_signal_connect (G_OBJECT (but), "clicked",
+								G_CALLBACK (servlist_connectnew_cb), 0);
+		gtk_widget_set_sensitive (but, FALSE);
+		gtk_box_pack_start (GTK_BOX (hbox), but, 0, 0, 0);
+	}
 
 	but = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
 	g_signal_connect (G_OBJECT (but), "clicked",
@@ -699,8 +901,6 @@ servlist_editserver_cb (GtkCellRendererText *cell, gchar *arg1, gchar *arg2,
 	if (!selected_net)
 		return;
 
-//	printf("edited server: %s -> %s\n", servname, arg2);
-
 	serv = servlist_server_find (selected_net, servname, NULL);
 	if (serv)
 	{
@@ -722,7 +922,7 @@ servlist_create_servlistbox (GtkWidget *box)
 	gtk_container_add (GTK_CONTAINER (box), vbox);
 
 	tree = gtkutil_create_list (vbox, _("Servers"), servlist_server_row_cb,
-										 servlist_editserver_cb, servlist_cellclick_cb);
+										 servlist_editserver_cb, servlist_serv_press_cb);
 #if 0
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
@@ -736,6 +936,54 @@ servlist_create_servlistbox (GtkWidget *box)
 	return tree;
 }
 
+static void
+servlist_combo_cb (GtkEntry *entry, gpointer userdata)
+{
+	if (!selected_net)
+		return;
+
+	if (!ignore_changed)
+	{
+		if (selected_net->encoding)
+			free (selected_net->encoding);
+		selected_net->encoding = strdup (entry->text);
+	}
+}
+
+static GtkWidget *
+servlist_create_charsetcombo (GtkTable *table)
+{
+	GtkWidget *cb;
+	GtkWidget *label;
+	int i;
+	static GList *cbitems = NULL;
+
+	label = gtk_label_new (_("Character Set:"));
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 7, 8,
+							GTK_FILL|GTK_SHRINK, GTK_FILL, 4, 1);
+	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+
+	if (cbitems == NULL)
+	{
+		cbitems = g_list_append (cbitems, "System default");
+		i = 0;
+		while (pages[i])
+		{
+			cbitems = g_list_append (cbitems, (char *)pages[i]);
+			i++;
+		}
+	}
+
+	cb = gtk_combo_new ();
+	gtk_combo_set_popdown_strings (GTK_COMBO (cb), cbitems);
+	g_signal_connect (G_OBJECT (GTK_COMBO (cb)->entry), "changed",
+							G_CALLBACK (servlist_combo_cb), NULL);
+	gtk_table_attach (GTK_TABLE (table), cb, 1, 2, 7, 8,
+						   GTK_FILL|GTK_EXPAND, 0, 0, 0);
+
+	return cb;
+}
+
 static void
 servlist_create_editbox (GtkWidget *box)
 {
@@ -762,6 +1010,10 @@ servlist_create_editbox (GtkWidget *box)
 	add_tip (entry_join, _("Channels to join, separated by commas, but not spaces!"));
 	entry_cmd = servlist_create_entry (table, _("Connect Command:"), 6,
 											G_STRUCT_OFFSET (ircnet, command));
+	add_tip (entry_cmd, _("Command to execute after connecting. Can be used to authenticate to NickServ"));
+
+	combo_encoding = servlist_create_charsetcombo (GTK_TABLE (table));
+
 	gtk_container_add (GTK_CONTAINER (hbox), table);
 
 	table = gtk_table_new (2, 3, FALSE);
@@ -772,6 +1024,10 @@ servlist_create_editbox (GtkWidget *box)
 	check_buttons[3] = servlist_create_check (3, table, 8, 0, _("Auto connect at startup"));
 	check_buttons[4] = servlist_create_check (4, table, 8, 1, _("Use a proxy server"));
 	check_buttons[5] = servlist_create_check (5, table, 8, 2, _("Accept invalid cert."));
+#ifndef USE_OPENSSL
+	gtk_widget_set_sensitive (check_buttons[2], FALSE);
+	gtk_widget_set_sensitive (check_buttons[5], FALSE);
+#endif
 	gtk_box_pack_end (GTK_BOX (vbox), table, 0, 0, 0);
 
 	servers_tree = servlist_create_servlistbox (hbox);
@@ -785,7 +1041,7 @@ servlist_create_editbox (GtkWidget *box)
 static void
 servlist_editmode_cb (GtkToggleButton *but, gpointer userdata)
 {
-	if (gtk_toggle_button_get_active (but))
+	if (but->active)
 	{
 		prefs.slist_edit = 1;
 	  	servlist_networks_populate (networks_tree, network_list, TRUE);
@@ -816,7 +1072,7 @@ servlist_create_list (GtkWidget *box)
 	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 
-	but = gtk_check_button_new_with_label ("Edit mode");
+	but = gtk_check_button_new_with_label (_("Edit mode"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (but),
 											prefs.slist_edit);
 	g_signal_connect (G_OBJECT (but), "toggled",
@@ -824,7 +1080,7 @@ servlist_create_list (GtkWidget *box)
 	gtk_box_pack_start (GTK_BOX (hbox), but, 0, 0, 0);
 
 	tree = gtkutil_create_list (vbox, _("Networks"), servlist_network_row_cb,
-										 servlist_celledit_cb, servlist_cellclick_cb);
+										 servlist_celledit_cb, servlist_net_press_cb);
 	servlist_networks_populate (tree, network_list, prefs.slist_edit);
 
 	servlist_create_editbox (pane);
@@ -832,6 +1088,50 @@ servlist_create_list (GtkWidget *box)
 	return tree;
 }
 
+/*static void
+skip_motd (GtkWidget * igad, gpointer serv)
+{
+	if (GTK_TOGGLE_BUTTON (igad)->active)
+		prefs.skipmotd = TRUE;
+	else
+		prefs.skipmotd = FALSE;
+}*/
+
+static void
+no_servlist (GtkWidget * igad, gpointer serv)
+{
+	if (GTK_TOGGLE_BUTTON (igad)->active)
+		prefs.slist_skip = TRUE;
+	else
+		prefs.slist_skip = FALSE;
+}
+
+static void
+servlist_skip (GtkWidget *box)
+{
+	GtkWidget *hbox, *but;
+
+	hbox = gtk_hbox_new (FALSE, 5);
+	gtk_box_pack_start (GTK_BOX (box), hbox, 0, 0, 0);
+/*
+	but = gtk_check_button_new_with_label (_("Skip MOTD"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (but),
+											prefs.skipmotd);
+	g_signal_connect (G_OBJECT (but), "toggled",
+							G_CALLBACK (skip_motd), 0);
+	gtk_box_pack_start (GTK_BOX (hbox), but, TRUE, FALSE, 0);
+	gtk_widget_show (but);
+	add_tip (but, _("Don't display the message-of-the-day when logging in"));
+*/
+	but = gtk_check_button_new_with_label (_("No server list on startup"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (but),
+											prefs.slist_skip);
+	g_signal_connect (G_OBJECT (but), "toggled",
+							G_CALLBACK (no_servlist), 0);
+	gtk_box_pack_start (GTK_BOX (hbox), but, TRUE, FALSE, 0);
+	gtk_widget_show (but);
+}
+
 void
 fe_serverlist_open (session *sess)
 {
@@ -844,9 +1144,11 @@ fe_serverlist_open (session *sess)
 	}
 
 	servlist_sess = sess;
+	connectnew_button = NULL;
 
 	serverlist_win = win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (win), _("X-Chat: Server List"));
+	gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_MOUSE);
 	gtkutil_set_icon (win);
 	gtk_container_set_border_width (GTK_CONTAINER (win), 5);
 
@@ -859,8 +1161,19 @@ fe_serverlist_open (session *sess)
 	servlist_create_infobox (vbox);
 	networks_tree = servlist_create_list (vbox);
 	servlist_create_buttons (vbox);
+	servlist_skip (vbox);
 
 	gtk_widget_show_all (win);
 	if (!prefs.slist_edit)
 		gtk_widget_hide (editbox);
+
+	/* force selection */
+	servlist_network_row_cb (gtk_tree_view_get_selection (GTK_TREE_VIEW (networks_tree)), NULL);
+
+	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (networks_tree))),
+							"changed", G_CALLBACK (servlist_network_row_cb), NULL);
+
+	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (servers_tree))),
+							"changed", G_CALLBACK (servlist_server_row_cb), NULL);
 }
+
