@@ -3,14 +3,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
+#include "../common/text.h"
 #include "../common/xchatc.h"
 #include "fe-gtk.h"
 #include "gtkutil.h"
 #include "maingui.h"
 #include "palette.h"
+#include "pixmaps.h"
 
 #include <gtk/gtkcolorseldialog.h>
 #include <gtk/gtktable.h>
@@ -38,6 +42,10 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkhscale.h>
 
+
+GtkStyle *create_input_style (void);
+
+
 static int last_selected_page = 0;
 static struct xchatprefs setup_prefs;
 
@@ -52,6 +60,7 @@ enum
 	ST_RADIO,
 	ST_NUMBER,
 	ST_HSCALE,
+	ST_LABEL,
 };
 
 typedef struct
@@ -69,7 +78,8 @@ static const setting textbox_settings[] =
 {
 	{ST_EFONT,  N_("Font:"), P_OFFSET(font_normal), 0, 0, sizeof prefs.font_normal},
 	{ST_EFILE,  N_("Background image:"), P_OFFSET(background), 0, 0, sizeof prefs.background},
-	{ST_ENTRY,  N_("Time stamp format:"), P_OFFSET(stamp_format),0,0,sizeof prefs.stamp_format},
+	{ST_ENTRY,  N_("Time stamp format:"), P_OFFSET(stamp_format),
+					N_("See strftime manpage for details."),0,sizeof prefs.stamp_format},
 	{ST_TOGGLE, N_("Time stamp text"), P_OFFINT(timestamp),0,0,0},
 	{ST_TOGGLE, N_("Transparent background"), P_OFFINT(transparent),0,0,0},
 	{ST_TOGGLE, N_("Indent nicks"), P_OFFINT(indent_nicks),0,0,0},
@@ -77,21 +87,31 @@ static const setting textbox_settings[] =
 	{ST_TOGGLE, N_("Colored nicks"), P_OFFINT(colorednicks),0,0,0},
 	{ST_TOGGLE, N_("Strip mIRC color"), P_OFFINT(stripcolor),0,0,0},
 	{ST_NUMBER,	N_("Scrollback lines:"), P_OFFINT(max_lines),0,0,100000},
-	{ST_HSCALE, N_("Tint blue:"), P_OFFINT(tint_blue),0,0,0},
 	{ST_HSCALE, N_("Tint red:"), P_OFFINT(tint_red),0,0,0},
 	{ST_HSCALE, N_("Tint green:"), P_OFFINT(tint_green),0,0,0},
+	{ST_HSCALE, N_("Tint blue:"), P_OFFINT(tint_blue),0,0,0},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
 static const setting inputbox_settings[] =
 {
+	{ST_TOGGLE, N_("Interpret %nnn as an ASCII value"), P_OFFINT(perc_ascii),0,0,0},
 	{ST_TOGGLE, N_("Automatic nick completion"), P_OFFINT(nickcompletion),0,0,0},
+	{ST_TOGGLE, N_("Interpret %C, %B as Color, Bold etc"), P_OFFINT(perc_color),0,0,0},
 	{ST_TOGGLE, N_("Use the Text box font"), P_OFFINT(style_inputbox),0,0,0},
-	{ST_TOGGLE, N_("Interpret %nnn as an ASCII value"), P_OFFINT(perc_color),0,0,0},
 	{ST_ENTRY, N_("Nick completion suffix:"), P_OFFSET(nick_suffix),0,0,sizeof prefs.nick_suffix},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
+static char *lagmenutext[] = 
+{
+	N_("Off"),
+	N_("Graph"),
+	N_("Info text"),
+	N_("Both"),
+	NULL
+};
+
 static char *ulmenutext[] = 
 {
 	N_("A-Z, Ops first"),
@@ -104,10 +124,12 @@ static char *ulmenutext[] =
 
 static const setting userlist_settings[] =
 {
+	{ST_MENU,	N_("Lag meter:"), P_OFFINT(lagometer), 0, lagmenutext, 0},
+	{ST_MENU,	N_("Throttle meter:"), P_OFFINT(throttlemeter), 0, lagmenutext, 0},
+	{ST_MENU,	N_("Userlist sorted by:"), P_OFFINT(userlist_sort), 0, ulmenutext, 0},
+	{ST_ENTRY,	N_("Double-click command:"), P_OFFSET(doubleclickuser), 0, 0, sizeof prefs.doubleclickuser},
 	{ST_TOGGLE, N_("Show hostnames in userlist"), P_OFFINT(showhostname_in_userlist), 0, 0, 0},
 	{ST_TOGGLE, N_("Userlist buttons enabled"), P_OFFINT(userlistbuttons), 0, 0, 0},
-	{ST_ENTRY,	N_("Double-click command:"), P_OFFSET(doubleclickuser), 0, 0, sizeof prefs.doubleclickuser},
-	{ST_MENU,	N_("Userlist sorted by:"), P_OFFINT(userlist_sort), 0, ulmenutext, 0},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -122,21 +144,22 @@ static char *tabpos[] =
 {
 	N_("Bottom"),
 	N_("Top"),
-	N_("Left"),
-	N_("Right"),
+/*	N_("Left"),
+	N_("Right"),*/
 	N_("Hidden"),
 	NULL
 };
 
 static const setting tabs_settings[] =
 {
+	{ST_MENU,	N_("Show tabs at:"), P_OFFINT(tabs_position), 0, tabpos, 0},
 	{ST_MENU,	N_("Open channels in:"), P_OFFINT(tabchannels), 0, tabwin, 0},
 	{ST_MENU,	N_("Open dialogs in:"), P_OFFINT(privmsgtab), 0, tabwin, 0},
 	{ST_MENU,	N_("Open utilities in:"), P_OFFINT(windows_as_tabs), N_("Open DCC, Ignore, Notify etc, in tabs or windows?"), tabwin, 0},
-	{ST_MENU,	N_("Show tabs at:"), P_OFFINT(tabs_position), 0, tabpos, 0},
-	{ST_TOGGLE, N_("Open extra tab for each server"), P_OFFINT(use_server_tab), 0, 0, 0},
+	{ST_NUMBER,	N_("Shorten tabs to:"), P_OFFINT(truncchans), 0, (char **)N_("letters."), 99},
+	{ST_TOGGLE, N_("Open tab for server messages"), P_OFFINT(use_server_tab), 0, 0, 0},
 	{ST_TOGGLE, N_("Only highlight tabs on channel messages"), P_OFFINT(limitedtabhighlight), 0, 0, 0},
-	{ST_TOGGLE, N_("Open extra tab for server notices"), P_OFFINT(notices_tabs), 0, 0, 0},
+	{ST_TOGGLE, N_("Open tab for server notices"), P_OFFINT(notices_tabs), 0, 0, 0},
 	{ST_TOGGLE, N_("Pop new tabs to front"), P_OFFINT(newtabstofront), 0, 0, 0},
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -144,9 +167,12 @@ static const setting tabs_settings[] =
 static const setting filexfer_settings[] =
 {
 	{ST_ENTRY,	N_("Download files to:"), P_OFFSET(dccdir), 0, 0, sizeof prefs.dccdir},
-	{ST_ENTRY,	N_("DCC IP address:"), P_OFFSET(dcc_ip_str), 0, 0, sizeof prefs.dcc_ip_str},
+	{ST_ENTRY,	N_("Move completed files to:"), P_OFFSET(dcc_completed_dir), 0, 0, sizeof prefs.dcc_completed_dir},
+	{ST_ENTRY,	N_("DCC IP address:"), P_OFFSET(dcc_ip_str),
+					N_("Claim you are at this address when offering files."), 0, sizeof prefs.dcc_ip_str},
 	{ST_NUMBER,	N_("First DCC send port:"), P_OFFINT(first_dcc_send_port), 0, 0, 65535},
 	{ST_NUMBER,	N_("Last DCC send port:"), P_OFFINT(last_dcc_send_port), 0, 0, 65535},
+	{ST_LABEL,	N_("(Leave ports at zero for full range).")},
 	{ST_TOGGLE, N_("Auto open DCC send list"), P_OFFINT(autoopendccsendwindow), 0, 0, 0},
 	{ST_TOGGLE, N_("Convert spaces to underscore"), P_OFFINT(dcc_send_fillspaces),
 					N_("In filenames, before sending"), 0, 0},
@@ -154,7 +180,16 @@ static const setting filexfer_settings[] =
 	{ST_TOGGLE, N_("Save nickname in filenames"), P_OFFINT(dccwithnick), 0, 0, 0},
 	{ST_TOGGLE, N_("Auto open DCC receive list"), P_OFFINT(autoopendccrecvwindow), 0, 0, 0},
 	{ST_TOGGLE, N_("Get my IP from IRC server"), P_OFFINT(ip_from_server),
-					N_("/WHOIS yourself to find your real address"), 0, 0},
+					N_("/WHOIS yourself to find your real address. Use this if you have a 192.168.*.* address!"), 0, 0},
+	{ST_NUMBER,	N_("Max. send CPS:"), P_OFFINT(dcc_max_send_cps), 
+					N_("Max. speed for one transfer"), 0, 1000000},
+	{ST_NUMBER,	N_("Max. receive CPS:"), P_OFFINT(dcc_max_get_cps),
+					N_("Max. speed for one transfer"), 0, 1000000},
+	{ST_NUMBER,	N_("Max. global send CPS:"), P_OFFINT(dcc_global_max_send_cps),
+					N_("Max. speed for all traffic"), 0, 1000000},
+	{ST_NUMBER,	N_("Max. global receive CPS:"), P_OFFINT(dcc_global_max_get_cps),
+					N_("Max. speed for all traffic"), 0, 1000000},
+	{ST_LABEL,	N_("(Leave at zero for full speed file transfers).")},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -163,7 +198,13 @@ static const setting general_settings[] =
 	{ST_ENTRY,	N_("Default quit message:"), P_OFFSET(quitreason), 0, 0, sizeof prefs.quitreason},
 	{ST_ENTRY,	N_("Default part message:"), P_OFFSET(partreason), 0, 0, sizeof prefs.partreason},
 	{ST_ENTRY,	N_("Default away message:"), P_OFFSET(awayreason), 0, 0, sizeof prefs.awayreason},
+#ifndef WIN32
+	{ST_LABEL,	N_("(Can be a text file relative to ~/.xchat2/).")},
+#else
+	{ST_LABEL,	N_("(Can be a text file relative to config\\).")},
+#endif
 	{ST_ENTRY,	N_("Extra words to highlight on:"), P_OFFSET(bluestring), 0, 0, sizeof prefs.bluestring},
+	{ST_LABEL,	N_("(Separate multiple words with commas).")},
 	{ST_TOGGLE,	N_("Show away once"), P_OFFINT(show_away_once), N_("Show identical away messages only once"), 0, 0},
 	{ST_TOGGLE,	N_("Beep on private messages"), P_OFFINT(beepmsg), 0, 0, 0},
 	{ST_TOGGLE,	N_("Automatically unmark away"), P_OFFINT(auto_unmark_away), N_("Unmark yourself as away before sending messages"), 0, 0},
@@ -177,7 +218,9 @@ static const setting general_settings[] =
 static const setting logging_settings[] =
 {
 	{ST_ENTRY,	N_("Log filename mask:"), P_OFFSET(logmask), 0, 0, sizeof prefs.logmask},
+	{ST_LABEL,	N_("(%s=Server %c=Channel %n=Network).")},
 	{ST_ENTRY,	N_("Log timestamp format:"), P_OFFSET(timestamp_log_format), 0, 0, sizeof prefs.timestamp_log_format},
+	{ST_LABEL,	N_("(See strftime manpage for details).")},
 	{ST_TOGGLE,	N_("Enable logging of conversations"), P_OFFINT(logging), 0, 0, 0},
 	{ST_TOGGLE,	N_("Insert timestamps in logs"), P_OFFINT(timestamp_logs), 0, 0, 0},
 	{ST_END, 0, 0, 0, 0, 0}
@@ -196,6 +239,7 @@ static char *proxytypes[] =
 static const setting network_settings[] =
 {
 	{ST_ENTRY,	N_("Address to bind to:"), P_OFFSET(hostname), 0, 0, sizeof prefs.hostname},
+	{ST_LABEL,	N_("(Only useful for computers with multiple addresses).")},
 	{ST_ENTRY,	N_("Proxy server:"), P_OFFSET(proxy_host), 0, 0, sizeof prefs.proxy_host},
 	{ST_NUMBER,	N_("Proxy port:"), P_OFFINT(proxy_port), 0, 0, 65535},
 	{ST_MENU,	N_("Proxy type:"), P_OFFINT(proxy_type), 0, proxytypes, 0},
@@ -220,13 +264,13 @@ setup_create_toggle (GtkWidget *box, int row, const setting *set)
 {
 	GtkWidget *wid;
 
-	wid = gtk_check_button_new_with_label (set->label);
+	wid = gtk_check_button_new_with_label (_(set->label));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid),
 											setup_get_int (&setup_prefs, set));
 	g_signal_connect (G_OBJECT (wid), "toggled",
 							G_CALLBACK (setup_toggle_cb), (gpointer)set);
 	if (set->tooltip)
-		add_tip (wid, set->tooltip);
+		add_tip (wid, _(set->tooltip));
 	gtk_box_pack_start (GTK_BOX (box), wid, 0, 0, 0);
 }
 
@@ -241,7 +285,7 @@ setup_create_spin (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *label, *wid, *rbox, *align;
 
-	label = gtk_label_new (set->label);
+	label = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
@@ -253,11 +297,19 @@ setup_create_spin (GtkWidget *table, int row, const setting *set)
 	gtk_container_add (GTK_CONTAINER (align), rbox);
 
 	wid = gtk_spin_button_new_with_range (0, set->extra, 1);
+	if (set->tooltip)
+		add_tip (wid, _(set->tooltip));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (wid),
 										setup_get_int (&setup_prefs, set));
 	g_signal_connect (G_OBJECT (wid), "value-changed",
 							G_CALLBACK (setup_spin_cb), (gpointer)set);
 	gtk_box_pack_start (GTK_BOX (rbox), wid, 0, 0, 0);
+
+	if (set->list)
+	{
+		label = gtk_label_new (_((char *)set->list));
+		gtk_box_pack_start (GTK_BOX (rbox), label, 0, 0, 5);
+	}
 }
 
 
@@ -272,20 +324,18 @@ setup_create_hscale (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *wid;
 
-	wid = gtk_hscale_new_with_range(0., 255., 1.);
+	wid = gtk_label_new (_(set->label));
+	gtk_misc_set_alignment (GTK_MISC (wid), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), wid, 0, 1, row, row + 1,
+							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+
+	wid = gtk_hscale_new_with_range (0., 255., 1.);
+	gtk_scale_set_value_pos (GTK_SCALE (wid), GTK_POS_RIGHT);
 	gtk_range_set_value (GTK_RANGE (wid), setup_get_int (&setup_prefs, set));
 	g_signal_connect (G_OBJECT(wid), "value_changed",
 							G_CALLBACK (setup_hscale_cb), (gpointer)set);
-	if (set->tooltip)
-		add_tip (wid, set->tooltip);
-
 	gtk_table_attach (GTK_TABLE (table), wid, 1, 5, row, row + 1,
-						GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-
-	wid = gtk_label_new (set->label);
-	gtk_misc_set_alignment (GTK_MISC (wid), 1.0, 1.0);
-	gtk_table_attach (GTK_TABLE (table), wid, 0, 1, row, row + 1,
-						GTK_FILL, GTK_FILL, 0, 0);
+							GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 }
 
 #if 0
@@ -329,24 +379,24 @@ setup_menu_cb (GtkWidget *item, const setting *set)
 static void
 setup_create_menu (GtkWidget *table, int row, const setting *set)
 {
-	GtkWidget *wid, *menu, *item;
+	GtkWidget *wid, *menu, *item, *align;
 	int i;
 	char **text = set->list;
 
-	wid = gtk_label_new (set->label);
+	wid = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (wid), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), wid, 0, 1, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
 	wid = gtk_option_menu_new ();
 	if (set->tooltip)
-		add_tip (wid, set->tooltip);
+		add_tip (wid, _(set->tooltip));
 	menu = gtk_menu_new ();
 
 	i = 0;
 	while (text[i])
 	{
-		item = gtk_menu_item_new_with_label (text[i]);
+		item = gtk_menu_item_new_with_label (_(text[i]));
 		g_object_set_data (G_OBJECT (item), "n", GINT_TO_POINTER (i));
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -359,7 +409,9 @@ setup_create_menu (GtkWidget *table, int row, const setting *set)
 	gtk_option_menu_set_history (GTK_OPTION_MENU (wid),
 										  setup_get_int (&setup_prefs, set));
 
-	gtk_table_attach (GTK_TABLE (table), wid, 1, 2, row, row + 1,
+	align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (align), wid);
+	gtk_table_attach (GTK_TABLE (table), align, 1, 4, row, row + 1,
 							GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 }
 
@@ -370,14 +422,13 @@ setup_filereq_cb (GtkWidget *entry, void *data2, char *file)
 	{
 		if (file[0])
 			gtk_entry_set_text (GTK_ENTRY (entry), file);
-		free (file);
 	}
 }
 
 static void
 setup_browsefile_cb (GtkWidget *button, GtkWidget *entry)
 {
-	gtkutil_file_req (_("Choose File"), setup_filereq_cb, entry, 0, FALSE);
+	gtkutil_file_req (_("Select an Image File"), setup_filereq_cb, entry, 0, FALSE);
 }
 
 static void
@@ -408,7 +459,8 @@ setup_browsefont_cb (GtkWidget *button, GtkWidget *entry)
 
 	sel = (GtkFontSelection *) dialog->fontsel;
 
-	gtk_font_selection_set_font_name (sel, GTK_ENTRY (entry)->text);
+	if (GTK_ENTRY (entry)->text[0])
+		gtk_font_selection_set_font_name (sel, GTK_ENTRY (entry)->text);
 
 	g_object_set_data (G_OBJECT (dialog->ok_button), "e", entry);
 
@@ -426,28 +478,40 @@ setup_entry_cb (GtkEntry *entry, setting *set)
 	setup_set_str (&setup_prefs, set, entry->text);
 }
 
+static void
+setup_create_label (GtkWidget *table, int row, const setting *set)
+{
+	gtk_table_attach (GTK_TABLE (table), gtk_label_new (_(set->label)),
+							1, 2, row, row + 1, GTK_SHRINK | GTK_FILL,
+							GTK_SHRINK | GTK_FILL, 0, 0);
+}
+
 static void
 setup_create_entry (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *label;
 	GtkWidget *wid, *bwid;
 
-	label = gtk_label_new (set->label);
+	label = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
 	wid = gtk_entry_new ();
+	if (set->tooltip)
+		add_tip (wid, _(set->tooltip));
 	gtk_entry_set_max_length (GTK_ENTRY (wid), set->extra);
 	gtk_entry_set_text (GTK_ENTRY (wid), setup_get_str (&setup_prefs, set));
 	g_signal_connect (G_OBJECT (wid), "changed",
 							G_CALLBACK (setup_entry_cb), (gpointer)set);
 
-	gtk_table_attach (GTK_TABLE (table), wid, 1, 4, row, row + 1,
-							GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-
-	if (set->type != ST_ENTRY)
+	if (set->type == ST_ENTRY)
+		gtk_table_attach (GTK_TABLE (table), wid, 1, 5, row, row + 1,
+								GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+	else
 	{
+		gtk_table_attach (GTK_TABLE (table), wid, 1, 4, row, row + 1,
+								GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 		bwid = gtk_button_new_with_label (_("Browse..."));
 		gtk_table_attach (GTK_TABLE (table), bwid, 4, 5, row, row + 1,
 								GTK_SHRINK, GTK_SHRINK, 0, 0);
@@ -514,6 +578,9 @@ setup_create_page (const setting *set)
 		case ST_HSCALE:
 			setup_create_hscale (tab, row, &set[i]);
 			break;
+		case ST_LABEL:
+			setup_create_label (tab, row, &set[i]);
+			break;
 		}
 		i++;
 		row++;
@@ -609,7 +676,7 @@ setup_create_other_color (char *text, int num, int row, GtkWidget *tab)
 
 	label = gtk_label_new (text);
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_table_attach (GTK_TABLE (tab), GTK_WIDGET (label), 0, 1, row, row + 1,
+	gtk_table_attach (GTK_TABLE (tab), label, 0, 1, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 	setup_create_color_button (tab, num, row, 1);
 }
@@ -631,7 +698,7 @@ setup_create_color_page (void)
 
 	label = gtk_label_new (_("mIRC colors:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_table_attach (GTK_TABLE (tab), GTK_WIDGET (label), 0, 1, 0, 1,
+	gtk_table_attach (GTK_TABLE (tab), label, 0, 1, 0, 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
 	for (i = 0; i < 16; i++)
@@ -639,8 +706,13 @@ setup_create_color_page (void)
 
 	setup_create_other_color (_("Foreground:"), 18, 1, tab);
 	setup_create_other_color (_("Background:"), 19, 2, tab);
-	setup_create_other_color (_("Mark fore:"), 17, 3, tab);
-	setup_create_other_color (_("Mark back:"), 16, 4, tab);
+
+	setup_create_other_color (_("Mark fore:"), 17, 6, tab);
+	setup_create_other_color (_("Mark back:"), 16, 7, tab);
+
+	setup_create_other_color (_("New Data:"), 20, 10, tab);
+	setup_create_other_color (_("New Message:"), 22, 11, tab);
+	setup_create_other_color (_("Highlight:"), 21, 12, tab);
 
 	return box;
 }
@@ -765,13 +837,13 @@ setup_create_tree (GtkWidget *box, GtkWidget *book)
 	do
 	{
 		gtk_tree_store_append (model, &iter, NULL);
-		gtk_tree_store_set (model, &iter, 0, cata[i], 1, -1, -1);
+		gtk_tree_store_set (model, &iter, 0, _(cata[i]), 1, -1, -1);
 		i++;
 
 		do
 		{
 			gtk_tree_store_append (model, &child_iter, &iter);
-			gtk_tree_store_set (model, &child_iter, 0, cata[i], 1, page, -1);
+			gtk_tree_store_set (model, &child_iter, 0, _(cata[i]), 1, page, -1);
 			if (page == last_selected_page)
 				sel_iter = gtk_tree_iter_copy (&child_iter);
 			page++;
@@ -820,9 +892,27 @@ setup_apply (struct xchatprefs *pr)
 	int done_main = FALSE;
 	session *sess;
 	GtkStyle *old_style;
+	int new_pix = FALSE;
+
+	if (strcmp (pr->background, prefs.background) != 0)
+		new_pix = TRUE;
 
 	memcpy (&prefs, pr, sizeof (prefs));
-	gtkutil_simpledialog ("apply not fully implemented (yet).\n");
+
+#ifdef WIN32
+	mkdir (prefs.dccdir);
+	mkdir (prefs.dcc_completed_dir);
+#else
+	mkdir (prefs.dccdir, S_IRUSR | S_IWUSR | S_IXUSR);
+	mkdir (prefs.dcc_completed_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+#endif
+
+	if (new_pix)
+	{
+		if (channelwin_pix)
+			g_object_unref (channelwin_pix);
+		channelwin_pix = pixmap_load_from_file (prefs.background);
+	}
 
 	old_style = input_style;
 	input_style = create_input_style ();
@@ -843,24 +933,36 @@ setup_apply (struct xchatprefs *pr)
 		{
 			setup_apply_to_sess (sess);
 		}
+
+		if (prefs.logging)
+			log_open (sess);
+		else
+			log_close (sess);
+
 		list = list->next;
 	}
 
+	mg_apply_setup ();
+
 	g_object_unref (old_style);
 }
 
+#if 0
 static void
 setup_apply_cb (GtkWidget *but, GtkWidget *win)
 {
 	/* setup_prefs -> xchat */
 	setup_apply (&setup_prefs);
 }
+#endif
 
 static void
 setup_ok_cb (GtkWidget *but, GtkWidget *win)
 {
 	gtk_widget_destroy (win);
 	setup_apply (&setup_prefs);
+	save_config ();
+	palette_save ();
 }
 
 static GtkWidget *
@@ -888,10 +990,13 @@ setup_window_open (void)
 	gtk_box_pack_end (GTK_BOX (hbox), hbbox, FALSE, FALSE, 0);
 
 	/* standard buttons */
+	/* GNOME doesn't like apply */
+#if 0
 	wid = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 	g_signal_connect (G_OBJECT (wid), "clicked",
 							G_CALLBACK (setup_apply_cb), win);
 	gtk_box_pack_start (GTK_BOX (hbbox), wid, FALSE, FALSE, 0);
+#endif
 
 	wid = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
 	g_signal_connect (G_OBJECT (wid), "clicked",
