@@ -51,16 +51,48 @@
 
 
 GdkPixbuf *
-get_prefix_pixbuf (char prefix)
+get_user_icon (server *serv, struct User *user)
 {
-	switch (prefix)
+	char *pre;
+	int level;
+
+	if (!user)
+		return NULL;
+
+	/* these ones are hardcoded */
+	switch (user->prefix[0])
 	{
-	case '*': return pix_admin;
+	case 0: return NULL;
 	case '@': return pix_op;
 	case '%': return pix_hop;
 	case '+': return pix_voice;
 	}
-	return 0;
+
+	/* find out how many levels above Op this user is */
+	pre = strchr (serv->nick_prefixes, '@');
+	if (pre && pre != serv->nick_prefixes)
+	{
+		pre--;
+		level = 0;
+		while (1)
+		{
+			if (pre[0] == user->prefix[0])
+			{
+				switch (level)
+				{
+				case 0: return pix_red;	/* 1 level above op */
+				case 1: return pix_purple;	 /* 2 levels above op */
+				}
+				break;	/* 3+, no icons */
+			}
+			level++;
+			if (pre == serv->nick_prefixes)
+				break;
+			pre--;
+		}
+	}
+
+	return NULL;
 }
 
 void
@@ -223,7 +255,7 @@ fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 		gtk_list_store_insert (GTK_LIST_STORE (model), &iter, row);
 	}
 
-	pix = get_prefix_pixbuf (newuser->prefix[0]);
+	pix = get_user_icon (sess->server, newuser);
 
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 							  0, pix,
@@ -235,8 +267,9 @@ fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 	/* is it me? */
 	if (!rfc_casecmp (newuser->nick, sess->server->nick) && sess->gui->nick_box)
 	{
-		sess->res->prefix = newuser->prefix[0];
-		mg_set_access_icon (sess->gui, pix);
+		sess->res->myself = newuser;
+		if (!sess->gui->is_tab || sess == current_tab)
+			mg_set_access_icon (sess->gui, pix);
 	}
 
 #if 0
@@ -269,6 +302,7 @@ void
 fe_userlist_clear (session *sess)
 {
 	gtk_list_store_clear (sess->res->user_model);
+	sess->res->myself = NULL;
 }
 
 static void
@@ -299,7 +333,7 @@ userlist_dnd_drop (GtkTreeView *widget, GdkDragContext *context,
 		{
 			if (next)
 				*next = 0;
-			dcc_send (current_sess, tbuf, user->nick, p + 5);
+			dcc_send (current_sess, tbuf, user->nick, p + 5, prefs.dcc_max_send_cps);
 		}
 		if (!next)
 			break;
@@ -372,69 +406,61 @@ userlist_add_columns (GtkTreeView * treeview)
 	}
 }
 
-static int last_button = 0;
-
-static void
-userlist_cursor_cb (GtkTreeView *treeview, gpointer user_data)
-{
-	session *sess = current_sess;
-	char **nicks;
-	int i;
-
-	if (last_button == 100)
-	{
-		if (prefs.doubleclickuser[0])
-		{
-			nicks = userlist_selection_list (sess->gui->user_tree, &i);
-			if (nicks)
-			{
-				nick_command_parse (sess, prefs.doubleclickuser, nicks[0],
-										  nicks[0]);
-				free (nicks);
-			}
-		}
-		return;
-	}
-
-	if (last_button == 3)
-	{
-		nicks = userlist_selection_list (sess->gui->user_tree, &i);
-		if (nicks)
-		{
-			menu_nickmenu (sess, NULL, nicks[0], i);
-			free (nicks);
-		}
-	}
-}
-
 static gint
 userlist_click_cb (GtkWidget *widget, GdkEventButton *event, gpointer userdata)
 {
-	session *sess = current_sess;
 	char **nicks;
 	int i;
+	GtkTreeSelection *sel;
+	GtkTreePath *path;
 
 	if (!event)
 		return FALSE;
 
-	last_button = event->button;
-
-	if (event->type == GDK_2BUTTON_PRESS)
-		last_button = 100;
+	if (event->type == GDK_2BUTTON_PRESS && prefs.doubleclickuser[0])
+	{
+		nicks = userlist_selection_list (widget, &i);
+		if (nicks)
+		{
+			nick_command_parse (current_sess, prefs.doubleclickuser, nicks[0],
+									  nicks[0]);
+			free (nicks);
+		}
+		return TRUE;
+	}
 
 	if (event->button == 3)
 	{
-		nicks = userlist_selection_list (sess->gui->user_tree, &i);
-		if (nicks)
+		/* do we have a multi-selection? */
+		nicks = userlist_selection_list (widget, &i);
+		if (nicks && i > 1)
 		{
-			if (i > 1)
-				menu_nickmenu (sess, event, nicks[0], i);
+			menu_nickmenu (current_sess, event, nicks[0], i);
 			free (nicks);
+			return TRUE;
+		}
+		if (nicks)
+			free (nicks);
+
+		sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
+			 event->x, event->y, &path, 0, 0, 0))
+		{
+			gtk_tree_selection_unselect_all (sel);
+			gtk_tree_selection_select_path (sel, path);
+			gtk_tree_path_free (path);
+			nicks = userlist_selection_list (widget, &i);
+			if (nicks)
+			{
+				menu_nickmenu (current_sess, event, nicks[0], i);
+				free (nicks);
+			}
+		} else
+		{
+			gtk_tree_selection_unselect_all (sel);
 		}
 
-		/* don't let it unselect a multiselection */
-		if (i > 1)
-			return TRUE;
+		return TRUE;
 	}
 
 	return FALSE;
@@ -444,7 +470,7 @@ GtkWidget *
 userlist_create (GtkWidget *box)
 {
 	GtkWidget *sw, *treeview;
-	static GtkTargetEntry dnd_targets[] =
+	static const GtkTargetEntry dnd_targets[] =
 	{
 		{"text/uri-list", 0, 1}
 	};
@@ -458,6 +484,7 @@ userlist_create (GtkWidget *box)
 												GTK_POLICY_NEVER,
 											  GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start (GTK_BOX (box), sw, TRUE, TRUE, 0);
+	gtk_widget_show (sw);
 
 	treeview = gtk_tree_view_new ();
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
@@ -477,12 +504,11 @@ userlist_create (GtkWidget *box)
 							G_CALLBACK (userlist_dnd_drop), 0);
 	g_signal_connect (G_OBJECT (treeview), "button_press_event",
 							G_CALLBACK (userlist_click_cb), 0);
-	g_signal_connect (G_OBJECT (treeview), "cursor_changed",
-							G_CALLBACK (userlist_cursor_cb), 0);
 
 	userlist_add_columns (GTK_TREE_VIEW (treeview));
 
 	gtk_container_add (GTK_CONTAINER (sw), treeview);
+	gtk_widget_show (treeview);
 
 	return treeview;
 }
