@@ -13,10 +13,6 @@
 #define GTK_IS_XTEXT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_XTEXT))
 #define GTK_XTEXT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_XTEXT, GtkXTextClass))
 
-#define FONT_1BYTE 0
-#define FONT_2BYTE 1
-#define FONT_SET 2
-
 #define ATTR_BOLD '\002'
 #define ATTR_COLOR '\003'
 #define ATTR_BEEP '\007'
@@ -41,6 +37,8 @@ typedef struct {
 	int last_offset_start;
 	int last_offset_end;
 
+	int last_pixel_pos;
+
 	int pagetop_line;
 	int pagetop_subline;
 	textentry *pagetop_ent;			/* what's at xtext->adj->value */
@@ -53,6 +51,7 @@ typedef struct {
 
 	unsigned int time_stamp:1;
 	unsigned int scrollbar_down:1;
+	unsigned int needs_recalc:1;
 
 } xtext_buffer;
 
@@ -111,17 +110,15 @@ struct _GtkXText
 	int hilight_start;
 	int hilight_end;
 
+	guint16 fontwidth[128];	  /* each char's width, only the ASCII ones */
+
 #ifdef USE_XFT
 	XftColor color[20];
 	XftColor *xft_fg;
 	XftColor *xft_bg;				/* both point into color[20] */
-	guint16 fontwidth[128];	  /* each char's width, only the ASCII ones */
 	XftDraw *xftdraw;
 	XftFont *font;
-#endif
-
-#ifdef USE_PANGO
-	guint16 fontwidth[128];	  /* each char's width, only the ASCII ones */
+#else
 	struct
 	{
 		PangoFontDescription *font;
@@ -131,12 +128,6 @@ struct _GtkXText
 	PangoLayout *layout;
 #endif
 
-#if !defined(USE_PANGO) && !defined(USE_XFT)
-	guint16 fontwidth[256];	  /* each char's width, only for FONT_1BYTE type */
-	GdkFont *font;
-#endif
-
-	int fonttype;				/* FONT_* (unused for xft) */
 	int fontsize;
 	int space_width;				  /* width (pixels) of the space " " character */
 	int stamp_width;				  /* width of "[88:88:88]" */
@@ -161,7 +152,6 @@ struct _GtkXText
 	unsigned int button_down:1;
 	unsigned int bold:1;
 	unsigned int underline:1;
-	unsigned int reverse:1;
 	unsigned int transparent:1;
 	unsigned int separator:1;
 	unsigned int shaded:1;
@@ -176,6 +166,7 @@ struct _GtkXText
 	unsigned int in_hilight:1;
 	unsigned int un_hilight:1;
 	unsigned int recycle:1;
+	unsigned int avoid_trans:1;
 };
 
 struct _GtkXTextClass
@@ -200,9 +191,8 @@ void *gtk_xtext_search (GtkXText * xtext, const unsigned char *text, void *start
 
 gboolean gtk_xtext_is_empty (GtkXText * xtext);
 typedef void (*GtkXTextForeach) (GtkXText *xtext, unsigned char *text, void *data);
-void gtk_xtext_foreach (GtkXText * xtext, GtkXTextForeach func, void *data);
+void gtk_xtext_foreach (xtext_buffer *buf, GtkXTextForeach func, void *data);
 
-void gtk_xtext_set_color_paste (GtkXText *xtext, gboolean  color_paste);
 void gtk_xtext_set_error_function (GtkXText *xtext, GtkWidget *(*error_function) (char *));
 void gtk_xtext_set_indent (GtkXText *xtext, gboolean indent);
 void gtk_xtext_set_max_indent (GtkXText *xtext, int max_auto_indent);
@@ -216,6 +206,6 @@ void gtk_xtext_set_wordwrap (GtkXText *xtext, gboolean word_wrap);
 
 xtext_buffer *gtk_xtext_buffer_new (GtkXText *xtext);
 void gtk_xtext_buffer_free (xtext_buffer *buf);
-void gtk_xtext_buffer_show (GtkXText *xtext, xtext_buffer *buf);
+void gtk_xtext_buffer_show (GtkXText *xtext, xtext_buffer *buf, int render);
 
 #endif
